;***************************************;***************************************
;                               		
;    Filename:							LightHouseBeacon                               
;    Date:                        		11/4/2010             
;    File Version:                    	0.1  
;                                         
;    Author:                          	jeff bachiochi    
;    Company:                           imagine that
;										www.imaginethatnow.com  
;                                                        
;***************************************;***************************************
;                                                         
;    Files required: P18F2431.INC  
;
;	NOTES	This is modular code.  It requires less room but executes slower 
;		USe of indirect registers makes it difficult to understand                            
;                                                      
;***************************************;***************************************
;
	LIST P=18F2431, F=INHX32 ;directive to define processor
	#include <P18F2431.INC>	 ;processor specific variable definitions
;
;***************************************;***************************************
;
;	Configuration bits
;                                                        
;***************************************;***************************************
;
;   Oscillator Selection:
;     OSC = LP             LP
;     OSC = XT             XT
;     OSC = HS             HS
;     OSC = RC2            External RC, RA6 is CLKOUT
;     OSC = EC             EC, RA6 is CLKOUT
;     OSC = ECIO           EC, RA6 is I/O
;     OSC = HSPLL          HS-PLL Enabled
;     OSC = RCIO           External RC, RA6 is I/O
	CONFIG	OSC = IRCIO          		; Internal RC, RA6 & RA7 are I/O
;     OSC = IRC            Internal RC, RA6 is CLKOUT, RA7 is I/O
;     OSC = RC1            External RC, RA6 is CLKOUT
;     OSC = RC             External RC, RA6 is CLKOUT
;
;   Fail-Safe Clock Monitor Enable:
;     FCMEN = OFF          Disabled
;     FCMEN = ON           Enabled
;
;   Internal/External Switch-Over:
;     IESO = OFF           Disabled
;     IESO = ON            Enabled
;
;   Power-up Timer:
;     PWRTEN = ON          Enabled
;     PWRTEN = OFF         Disabled
;
;   Brown-out Reset:
;     BOREN = OFF          Disabled
;     BOREN = ON           Enabled
;
;   Brown-out Voltage:
;     BORV = 45            4.5V
;     BORV = 42            4.2V
;     BORV = 27            2.7V
;     BORV = 20            2.0V
;
;   Watchdog Timer:
	CONFIG     WDTEN = OFF          	; Disabled
;     WDTEN = ON           Enabled
;
;   Watchdog Timer Enable Window:
;     WINEN = ON           Enabled
;     WINEN = OFF          Disabled
;
;   Watchdog Postscaler:
;     WDPS = 1             1:1
;     WDPS = 2             1:2
;     WDPS = 4             1:4
;     WDPS = 8             1:8
;     WDPS = 16            1:16
;     WDPS = 32            1:32
;     WDPS = 64            1:64
;     WDPS = 128           1:128
;     WDPS = 256           1:256
;     WDPS = 512           1:512
;     WDPS = 1024          1:1024
;     WDPS = 2048          1:2048
;     WDPS = 4096          1:4096
;     WDPS = 8192          1:8192
;     WDPS = 16384         1:16384
;     WDPS = 32768         1:32768
;
;   Timer1 Oscillator MUX:
;     T1OSCMX = OFF        Active
;     T1OSCMX = ON         Inactive
;
;   High-Side Transistors Polarity:
;     HPOL = LOW           Active low
;     HPOL = HIGH          Active high
;
;   Low-Side Transistors Polarity:
;     LPOL = LOW           Active low
;     LPOL = HIGH          Active high
;
;   PWM output pins Reset state control:
;     PWMPIN = ON          Enabled
;     PWMPIN = OFF         Disabled
;
;   MCLR Enable:
;     MCLRE = OFF          Disabled
;     MCLRE = ON           Enabled
;
;   Stack Overflow Reset:
;     STVREN = OFF         Disabled
;     STVREN = ON          Enabled
;
;   Low Voltage Programming:
;     LVP = OFF            Disabled
;     LVP = ON             Enabled
;
;   Background Debugger Enable:
;     DEBUG = ON           Enabled
;     DEBUG = OFF          Disabled
;
;   Code Protection Block 0:
;     CP0 = ON             Enabled
;     CP0 = OFF            Disabled
;
;   Code Protection Block 1:
;     CP1 = ON             Enabled
;     CP1 = OFF            Disabled
;
;   Code Protection Block 2:
;     CP2 = ON             Enabled
;     CP2 = OFF            Disabled
;
;   Code Protection Block 3:
;     CP3 = ON             Enabled
;     CP3 = OFF            Disabled
;
;   Boot Block Code Protection:
;     CPB = ON             Enabled
;     CPB = OFF            Disabled
;
;   Data EEPROM Code Protection:
;     CPD = ON             Enabled
;     CPD = OFF            Disabled
;
;   Write Protection Block 0:
;     WRT0 = ON            Enabled
;     WRT0 = OFF           Disabled
;
;   Write Protection Block 1:
;     WRT1 = ON            Enabled
;     WRT1 = OFF           Disabled
;
;   Boot Block Write Protection:
;     WRTB = ON            Enabled
;     WRTB = OFF           Disabled
;
;   Configuration Register Write Protection:
;     WRTC = ON            Enabled
;     WRTC = OFF           Disabled
;
;   Data EEPROM Write Protection:
;     WRTD = ON            Enabled
;     WRTD = OFF           Disabled
;
;   Table Read Protection Block 0:
;     EBTR0 = ON           Enabled
;     EBTR0 = OFF          Disabled
;
;   Table Read Protection Block 1:
;     EBTR1 = ON           Enabled
;     EBTR1 = OFF          Disabled
;
;   Boot Block Table Read Protection:
;     EBTRB = ON           Enabled
;     EBTRB = OFF          Disabled
;
;***************************************;***************************************
;
;	Constant definitions
;                                                        
;***************************************;***************************************
;
;	PORTA
LED5	equ		1<<0
LED5P	equ		LATA
LED6	equ		1<<1
LED6P	equ		LATA
LED7	equ		1<<2
LED7P	equ		LATA
LED8	equ		1<<3
LED8P	equ		LATA
;		equ		4
;		equ		5
;		equ		6
;		equ		7
;
;	PORTB
LED1	equ		1<<0
LED1P	equ		LATB
LED2	equ		1<<1
LED2P	equ		LATB
LED3	equ		1<<2
LED3P	equ		LATB
LED4	equ		1<<3
LED4P	equ		LATB
;		equ		4
;		equ		5
;PGD	equ		6
;PGC	equ		7
;
;	PORTC
LED9	equ		1<<0
LED9P	equ		LATC
LED10	equ		1<<1
LED10P	equ		LATC
LED11	equ		1<<2
LED11P	equ		LATC
LED12	equ		1<<3
LED12P	equ		LATC
LED13	equ		1<<4
LED13P	equ		LATC
LED14	equ		1<<5
LED14P	equ		LATC
LED15	equ		1<<6
LED15P	equ		LATC
LED16	equ		1<<7
LED16P	equ		LATC
;
LEDCOUNT	equ	D'16'
PWMMAX	equ		255
STEPS	equ		0x10	
;
;***************************************;***************************************
;
;	Variable definitions
;                                                        
;***************************************;***************************************
;
		cblock	0x000
WREG_TEMP								;variable in RAM for context saving 
STATUS_TEMP								;variable in RAM for context saving
BSR_TEMP								;variable in RAM for context saving
;
PWM1:LEDCOUNT
BIT1V:LEDCOUNT
PORT1V:LEDCOUNT
POSITION1:LEDCOUNT
FLAGS:LEDCOUNT/4
PPPA
PPPABM
PPV
PPPAV
COUNT
PHASE
TEMP
CYCLE
		endc								
;                                                        
;***************************************;***************************************
;
;	EEPROM data
;                                                        
;***************************************;***************************************
;
;		DE	"Test Data",0,1,2,3,4,5
;                                                        
;***************************************;***************************************
;
;	Reset vector
;                                                        
;***************************************;***************************************
;
	
RESET_VECTOR:		ORG	0x0000

		goto	Init					; go to start of initialization code
;                                                        
;***************************************;***************************************
;
;	High priority interrupt vector
;                                                        
;***************************************;***************************************
;
HI_INT_VECTOR:		ORG	0x0008

		bra	HighInt		;go to high priority interrupt routine
;                                                        
;***************************************;***************************************
;
;	Low priority interrupt vector and routine
;                                                        
;***************************************;***************************************
;
LOW_INT_VECTOR:		ORG	0x0018

		bra	LowInt		;go to low priority interrupt routine
;                                                        
;***************************************;***************************************
;
;	High priority interrupt routine
;                                                        
;***************************************;***************************************
;
HighInt:
		
;	*** high priority interrupt code goes here ***


		retfie	FAST
;                                                        
;***************************************;***************************************
;
;	Low priority interrupt routine
; The low priority interrupt code is placed here.
;                                                        
;***************************************;***************************************
;
LowInt:
		movff	STATUS,STATUS_TEMP	;save STATUS register
		movff	WREG,WREG_TEMP		;save working register
		movff	BSR,BSR_TEMP		;save BSR register

;	*** low priority interrupt code goes here ***


		movff	BSR_TEMP,BSR		;restore BSR register
		movff	WREG_TEMP,WREG		;restore working register
		movff	STATUS_TEMP,STATUS	;restore STATUS register
		retfie
;                                                        
;***************************************;***************************************
;
Init:
Init_OSCCON
		movlw	0x70
		movwf	OSCCON
;		
Init_AD:
		clrf	ANSEL0
;		clrf	ANSEL1
;		
Init_PORTA:
		movlw	b'11110000'				; x, x, x, x, LED8:5=L
		movwf	PORTA
		movlw	b'11110000'				; x, x, x, x, LED8:5=O
		movwf	TRISA		
;
Init_PORTB:
		movlw	b'11110000'				; x, x, x, x, LED4:1=L
		movwf	PORTB
		movlw	b'11110000'				; x, x, x, x, LED4:1=O
		movwf	TRISB
;
Init_PORTC:
		movlw	b'00000000'				; x, x, x, x, LED16:9=L
		movwf	PORTC
		movlw	b'00000000'				; x, x, x, x, LED16:9=O
		movwf	TRISC
;
Init_TMR2:
		movlw	b'01111111'				; x, post=/16, TMR2=EN, pre=/16
		movwf	T2CON
		movlw	D'255'
		movwf	PR2
;
Init_Variables:
		lfsr	0,	0x000
;
Bank0_Loop:		
		clrf	POSTINC0
		tstfsz	FSR0L
		goto	Bank0_Loop	
;
		lfsr	0,	BIT1V
;
		movlw	LED1
		movwf	POSTINC0
		movlw	LED2
		movwf	POSTINC0
		movlw	LED3
		movwf	POSTINC0
		movlw	LED4
		movwf	POSTINC0
		movlw	LED5
		movwf	POSTINC0
		movlw	LED6
		movwf	POSTINC0
		movlw	LED7
		movwf	POSTINC0
		movlw	LED8
		movwf	POSTINC0
		movlw	LED9
		movwf	POSTINC0
		movlw	LED10
		movwf	POSTINC0
		movlw	LED11
		movwf	POSTINC0
		movlw	LED12
		movwf	POSTINC0
		movlw	LED13
		movwf	POSTINC0
		movlw	LED14
		movwf	POSTINC0
		movlw	LED15
		movwf	POSTINC0
		movlw	LED16
		movwf	POSTINC0
;
		lfsr	0,	PORT1V
;
		movlw	LED1P
		movwf	POSTINC0
		movlw	LED2P
		movwf	POSTINC0
		movlw	LED3P
		movwf	POSTINC0
		movlw	LED4P
		movwf	POSTINC0
		movlw	LED5P
		movwf	POSTINC0
		movlw	LED6P
		movwf	POSTINC0
		movlw	LED7P
		movwf	POSTINC0
		movlw	LED8P
		movwf	POSTINC0
		movlw	LED9P
		movwf	POSTINC0
		movlw	LED10P
		movwf	POSTINC0
		movlw	LED11P
		movwf	POSTINC0
		movlw	LED12P
		movwf	POSTINC0
		movlw	LED13P
		movwf	POSTINC0
		movlw	LED14P
		movwf	POSTINC0
		movlw	LED15P
		movwf	POSTINC0
		movlw	LED16P
		movwf	POSTINC0
;
		lfsr	0,	POSITION1
;
		movlw	0x01
		movwf	POSTINC0
		movlw	0x02
		movwf	POSTINC0
		movlw	0x04
		movwf	POSTINC0
		movlw	0x08
		movwf	POSTINC0
		movlw	0x10
		movwf	POSTINC0
		movlw	0x20
		movwf	POSTINC0
		movlw	0x40
		movwf	POSTINC0
		movlw	0x80
		movwf	POSTINC0
		movlw	0x01
		movwf	POSTINC0
		movlw	0x02
		movwf	POSTINC0
		movlw	0x04
		movwf	POSTINC0
		movlw	0x08
		movwf	POSTINC0
		movlw	0x10
		movwf	POSTINC0
		movlw	0x20
		movwf	POSTINC0
		movlw	0x40
		movwf	POSTINC0
		movlw	0x80
		movwf	POSTINC0																					
;
		movlw	0x10
		movwf	PHASE
		movwf	CYCLE		
;
		movlw	b'00000001'
		movwf	FLAGS
		movlw	b'00000000'
		movwf	FLAGS+1		
		goto	Main		
;                                                        
;***************************************;***************************************
;
Main:
;                                                        
;***************************************
;
Main_PWM:
		call	DoPWM
		decfsz	PHASE,	F				; skip next when PHASE=0 (every STEP times)
		goto	Main_Exit	
;                                                        
;***************************************
;
		movlw	STEPS
		movwf	PHASE
		btfss	PIR1,	TMR2IF			; skip next if Timer 0 has overflowed
		goto	Main_Exit
;
		bcf		PIR1,	TMR2IF			; clear timer 0 interrupt		
		call	Ramp
		decfsz	CYCLE,	F				; skip next when CYCLE=0 (every STEP times)
		goto	Main_Exit
;                                                        
;***************************************
;
		movlw	STEPS
		movwf	CYCLE				
		rlcf	FLAGS+1,	W
		rlcf	FLAGS,	F
		rlcf	FLAGS+1,	F
;                                                        
;***************************************
;
Main_Exit:	
		goto	Main	
;                                                        
;***************************************;***************************************
;
;	DoPWM
;                                                        
;***************************************;***************************************
;
DoPWM:
		movlw	LEDCOUNT
		movwf	TEMP
		lfsr	0,	PORT1V	
;
DoPWM_Loop:
		decf	TEMP,	W
		movff	PLUSW0,	PPPA
		movlw	0x0F
		movwf	FSR2H					; port is an SFR
		movff	PPPA,	FSR2L			; port's address as pointer
;
		decf	TEMP,	W
		addlw	BIT1V-PORT1V
		movff	PLUSW0,	PPPABM		

;
		lfsr	1,	PWM1							
		decf	TEMP,	W
		addwf	FSR1L,	F
		movf	PHASE,	W				
		cpfsgt	INDF1					; skip next if PWM's setting>PHASE
		goto	DoPWM_Off
;
DoPWM_On:
		movf	PPPABM,	W				; get port's bit mask
		iorwf	INDF2,	F				; turn on PWM's bit in port's value
		goto	DoPWM_Continue
;
DoPWM_Off:
		comf	PPPABM,	W				; get complement of port's bit mask
		andwf	INDF2,	F				; turn off PWM's bit in port's value
;
DoPWM_Continue:
		decfsz	TEMP,	F
		goto	DoPWM_Loop
;
DoPWM_Exit:
		return
;                                                        
;***************************************;***************************************
;
;	Ramp
;
;	COUNT	=	(1 to LEDCOUNT)
;                                                        
;***************************************;***************************************
;
Ramp:	
		movlw	LEDCOUNT
		movwf	TEMP
		lfsr	0,	FLAGS
							
;
Ramp_Loop:
		decf	TEMP,	W
		lfsr	1,	PWM1
		addwf	FSR1L,	F
		lfsr	2,	POSITION1	
		addwf	FSR2L,	F
		bcf		STATUS,	C
		rrcf	WREG,	W
		bcf		STATUS,	C
		rrcf	WREG,	W
		bcf		STATUS,	C
		rrcf	WREG,	W				; offset into FLAGS Array (0 ro 1)		
		movf	PLUSW0,	W				; get value of FLAGS offset
		andwf	INDF2,	W				; and with position bit mask
		tstfsz	WREG					; skip next if direction=0 (FLAGSx.position)
		goto	Rampup					; else direction=1, increment PWM's value
		goto	Rampdown				; decrement PWM's value				
;
Rampup:
		incf	INDF1,	W				; increment the PWM's value
		tstfsz	WREG					; skip next if the count is zero (overflow)
		goto	Rampup1
;
		setf	WREG					; set the count to max
;
Rampup1:
		goto	Ramp_Continue
;                                                        
Rampdown:
		movf	INDF1,	W				; get the PWM's count
		tstfsz	WREG					; skip next if its already minimum
		decf	WREG,	W				; else reduce the PWM's count
;
Ramp_Continue:
		movff	WREG,	POSTDEC1		; save it (and decrement to next value)
		decfsz	TEMP,	F
		goto	Ramp_Loop
;
Ramp_Exit:	
		return						
;                                                        
;***************************************;***************************************
;
;	End of program
;                                                        
;***************************************;***************************************
;
		END
