/* ----------------------------------------------------------
		
		Psychedelic led lights with dsPic 30F2020 
		Marco Signorini - marco.signorini@libero.it
		
		File: hardware.h
	-----------------------------------------------------------
*/

#ifndef _HARDWARE_H_
#define _HARDWARE_H_

/* Include proper dspic header file */
#include <p30fxxxx.h>

/* ------------------ Possible symbols definitions are: ----------------
	_ICD_ENABLED_ : to be defined to compile this firmware for ICD
	_SIMULATE_    : to be defined to compile this firmware for simulator
	_RS232DEBUG_  : to enable all RS232 coefficient dump subsystem
	_SOFT_COMPRESSION_ : to select a soft light amplitude compression
   --------------------------------------------------------------------- */

/* define here the number of samples in output buffers */
#ifdef _ICD_ENABLED_
	#define SMPBUFFER			16
	#define POWERSHIFTVAL		4
#else
	#define SMPBUFFER			32
	#define POWERSHIFTVAL		5
#endif

/* input buffer is implemented as double buffer: the
	dsp engine is able to evaluate a block when the ADC is
	fitting the next block (and viceversa) */
#define SMPINBUFFER		2*SMPBUFFER

/* External connections and relative macros */
#define PGA_CS_TRIS(a) 	TRISAbits.TRISA9 = (a)
#define SET_PGA_CS			LATAbits.LATA9 = 1;
#define RES_PGA_CS			LATAbits.LATA9 = 0;

/* Other usefull macros */
#define CLAMP_UINT_TO_UCHAR(a) ((a) < ((unsigned int)0x00FF))? ((unsigned char)(a)) : ((unsigned char)0xFF)

/* Debug macros */
#ifdef _RS232DEBUG_
	#include "rs232.h"
	#define DBG_INIT						unsigned char ucDbgPrescaler = 0; RS232_Init()
	#define DBG_START						ucDbgPrescaler++; ucDbgPrescaler &= 0x1F; if (ucDbgPrescaler == 0) RS232_TxChar('#');
	#define DBG_END							if (ucDbgPrescaler == 0) RS232_TxChar('@');
	#define DBG_SENDCHAR(a)			if (ucDbgPrescaler == 0) RS232_TxChar((a));
	#define DBG_SENDVAL(a,sz)		if (ucDbgPrescaler == 0) RS232_TxValue((a),(sz));
	#define TXBUFFLENGTH				64
#else
	#define DBG_INIT
	#define DBG_START
	#define DBG_END
	#define DBG_SENDCHAR(a)
	#define DBG_SENDVAL(a,sz)
	#define TXBUFFLENGTH				1
#endif /* _RS232DEBUG_*/


#endif /* _HARDWARE_H_ */
