/* ----------------------------------------------------------
		
		Psychedelic led lights with dsPic 30F2020 
		Marco Signorini - marco.signorini@libero.it
		
		File: adc.c
	-----------------------------------------------------------
*/

#include "hardware.h"
#include "adc.h"
#include "inbuffer.h"

/* Initialize the ADC subsystem */
/* The ADC subsystem will use TMR1 to define the sampling period */
void ADC_Init(void)
{
	PR1 = 0x2E2;									/* FNy=20KHz -> Fsmp = 40KHz -> 25us */	
	T1CONbits.TON = 1;						/* Start TMR1 */
	
	ADCONbits.ADCS = 3; 					/* 13.3 MHz @ 30 MIPS for the ADC */
	ADCONbits.FORM = 1; 					/* Output in fractional format */

	ADPCFG = 0xFFFF;							/* All pins are digital in/out */
	ADPCFGbits.PCFG6 = 0;					/* except for AN6 that's the analog input */
	TRISBbits.TRISB6 = 1;					
	
	ADCPC1bits.TRGSRC3 = 0x0C;		/* TMR1 is defining the sampling period */
	ADCPC1bits.IRQEN3 = 1;				/* Enable interrupt on AN7 and AN6 conversion completed */
	
	IFS0bits.ADIF = 0;						/* Clear all pending AD interrupts... */
	IEC0bits.ADIE = 1;						/* ... and enable the AD interrupt request */
	
	ADCONbits.ADON = 1; 					/* Enable the ADC subsystem */
}

/* ----------------------------------------------------------- */

/* A/D interrupt service routine (ISR) */
void __attribute__((__interrupt__)) _ADCInterrupt(void)
{
	/* Clear the pending interrupt */
	IFS0bits.ADIF = 0;
	ADSTATbits.P3RDY = 0;
	
	/* Store current sample */
	INBUF_AddSample(ADCBUF6);
}

/* ----------------------------------------------------------- */
