/* ----------------------------------------------------------
		
		Psychedelic led lights with dsPic 30F2020 
		Marco Signorini - marco.signorini@libero.it
		
		File: filterbank.c
	-----------------------------------------------------------
*/

#include "hardware.h"
#include "filterbank.h"
#include "inbuffer.h"

/* Here is the definition of buffers */
fractional gaLocalBuffer[SMPBUFFER] __attribute__((space(xmemory)));

/* Here are the filters coefficients references */
extern IIRTransposedStruct iirhipassdcFilter;
extern IIRTransposedStruct iirlowpassFilter;
extern IIRTransposedStruct iirbandpassFilter;
extern IIRTransposedStruct iirhipassFilter;

/* ----------------------------------------------------------- */

void FLTBNK_Init(void)
{
	/* Initialize the filter bank */
	IIRTransposedInit(&iirhipassdcFilter);
	IIRTransposedInit(&iirlowpassFilter);
	IIRTransposedInit(&iirbandpassFilter);
	IIRTransposedInit(&iirhipassFilter);
}

/* ----------------------------------------------------------- */

fractional* FLTBNK_FilterDC(void)
{
	/* Filter incoming data */
	IIRTransposed(SMPBUFFER, &gaLocalBuffer[0], INBUF_GetBuffer(), &iirhipassdcFilter);

	/* Unbiased data are now stored in local buffer */
	return &gaLocalBuffer[0];
}

/* ----------------------------------------------------------- */

fractional* FLTBNK_FilterLow(void)
{
	/* Filter incoming data: unbiased data (input to be filtered out) are in gaLocalBuffer; */
	/* we use the original (biased) input buffer as temporary output buffer */
	IIRTransposed(SMPBUFFER, INBUF_GetBuffer(), &gaLocalBuffer[0], &iirlowpassFilter);
	
	return INBUF_GetBuffer();
}

/* ----------------------------------------------------------- */

fractional* FLTBNK_FilterMid(void)
{
	/* Filter incoming data: unbiased data (input to be filtered out) are in gaLocalBuffer; */
	/* we use the original (biased) input buffer as temporary output buffer */
	IIRTransposed(SMPBUFFER, INBUF_GetBuffer(), &gaLocalBuffer[0], &iirbandpassFilter);
	
	return INBUF_GetBuffer();
}

/* ----------------------------------------------------------- */

fractional* FLTBNK_FilterHi(void)
{
	/* Filter incoming data: unbiased data (input to be filtered out) are in gaLocalBuffer; */
	/* we use the original (biased) input buffer as temporary output buffer */
	IIRTransposed(SMPBUFFER, INBUF_GetBuffer(), &gaLocalBuffer[0], &iirhipassFilter);
	
	return INBUF_GetBuffer();
}
