/* ----------------------------------------------------------
		
		Psychedelic led lights with dsPic 30F2020 
		Marco Signorini - marco.signorini@libero.it
		
		File: inbuffer.c
	-----------------------------------------------------------
*/

#include "inbuffer.h"
#include "hardware.h"

/* Internal useful macros */
#ifdef _SIMULATE_
	#define HANDLEOFFSET(a)
#else
	#define HANDLEOFFSET(a) (a) += 0x8000
#endif

/* Here is defined the input buffer and head/tail pointers */
fractional gaInBuffer[SMPINBUFFER] __attribute__((space(xmemory)));
fractional *gpHead;
fractional *gpTail;

/* ----------------------------------------------------------- */

void INBUF_Init(void)
{
	gpHead = gaInBuffer;
	gpTail = gaInBuffer;
}

/* ----------------------------------------------------------- */

void INBUF_AddSample(unsigned int uiSample)
{
	HANDLEOFFSET(uiSample);

	*gpHead++ = uiSample;
	if (gpHead == (gaInBuffer + SMPINBUFFER))
		gpHead = gaInBuffer;
}

/* ----------------------------------------------------------- */

unsigned char INBUF_GetFilledSize(void)
{
	/* Return the currently filled buffer */
	return (unsigned char) (gpHead - gpTail);
}

/* ----------------------------------------------------------- */

fractional* INBUF_GetBuffer(void)
{
	/* Return the tail of the buffer 
		(samples not already evaluated) */
	return gpTail;
}

/* ----------------------------------------------------------- */

void INBUF_Release(void)
{
	/* Skip to next buffer block */
	if (gpTail == gaInBuffer)
		gpTail += SMPBUFFER;
	else
		gpTail = gaInBuffer;
}
