/* ----------------------------------------------------------
		
		Psychedelic led lights with dsPic 30F2020 
		Marco Signorini - marco.signorini@libero.it
		
		File: mcp6s26.c
	-----------------------------------------------------------
*/

#include "hardware.h"
#include "mcp6s26.h"

/* Local variables */
static unsigned char gucPGAGain;

/* Useful definitions */
#define PGAMAXGAIN 0x06
#define PGAMINGAIN 0x00

#define PGA_SHUTDOWNCMD			0x20
#define PGA_SETGAINCMD			0x40
#define PGA_SETCHANNELCMD		0x41

#ifdef _SIMULATE_
	#define PGA_TX_DATA(cmd,val,dummy)
#else
	#define PGA_TX_DATA(cmd,val,dummy)\
		{\
			RES_PGA_CS;\
			SPI1BUF = (unsigned int) ((((unsigned int)(cmd))<<8) | (val));\
			while (!SPI1STATbits.SPIRBF)\
			{\
				ClrWdt();\
			}\
			(dummy) = SPI1BUF; /* Dummy read to clear the SPIRBF flag */\
			SET_PGA_CS;\
		}
#endif

/* ----------------------------------------------------------- */

/* To comunicate with the external PGA, this module
	 will use the SPI subsystem and one I/O pin */
void PGA_Init(void)
{
	unsigned char ucDummy;

	/* Set I/O pin as chip select */
	PGA_CS_TRIS(0);
	SET_PGA_CS;
	
	/* Initialize the SPI subsystem */
	SPI1CON1bits.MODE16 = 1;/* Communications are 16 wide */
	SPI1CON1bits.MSTEN = 1; /* I'm the master in SPI comms */
	SPI1CON1bits.CKE = 1; 	/* Transition on clock falling edge */
	SPI1CON1bits.PPRE = 2;	/* Primary prescaler set to 4 */
	SPI1CON1bits.SPRE = 4;	/* Secondary prescaler set to 4 */
	SPI1STATbits.SPIROV = 0;/* Clear any previous overflows */
	SPI1STATbits.SPIEN = 1; /* Enable the SPI subsystem */
	
	/* Initialize local variables */
	gucPGAGain = 0;

	/* Initialize the PGA */
	PGA_TX_DATA(PGA_SETCHANNELCMD, 0, ucDummy);
	PGA_TX_DATA(PGA_SETGAINCMD, gucPGAGain & 0x07, ucDummy);
}

/* ----------------------------------------------------------- */

void PGA_IncGain(void)
{
	unsigned char ucDummy;

	/* No more gain available */
	if (gucPGAGain == PGAMAXGAIN)
		return;
		
	/* Increase current gain */
	gucPGAGain++;
	if (gucPGAGain > PGAMAXGAIN)
		gucPGAGain = PGAMAXGAIN;
		
	/* Send to the PGA */
	PGA_TX_DATA(PGA_SETGAINCMD, gucPGAGain & 0x07, ucDummy);
}

/* ----------------------------------------------------------- */

void PGA_DecGain(void)
{
	unsigned char ucDummy;

	/* No more gain available */
	if (gucPGAGain == PGAMINGAIN)
		return;
	
	/* Decrease current gain */
	gucPGAGain--;
	if (gucPGAGain == 0xFF)
		gucPGAGain = 0;
		
	/* Send to the PGA */
	PGA_TX_DATA(PGA_SETGAINCMD, gucPGAGain & 0x07, ucDummy);
}

/* ----------------------------------------------------------- */

unsigned char PGA_GetCurGain()
{
	return gucPGAGain;
}

