/* ----------------------------------------------------------
		
		Psychedelic led lights with dsPic 30F2020 
		Marco Signorini - marco.signorini@libero.it
		
		File: pwmeter.c
	-----------------------------------------------------------
*/

#include "hardware.h"
#include "pwmeter.h"

int iZero __attribute__((space(ymemory))) = 0;

#ifdef _PWMETER_AVERAGE_ALG_1
	unsigned int PWMT_EvalBuffer(fractional* pBuffer, fractional *pPower)
	{
		register int result asm("A");
		unsigned char ucN;
		unsigned int uiNextSample;
		int *xmemory = pBuffer;
		int *ymemory = &iZero;
	
		/* Accumulate squared samples and previous calculated factor in the accumulator */
		result = __builtin_clr_prefetch(&xmemory, &uiNextSample, 2, &ymemory, &iZero, 0, 0);
		result = __builtin_add(*pPower,0);
		
		for (ucN = SMPBUFFER; ucN; --ucN)
		{
			result = __builtin_edac(uiNextSample, &xmemory, 2, &ymemory, 0, &uiNextSample);	
			result = __builtin_sftac(1);
		}
	
		/* The new calculated power is found in the accumulator */
		*pPower = ACCAH;
		
		/* Return an unsigned int proportional with the power found */
		uiNextSample = (0xFFFFL * (*pPower)) >> 18;
		
		return uiNextSample;
	}
#else
	unsigned int PWMT_EvalBuffer(fractional* pBuffer, fractional *pPower)
	{
		register int result asm("A");
		unsigned char ucN;
		unsigned int uiNextSample;	
		int *xmemory = pBuffer;
		int *ymemory = &iZero;
	
		/* Accumulate squared samples from the buffer in the accumulator */
		result = __builtin_clr_prefetch(&xmemory, &uiNextSample, 2, &ymemory, &iZero, 0, 0);
		for (ucN = SMPBUFFER; ucN; --ucN)
			result = __builtin_edac(uiNextSample, &xmemory, 2, &ymemory, 0, &uiNextSample);	
	
		/* Divide the accumulated result for the number of samples */
		/* and average with the previous calculated value */
		uiNextSample = __builtin_sac(result, POWERSHIFTVAL);
		*pPower = (((long)(uiNextSample)) + (*pPower)) >> 1;
		
		/* Return an unsigned int proportional with the power found */
		uiNextSample = (0xFFFFL * (*pPower)) >> 16;

		return uiNextSample;
	}
#endif

unsigned char PWMT_CountClipped(fractional *pBuffer)
{
	unsigned char ucN;
	unsigned char ucClipped;
	register fractional fMinVal = Q15(-0.99)
	register fractional fMaxVal = Q15(0.99)

	ucClipped = 0;
	for (ucN = SMPBUFFER; ucN; --ucN)
	{
		if (pBuffer[ucN] < fMinVal)
			ucClipped++;
		if (pBuffer[ucN] > fMaxVal)
			ucClipped++;
	}

	return ucClipped;
}

