;**********************************************************************
;                                                                     *
;    Filename:	    tripod_hd.asm                                     *
;    Date:          02/10/2010                                        *
;    File Version:  0.3.2                                             *
;                                                                     *
;    Author:     Dick Lord                                            *
;    Company:    uPward Concepts                                      *
;                                                                     * 
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required:   none                                           *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:                                                           *
;    File Version:  0.1.0	03/13/08	Stepping motor controller     *
;                                                                     *
;    File Version:  0.1.1   12/22/09    Adds LCD display              *
;                                                                     *
;    File Version:  0.1.2   12/28/09    Adds menu system              *
;                                                                     *
;    File Version:  0.2.0   12/29/09    Adds EEPROM save/load         *
;                                                                     *
;    File Version:  0.2.1   12/31/09    Adds ext step sw              *
;                                                                     *
;    File Version:  0.2.2   01/02/10    Adds slow pan                 *
;                                                                     *
;    File Version:  0.2.3   01/06/10    Adds programmable sys param   *
;                                                                     *
;    File Version:  0.2.4   01/21/10    Adds micro-stepping           *
;                                                                     *
;    File Version:  0.3.0   01/21/10    converted to 16f877a          *
;                                                                     *
;    File Version:  0.3.1   02/04/10    motor ramp & sys param page   *
;                                                                     *
;    File Version:  0.3.2   02/09/10    24 char display & halt        *
;                                                                     *
;**********************************************************************



	list      p=16f877            ; list directive to define processor
	#include <p16f877a.inc>        ; processor specific variable definitions
	
;	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_ON & _PWRTE_ON & _HS_OSC & _WRT_ENABLE_ON & _LVP_ON & _CPD_OFF

; '__CONFIG' directive is used to embed configuration data within .asm file.
; The lables following the directive are located in the respective .inc file.
; See respective data sheet for additional information on configuration word.
;
; CURRENT SOFTWARE VERSION
;
VERSION_H	EQU	'X'
VERSION_M	EQU	'3'
VERSION_L	EQU	'3'
;
; CONDITIONAL COMPILE
;
DISP24		EQU	1
DEBUG_IO	EQU	0
;
; register descriptions
w	EQU	0
W	equ	0
f	EQU	1
F	EQU	1

; ***** CONSTANTS

; OSC_FREQ		EQU	D'4000000'
OSC_FREQ		EQU	D'20000000'
;
BAUDRATE	EQU	D'9600'
BAUDSBRG	EQU	(OSC_FREQ / (BAUDRATE * D'64')) - 1
; T2_period	EQU	D'40' - 1	; value for 10ms interrupts
; T2_period		EQU	D'4' - 1	; value for 1ms interrupts
T2_period		EQU	D'8' - 1	; value for 25uS interrupts

;
PRE_COUNT_RATIO	EQU	D'4'	; ratio of timeout2 to interrupt rate
TIMER_RATIO		EQU D'100'	; ratio of timeout to timeout2
;
LCD_INIT_DELAY	EQU	D'20'
;
;		NUMBERS
;
TEN				EQU D'10'
HUNDRED			EQU	D'100'
THOUSAND		EQU	D'1000'
TENTHOU			EQU	D'10000'
;
;	Character definitions for rs-232 port
;
EOL_CHAR		EQU	0x0d
;
; ******* Fixed System Parameters
;
JOG_DELAY		EQU	D'128'		; delay from incr/decr to first jog
JOG_COUNT_VAL	EQU	D'50'		; jog steps before high speed jogging
STEP_DELAY1		EQU	D'5'		; slow jog rate delay
STEP_DELAY2		EQU	D'1'
ELEV_SLEW_RATE	EQU	D'10'		; elevation slew rate delay
PAN_SLEW_RATE	EQU	D'5'		; pan slew rate delay
;
SHUTTER_T1		EQU	4			; delay from shutter "1st press" to "full press"
SHUTTER_T2		EQU	4			; duration of shutter "full press"
;
MAX_ELEV_STEPCT	EQU	D'100'
MAX_PAN_STEPCT	EQU	D'100'
MAX_SETTLE_TIME	EQU	D'100'		; 1 second (.01 sec increments)
MAX_HOLD_TIME	EQU	D'250'		; 25 seconds (.1 second increments)
MAX_PROG_NUM	EQU	D'12' - 1
;
STEPS	EQU	D'500'
STEPS_H	EQU	STEPS/D'256'
STEPS_L	EQU	STEPS - D'256'*STEPS_H
;
; *********************************
; ***** HARDWARE DEFINITIONS  *****
; *********************************
		;	PORT-A	(EXT SWITCH input)
		;	| 0 | 0 | 0 | 0 | 0 | 0 | 1 | 0 |	= 0x02
		;	|ra7|ra6|ra5|ra4|ra3|ra2|ra1|ra0|
		;	| - | - | - | - | - | - |ext| - |	
		;
		;	PORT-B	(input switch and LCD display data interface)
		;	| 1 | 1 | 1 | 1 | 0 | 0 | 0 | 0 |	= 0xF0
		;	|rb7|rb6|rb5|rb4|rb3|rb2|rb1|rb0|
		;	|sw3|sw2|sw1|sw0|da3|da2|da1|da0|
		;
		;	PORT-C  (Serial port and LCD control)
		;	| 1 | 0 | 0 | 0 | 0 | 0 | 0 | 0 |	= 0x80
		;	|rc7|rc6|rc5|rc4|rc3|rc2|rc1|rc0|
		;	|RXi|TXo| - | - | - | E |R/W|RS0|
		;
		;	PORT-D	(Stepping motor driver outputs)
		;	| 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0 |	= 0x00
		;	|rd7|rd6|rd5|rd4|rd3|rd2|rd1|rd0|
		;	|pb-|pb+|pa-|pa+|eb-|eb+|ea-|ea+|
		;		initialize as all outputs
		;
		;	PORT-E  (Shutter outputs)
		;	| 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0 |	= 0x00
		;	| - | - | - | - |re3|re2|re1|re0|
		;	| - | - | - | - | - |LED|SH1|SH0|
		;
;
INIT_PORT_A	EQU	0x00
INIT_PORT_B	EQU	0x00
INIT_PORT_C	EQU	0x00
INIT_PORT_D	EQU	0xff
INIT_PORT_E	EQU	0x00
;
;	Stepping Motors
;  Azimuth stepper output = RD[7:4]
;  Elevation steppper out = RD[3:0]
;
; | wdg B | wdg A |
; | - | + | - | + |
;
MOTOR_PORT	EQU	PORTD
MTR_A_PLUS	EQU	0xFE	; motor winding A + polarity
MTR_A_MINUS	EQU	0xFD	; motor winding A - polarity
MTR_B_PLUS	EQU	0xFB	; motor winding B + polarity
MTR_B_MINUS	EQU	0xF7	; motor winding B - polarity
;
;	Serial port = RC[7:6]
;
;	Shutter release = RE[1:0]
SHUTTER_REG	EQU	PORTE
SW2			EQU	0	; full-press (shutter release)
SW1			EQU	1	;  half-press (focus)
LED_OUT		EQU	2	; IR LED shutter output
;
;	push-buttons
;	RB7 = Enter
;	RB6 = -->
;	RB5 = <--
;	RB4 = Exit
;
;
EXT_SW_PORT	EQU	PORTA
EXT_SW		EQU 1		; SW_EN_PORT bit position for external switch input
;
SW_PORT		EQU	PORTB
SWITCH_1	EQU	0		; leftmost switch bit position (after SWAPF)
SWITCH_2	EQU	1		; 2nd switch bit position
SWITCH_3	EQU	2		; 3rd switch bit position
SWITCH_4	EQU	3		; rightmost switch bit position
;
;	**************************
;	LCD definitions
;	**************************
;
;	LCD display
;	RB3 = d7
;	RB2 = d6
;	RB1 = d5
;	RB0 = d4
;	RC2	= E
;	RC1 = R/-w
;	RC0	= RS
;
; LCD Display ports and control signal names.
;
LCD_DATA        EQU     PORTB
LCD_DATA_TRIS   EQU     TRISB
LCD_CNTL        EQU     PORTC
;
; Bit positions for LCD_CNTL for control signals
;
E				EQU	2	; LCD Enable control line 			(RC2)
RW				EQU	1	; LCD Read/Write control line		(RC1)
RS				EQU	0	; LCD Register Select control line	(RC0)
;
; LCD Module command definitions
;
DISP_ON			EQU	0x00C		; Display on
DISP_ON_C		EQU	0x00E		; Display on, Cursor on
DISP_ON_B		EQU	0x00F		; Display on, Cursor on, Blink cursor
DISP_OFF		EQU	0x008		; Display off
CLR_DISP		EQU	0x001		; Clear the Display
ENTRY_INC		EQU	0x006		;
ENTRY_INC_S		EQU	0x007		;
ENTRY_DEC		EQU	0x004		;
ENTRY_DEC_S		EQU	0x005		;
DD_RAM_ADDR		EQU	0x080		; Least Significant 7-bit are for address
DD_RAM_UL		EQU	0x080		; Upper Left corner of the Display
;
; PROGRAM flags
;	prog_type
PROG_SLO_PAN_FLG	EQU	0
PROG_1D_FLG			EQU	1
PROG_2D_FLG			EQU	2
;	prog_run_flags
HALT_FLG			EQU	7		; set if HALT is detected
HALT_EN				EQU	6		; set to enable HALT detection
SINGLE_STEP_FLG		EQU	5		; set when single-stepping
POSITION_DISP_EN	EQU	4		; set to display position in slow-pan
SPEED_RAMP_EN		EQU	3		; set when Ramp-up/down is enabled
RAMP_UP_FLAG		EQU	2		; set at beginning of move for ramp-up
		;

; *********************************
; ***** VARIABLE DEFINITIONS  *****
; *********************************
;
; *** motor control variables
;
mmask2			EQU	0x20
mmask1			EQU	0x21
tmask2			EQU	0x22
tmask1			EQU	0x23
e_phase			EQU	0x24
p_phase			EQU	0x25
temp_tbl		EQU	0x26
m_ctrl_flg		EQU	0x27
;
; *** motion variables
;
pan_rate_h		EQU	0x28
pan_rate_l		EQU 0x29
;
elev_cur_pos_h	EQU	0x2a
elev_cur_pos_l	EQU	0x2b
pan_cur_pos_h	EQU	0x2c
pan_cur_pos_l	EQU	0x2d
new_pos_h		EQU	0x2e
new_pos_l		EQU	0x2f
;
; *** program execution variables
;
prog_run_flags	EQU	0x30
step_val		EQU	0x31
e_step_count	EQU	0x32
e_cur_step		EQU	0x33
p_step_count	EQU	0x34
p_cur_step		EQU	0x35
temp_ctr		EQU	0x36
;
; *** input menu execution variables
;
prog_num		EQU	0x37	; save/restore #
sys_state		EQU	0x38
jog_count		EQU	0x39
param_mode		EQU	0x3a
;
; RS-232 receive variables
;
rcvbyte			EQU	0x3b
rbufptr			EQU	0x3c
rcv_digit		EQU	0x3d
rcv_num_h		EQU	0x3e
rcv_num_l		EQU	0x3f
rxbuf			EQU	0xA0  	; receive buffer (A0-AF) accessed thru FSR
;
; *** Current System parameters
; *** Saved and restored as "System" parameters
;
shutter_T1		EQU	0x40		; delay from shutter "1st press" to "full press"
shutter_T2		EQU	0x41		; duration of shutter "full press"
								;
settle_time		EQU	0x42		; delay from motion to shutter
								;
pan_fast_rate	EQU	0x43		; pan fast slew rate timeout
pan_slow_rate	EQU	0x44		; pan start rate timeout
pan_ramp_dist	EQU	0x45		; pan deceleration steps
								;
elev_fast_rate	EQU	0x46		; elevation fast slew rate timeout
elev_slow_rate	EQU	0x47		; elevation start rate timeout
elev_ramp_dist	EQU	0x48		; elevation deceleration steps
;
SYS_PARAM_START	EQU shutter_T1
SYS_PARAM_END	EQU	elev_ramp_dist
SYS_PARAM_SIZE	EQU	SYS_PARAM_END - SYS_PARAM_START + 1
SYS_EE_ADDR		EQU	0x100 - SYS_PARAM_SIZE
				;
;
; *** Current Program parameters
; *** Saved and restored as "Programs" 1 thru 8
				;
prog_type		EQU	0x50
elev_start_h	EQU	0x51
elev_start_l	EQU	0x52
elev_step_h		EQU	0x53
elev_step_l		EQU	0x54
elev_stepct		EQU	0x55
pan_start_h		EQU	0x56
pan_start_l		EQU	0x57
pan_step_h		EQU	0x58
pan_step_l		EQU	0x59
pan_stepct		EQU	0x5A
hold_time		EQU	0x5B
		;
CUR_PROG_PARAM_START	EQU prog_type
CUR_PROG_PARAM_END		EQU hold_time
PROG_PARAM_SIZE		EQU	CUR_PROG_PARAM_END - CUR_PROG_PARAM_START + 1
		;
keypad_lock		EQU	0x5F
;
; *** display variables
;
SHDW_A			EQU	0x60
CHAR			EQU	0x61
TEMP			EQU	0x62
LO_TEMP			EQU	0x63
;
tempa			EQU	0x64
tempb			EQU	0x65
tempc			EQU	0x66
;
BCflag			EQU	0x67
;
digit_ctr		EQU	0x68
;
minuend_h		EQU	0x69	; minuend MSB
minuend_l		EQU	0x6a	; minuend LSB
;
tempm_h			EQU	0x6b	; temporary minuend
tempm_l			EQU	0x6c
;
subtrahend_h	EQU	0x6d	; subtrahend
subtrahend_l	EQU	0x6e
;
;
; *** interrupt handler variables
;
w_temp			EQU	0x70	; variable used for context saving 
status_temp		EQU	0x71	; variable used for context saving
pc_lath_temp	EQU	0x72
timeout_ctr		EQU	0x73
time_phase		EQU	0x74
pre_count		EQU	0x75		; interrupt count to timeout2
timeout			EQU	0x76
timeout2		EQU	0x77
temp_fsr		EQU	0x78
;
; *** keypad input variables
;
sw_state		EQU	0x79
sw_temp			EQU	0x7a
		;
rsign			EQU	0x7b
rdigit3			EQU	0x7c
rdigit2			EQU	0x7d
rdigit1			EQU	0x7e
rdigit0			EQU	0x7f
		;
		; ***********************
		; *  state definitions  *
		; ***********************
run_state			EQU	0x00
prog_state			EQU	0x01
elev_start_state	EQU	0x02
elev_step_state		EQU	0x03
elev_stepct_state	EQU	0x04
pan_start_state		EQU	0x05
pan_step_state		EQU	0x06
pan_stepct_state	EQU	0x07
settle_state		EQU	0x08
hold_state			EQU	0x09
get_state			EQU	0x0A
save_state			EQU	0x0B
confirm_state		EQU	0x0C
pan_final_state		EQU	0x0D
pan_time_state		EQU	0x0E
error_state			EQU	0x0F
		;
				; ***************************************
				; ******    START OF CODE SPACE    ******
				; ***************************************
				; ***************************************
		ORG     0x000             ; processor reset vector
		clrf    PCLATH            ; ensure page bits are cleared
  		goto    main              ; go to beginning of program
					;
				; *************************************
				; ******    INTERRUPT HANDLER    ******
				; *************************************
					;
					; save context
					;
		ORG     0x004             ; interrupt vector location
int_handler
;	bsf		SHUTTER_REG,LED_OUT		; turn on LED output
		movwf   w_temp				; save off current W register contents
		movf	STATUS,w			; move status register into W register
		bcf     STATUS,RP1			; ensure file register bank set to 0
		bcf     STATUS,RP0			; ensure file register bank set to 0
		movwf	status_temp			; save off contents of STATUS register
		movf	PCLATH,w			; get current program space page latch
		movwf	pc_lath_temp		; save a temporary copy
		movlw	high int_handler	; fix latch for goto's in interrupt
		movwf	PCLATH
					;
					; test for timer-2 interrupt
		btfss	PIR1,TMR2IF	; was interrupt source timer 2?
		 goto	int_rcv		; no, test for receive interrupt
					;
					; yes - timer-2 interrupt
int_t2
		bcf		PIR1,TMR2IF	; reset the interrupt flag
					;
					; **********************
					; * Motor control code *
					; **********************
		incf	time_phase,w	; increment time phase
		andlw	0x03
		movwf	time_phase
		btfsc	STATUS,Z		; does phase = 0
		 goto	 int_t2_0		; no, then don't update
					;
					; update masks if there is a change
					;
		btfss	m_ctrl_flg,0	; are we updating motor outputs ?
		 goto	int_t2_0		; no, skip update
		movf	tmask2,w		; copy new motor masks
		movwf	mmask2
		movf	tmask1,w
		movwf	mmask1
;		movf	tmask0,w
;		movwf	mmask0
		bcf		m_ctrl_flg,0	; clear update flag
					;
					; use masks to PWM motor outputs
int_t2_0
		movf	mmask2,w		; get 100% mask
;		btfss	time_phase,1	; test 50% duty cycle (10 KHz version)
		btfss	time_phase,0	; test 50% duty cycle (20 KHz version)
		 goto	 int_t2_0a
		andwf	mmask1,w		; apply 50% bits to motor outputs
;		goto	int_t2_0b		; set output
int_t2_0a
;		btfsc	time_phase,0	; test 25% duty cycle
;		btfsc	time_phase,1	; test 25% duty cycle
;		 andwf	mmask0,w		; apply 25% bits to motor outputs
; int_t2_0b
		movwf	MOTOR_PORT
					;
					; process timeouts
		decfsz	pre_count,f		; count off timeout ratio
		 goto	int_t2_exit
		movlw	PRE_COUNT_RATIO
		movwf	pre_count
					;
		movf	timeout2,f		; test timeout2 variable
		btfsc	STATUS,Z		; is it zero ?
		 goto	int_t2_2		; yes, leave it alone
		decf	timeout2,f		; if not yet zero then count down
int_t2_2
		decfsz	timeout_ctr,f	; count off timeout ratio
		 goto	int_t2_exit
		movlw	TIMER_RATIO
		movwf	timeout_ctr
					;
		movf	timeout,f		; test timeout variable
		btfsc	STATUS,Z		; is it zero ?
		 goto	int_t2_exit		; yes, leave it alone
		decf	timeout,f		; if not yet zero then count down
int_t2_exit
		goto	int_exit
	;
	; test for RS-232 receive
	;
int_rcv
		btfss	PIR1,RCIF		; is it a serial receive interrupt ?
		 goto	int_error		; no, unknown interrupt source
	;
	; RS-232 receive interrupt handler
	;	
int_rcv_01
		movf	RCREG,w				; get received byte
		movwf	rcvbyte				; save current receive value
;	call	OUT_UART
		bcf		PIR1,RCIF			; clear interrupt flag
		btfss	rbufptr,7			; is rcv buffer already full ?
		 goto	int_rcv_02			; no, then process message
		bsf		rbufptr,6			; yes, mark over-run
		goto	int_rcv_exit
int_rcv_02
	call	OUT_UART
		movf	FSR,w				; save FSR to temp
		movwf	temp_fsr
		movf	rbufptr,w			; get buffer pointer
		andlw	0x0f				; mask off flags
		addlw	rxbuf				; add base address of rcv buffer
		movwf	FSR
		movf	rcvbyte,w			; get received character
		movwf	INDF				; save char at buffer location
		xorlw	EOL_CHAR			; test for end of line character ?
		btfsc	STATUS,Z
		 bsf	 rbufptr,7			; yes, mark buffer as containing message
		incf	rbufptr,f			; increment pointer
		btfsc	rbufptr,4			; is buffer past end ?
		 bsf	 rbufptr,7			 ; mark as full
									;
		movf	temp_fsr,w			; restore FSR
		movwf	FSR	
int_rcv_exit
		goto	int_exit
	;
	; Spurious interrupt handler
	;
int_error
		clrf	PIR1	; whatever the interrupt was, flush it
						;
int_exit				;
						; return from interrupt - restore registers
		movf	pc_lath_temp,w	; get back high PC latch
		movwf	PCLATH
		bcf     STATUS,RP0		; ensure file register bank set to 0
		movf    status_temp,w	; retrieve copy of STATUS register
		movwf	STATUS			; restore pre-isr STATUS register contents
		swapf   w_temp,f		; restore pre-isr W register contents
		swapf   w_temp,w		; without touching Status bits
;	bcf		SHUTTER_REG,LED_OUT		; turn on LED output
		retfie					; return from interrupt

; *****************************************
; ******    START OF MAIN PROGRAM    ******
; *****************************************

main
		call	init_config		; set up all the control registers and ports
		;
		movlw	INIT_PORT_A
		movwf	SHDW_A
		movwf	PORTA
		;
		movlw	INIT_PORT_B
;		movwf	SHDW_B
		movwf	PORTB
		;
		movlw	INIT_PORT_C
		movwf	PORTC
		;
		clrf	rbufptr
;		call	init_adcon
		call	init_uart
		;
		clrf	m_ctrl_flg	; initialize motor variables
		clrf	step_val
		movlw	0xFF
		movwf	mmask2
		movwf	mmask1
;		movwf	mmask0
		movwf	tmask2
		movwf	tmask1
;		movwf	tmask0
;
		clrf	time_phase
		clrf	timeout
		clrf	timeout2
		movlw	PRE_COUNT_RATIO ; set division from interrupt rate to timeout-2
		movwf	pre_count
		movlw	TIMER_RATIO
		movwf	timeout_ctr
		;
		call	init_timer2	; set up slew-rate interrupt
		bsf		INTCON,PEIE	; enable peripheral interrupts
		bsf		INTCON,GIE	; global enable
		;
		clrf	sw_state
		clrf	keypad_lock		; keypad is active
		;
		call	DISPLAY_INIT	; initialize LCD display
		;
		movlw	HIGH greeting_txt	; set PC high bits
		movwf	PCLATH
		call	greeting_txt	; "  Digital Pan Head  "
		;						; " R.H.L.   Ver x.x.x "
		;
		movlw	HIGH make_up_dn_arrow	; set PC high bits
		movwf	PCLATH
		call	make_up_dn_arrow		; load LCD char gen with graphic
		;
		;
		movlw	HIGH main	; reset PC high bits
		movwf	PCLATH
		;
		clrf	new_pos_h
		clrf	new_pos_l
		;
		clrf	elev_cur_pos_h
		clrf	elev_cur_pos_l
		;
		clrf	pan_cur_pos_h
		clrf	pan_cur_pos_l
		;
		clrf	e_phase
		clrf	p_phase
		;
		clrf	prog_num
		;
		; initialize state of program space
		;
		clrf	prog_run_flags
		;
		clrf	prog_type
		clrf	elev_start_h
		clrf	elev_start_l
		clrf	elev_step_h
		clrf	elev_step_l
		clrf	elev_stepct
		;
		clrf	pan_start_h
		clrf	pan_start_l
		clrf	pan_step_h
		clrf	pan_step_l
		clrf	pan_stepct
		;
		movlw	D'40'			; 4 seconds
		movwf	hold_time
		;
		; initialize system parameters
		;
		movlw	HIGH load_sys_param_p1	; set PC high bits
		movwf	PCLATH
		call	load_sys_param_p1		; get system parameters from EEPROM
		movlw	HIGH main			; restore PC high bits
		movwf	PCLATH
		;
		call	read_sw_state	; check for SYS programming
		movf	sw_state,w		; power-on with SW_1 and SW_4
		xorlw	0x09
		btfsc	STATUS,Z
		 goto	sys_setup
		;
main0
		movlw	D'200'		; display title for 2 seconds
		movwf	timeout
main1
		movf	timeout,f
		btfss	STATUS,Z
		 goto	main1
		;
		movlw	HIGH serial_greeting_p1	; set PC high bits
		movwf	PCLATH
		call	serial_greeting_p1
		movlw	HIGH main1			; restore PC high bits
		movwf	PCLATH
		;
main2
		call	pan_step	; turn on pan motor
		call	elev_step	; turn on elevation motor
;
;		movf	tempa,w
;		movwf	PORTD
;		incf	tempa,f
;		goto	main2

;		movlw	0xC0
;		call	SEND_CMD
;		movf	PORTA,w
;		call	OUTHX
;
;		movlw	0x55
;		call	OUT_UART
;
;		movlw	3
;		movwf	timeout
; main3
;		movf	timeout,f
;		btfss	STATUS,Z
;		 goto	main3
;		goto	main2
		
		goto	set_run_state
main_loop
		btfss	rbufptr,7		; test for RS-232 command
		 goto	do_state
				;
				; jump to RS-232 receive handler
				;
		movlw	HIGH rcv_cmd_p1	; set up PC high bits
		movwf	PCLATH
		call	rcv_cmd_p1
		movlw	HIGH main_loop	; restore PC high bits
		movwf	PCLATH
		btfss	keypad_lock,5
		 goto	 do_state
		bcf		keypad_lock,5
		goto	main2

;	goto	main_loop
				;
				;
				; *************************************
				; *************************************
				; *****  SYSTEM	STATE MANAGEMENT  *****
				; *************************************
				; *************************************
				;
				; *****************************
				; *  state handling routines  *
				; *****************************
do_state
		movlw	HIGH state_table	; set up PC high bits
		movwf	PCLATH
		movf	sys_state,w			; get current state
		andlw	0x0F
		addwf	PCL,f
state_table
		goto	do_run_state			; 0x00
		goto	do_prog_state			; 0x01
		goto	do_elev_start_state		; 0x02
		goto	do_elev_step_state		; 0x03
		goto	do_elev_stepct_state	; 0x04
		goto	do_pan_start_state		; 0x05
		goto	do_pan_step_state		; 0x06
		goto	do_pan_stepct_state		; 0x07
;		goto	do_settle_state			; 0x08
		goto	do_error_state
;
		goto	do_hold_state			; 0x09
		goto	do_get_state			; 0x0A
		goto	do_save_state			; 0x0B
		goto	do_confirm_state		; 0x0C
		goto	do_pan_final_state		; 0x0D
		goto	do_pan_time_state		; 0x0E
		goto	do_error_state			; 0x0F
;

do_run_state		; 0x00
		;	run_prog_txt		; "  Run Current Prog  "
		;	get_run_button_txt	; "Prog  Get  Man  Run "
		call	read_sw_state
		btfsc	sw_state,SWITCH_1	; "Prog"
		 goto	set_prog_state
		btfsc	sw_state,SWITCH_2	; "Get"
		 goto	set_get_state
		btfsc	sw_state,SWITCH_3	; "Man"
		 goto	step_current_program
		btfsc	sw_state,SWITCH_4	; "Run"
		 goto	run_current_program
		goto	main_loop
;
do_prog_state		; 0x01
		;	prog_entry_txt		; " Program Entry Mode "
		;	prog_button_txt	    ; "EXIT  Pan  1D    2D "
		call	read_sw_state
		btfsc	sw_state,SWITCH_4	; "2D"
		 goto	set_prog_2d
		btfsc	sw_state,SWITCH_3	; "1D"
		 goto	set_prog_1d
		btfsc	sw_state,SWITCH_2	; "Pan"
		 goto	set_prog_slo_pan
		btfsc	sw_state,SWITCH_1	; "EXIT"
		 goto	set_run_state
		goto	main_loop				;
		;
do_elev_start_state	; 0x02
		call	output_elev_start		; display start position
		movlw	0
		call	set_elev_step_position		; set elev motor to start position
		;
		;	elev_start_txt		; "Elev Start Pos=     "
		;	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
		call	read_sw_state
		btfsc	sw_state,SWITCH_4	; "ENTER"
		 goto	set_elev_step_state
		btfsc	sw_state,SWITCH_1	; "Prev"
		 goto	set_prog_state
		btfsc	sw_state,SWITCH_3	; " up^ "
		 goto	elev_start_increment
		btfsc	sw_state,SWITCH_2	; " !dn "
		 goto	elev_start_decrement
		goto	main_loop
				;
do_elev_step_state	; 0x03
		call	output_elev_step
		movlw	1
		call	set_elev_step_position	; go to first step position
		;
		;	elev_step_txt		; "Elev Step Size=     "
		;	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
		call	read_sw_state
		btfsc	sw_state,SWITCH_4	; "ENTER"
		 goto	set_elev_stepct_state
		btfsc	sw_state,SWITCH_1	; "Prev"
		 goto	set_elev_start_state
		btfsc	sw_state,SWITCH_3	; " up^ "
		 goto	elev_step_increment
		btfsc	sw_state,SWITCH_2	; " !dn "
		 goto	elev_step_decrement
		goto	main_loop
				;
do_elev_stepct_state	; 0x04
		call	output_elev_stepct
		movf	elev_stepct,w
		call	set_elev_step_position	; goto current position
				;
		;	elev_stepct_txt		; "Elev #of Steps=     "
		;	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
		call	read_sw_state
		btfsc	sw_state,SWITCH_4	; "ENTER"
		 goto	set_pan_start_state
		btfsc	sw_state,SWITCH_1	; "Prev"
		 goto	set_elev_step_state
		btfsc	sw_state,SWITCH_3	; " up^ "
		 goto	elev_stepct_increment
		btfsc	sw_state,SWITCH_2	; " !dn "
		 goto	elev_stepct_decrement
		goto	main_loop
				;
do_pan_start_state	; 0x05
		call	output_pan_start		; display start position
		movlw	0
		call	set_pan_step_position		; set pan motor to start position
		;
		;	pan_start_txt		; " Pan Start Pos=     "
		;	pan_button_txt		; "Prev  <--  --> ENTER"
		call	read_sw_state
		btfsc	sw_state,SWITCH_4	; "ENTER"
		 goto	set_pan_next_state
		btfsc	sw_state,SWITCH_1	; "Prev"
		 goto	set_prog_state
		btfsc	sw_state,SWITCH_3	; " --> "
		 goto	pan_start_increment
		btfsc	sw_state,SWITCH_2	; " <-- "
		 goto	pan_start_decrement
		goto	main_loop
				;
do_pan_step_state	; 0x06
		call	output_pan_step
		movlw	1
		call	set_pan_step_position	; go to first step position
		;
		;	pan_step_txt		; " Pan Step Size=     "
		;	pan_button_txt		; "Prev  <--  --> ENTER"
		call	read_sw_state
		;
		btfsc	sw_state,SWITCH_4	; "ENTER"
		 goto	set_pan_stepct_state
		;
		btfsc	sw_state,SWITCH_1	; "Prev"
		 goto	set_pan_start_state
		;
		btfsc	sw_state,SWITCH_3	; " --> "
		 goto	pan_step_increment
		;
		btfsc	sw_state,SWITCH_2	; " <-- "
		 goto	pan_step_decrement
		;
		goto	main_loop
				;
do_pan_stepct_state	; 0x07
		call	output_pan_stepct
		movf	pan_stepct,w
		call	set_pan_step_position
				;
		;	pan_final_txt		; " Pan #of Steps=     "
		;	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
		call	read_sw_state
		;
		btfsc	sw_state,SWITCH_4	; "ENTER"
		 goto	set_hold_state
		;
		btfsc	sw_state,SWITCH_1	; "Prev"
		 goto	set_pan_step_state
		;
		btfsc	sw_state,SWITCH_3	; " --> "
;		 goto	pan_stepct_increment
		 goto	pan_stepct_cw
		;
		btfsc	sw_state,SWITCH_2	; " <-- "
;		 goto	pan_stepct_decrement
		 goto	pan_stepct_ccw
		;
		goto	main_loop
				;
; do_settle_state		; 0x08
;		call	output_settle_time	; display it
		;
		;	settle_txt			; " Settling Time=     "
		;	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
;		call	read_sw_state
;		btfsc	sw_state,SWITCH_4	; "ENTER"
;		 goto	set_hold_state
;		btfsc	sw_state,SWITCH_1	; "Prev"
;		 goto	set_pan_stepct_state
;		btfsc	sw_state,SWITCH_3	; " up^ "
;		 goto	settle_increment
;		btfsc	sw_state,SWITCH_2	; " !dn "
;		 goto	settle_decrement
;		goto	main_loop
				;
do_hold_state		; 0x09
		call	output_hold_time	; display it
		;
		;	holding_txt			; "  Holding Time=     "
		;	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
		call	read_sw_state
		btfsc	sw_state,SWITCH_4	; "ENTER"
		 goto	set_save_state
		btfsc	sw_state,SWITCH_1	; "Prev"
		 goto	set_pan_stepct_state
		btfsc	sw_state,SWITCH_3	; " up^ "
		 goto	hold_increment
		btfsc	sw_state,SWITCH_2	; " !dn "
		 goto	hold_decrement
		goto	main_loop
				;
do_get_state		; 0x0A
		call	output_prog_num		; display program number
		call	show_prog_type		; show type of program
		;
		;	get_prog_txt		; "      Get Prog#     "
		;	load_button_txt		; "EXIT  !dn  up^  LOAD"
		call	read_sw_state
		btfsc	sw_state,SWITCH_4	; "LOAD"
		 goto	prog_load
		btfsc	sw_state,SWITCH_1	; "EXIT"
		 goto	set_run_state
		btfsc	sw_state,SWITCH_3	; " up^ "
		 goto	prognum_increment
		btfsc	sw_state,SWITCH_2	; " !dn "
		 goto	prognum_decrement
		goto	main_loop
				;
prog_load
		movf	prog_num,w	
		call	check_program	
		andlw	0xF8
		btfss	STATUS,Z
		 goto	prog_load_1
		call	load_program
		goto	set_run_state
prog_load_1
		movlw	high no_program_txt
		movwf	PCLATH
		call	no_program_txt		;  " No Program     EXIT"
		movlw	high run_program	; reset main page	
		movwf	PCLATH
prog_load_2
		call	read_sw_state
		movf	sw_state,f
		btfss	STATUS,Z
		 goto	prog_load_2
prog_load_3
		call	read_sw_state
		movf	sw_state,f
		btfsc	sw_state,SWITCH_4	; "EXIT"
		 goto	set_get_state
		goto	prog_load_2
				;
show_prog_type
		movf	prog_num,w	
		call	check_program
		movwf	tempa			; save current prog type
		movlw	0x80
		call	SEND_CMD
		movlw	'['
		call	OUTCH
		movf	tempa,w
		andlw	0xF8
		btfss	STATUS,Z			; is it invalid
		 goto	show_prog_type_1
		btfsc	tempa,PROG_SLO_PAN_FLG
		 goto	show_prog_type_2
		btfsc	tempa,PROG_1D_FLG
		 goto	show_prog_type_3
		btfsc	tempa,PROG_2D_FLG
		 goto	show_prog_type_4
show_prog_type_1
		movlw	'-'
		call	OUTCH
		movlw	'-'
		call	OUTCH
		movlw	'-'
		call	OUTCH
		movlw	']'
		call	OUTCH
		return
show_prog_type_2
		movlw	'S'
		call	OUTCH
		movlw	'l'
		call	OUTCH
		movlw	'o'
		call	OUTCH
		movlw	']'
		call	OUTCH
		return
show_prog_type_3
		movlw	'1'
		call	OUTCH
		movlw	'a'
		call	OUTCH
		movlw	'x'
		call	OUTCH
		movlw	']'
		call	OUTCH
		return
show_prog_type_4
		movlw	'2'
		call	OUTCH
		movlw	'a'
		call	OUTCH
		movlw	'x'
		call	OUTCH
		movlw	']'
		call	OUTCH
		return
		;
do_save_state		; 0x0B
		call	home_position
		;
		call	output_prog_num		; display it
		;
		;	save_as_txt			; "  Save as Prog#    ?"
		;	save_button_txt		; "EXIT  !dn  up^  SAVE"
		call	read_sw_state
		btfsc	sw_state,SWITCH_4	; "SAVE"
		 goto	set_confirm_state
		btfsc	sw_state,SWITCH_1	; "EXIT"
		 goto	set_run_state
		btfsc	sw_state,SWITCH_3	; " up^ "
		 goto	prognum_increment
		btfsc	sw_state,SWITCH_2	; " !dn "
		 goto	prognum_decrement
		goto	main_loop
				;
do_confirm_state	; 0x0C
		;	replace_txt			; "Replace Current Prog"
		;	yes_no_button_txt	; "Prev  ---  NO   YES "
		call	read_sw_state
		btfsc	sw_state,SWITCH_4	; "YES"
		 goto	prog_save
		btfsc	sw_state,SWITCH_3	; "NO"
		 goto	set_run_state
		btfsc	sw_state,SWITCH_1	; "Prev"
		 goto	set_save_state
		goto	main_loop
				;
prog_save
		movlw	high saving_prog_txt ; set up for page2 call
		movwf	PCLATH
		call	saving_prog_txt		; "   Saving Program   "
		movlw	HIGH save_program	; reset PC high bits
		movwf	PCLATH
				;
		call	save_program
									;
		movlw	D'50'				; display "saving" message for 1/2 second
		movwf	timeout
prog_sav_1
		movf	timeout,f
		btfss	STATUS,Z
		 goto	prog_sav_1
		goto	set_run_state
				;
do_pan_final_state	; 0x0D
		call	output_pan_step			; display step position as final
		movlw	1
		call	set_pan_step_position	; go to pan_final_position position
		;								; (same variable as step poaition)
		;	pan_final_txt		; " Pan Final Pos=     "
		;	pan_button_txt		; "Prev  <--  --> ENTER"
		call	read_sw_state
		btfsc	sw_state,SWITCH_4	; "ENTER"
		 goto	set_pan_time_state
		btfsc	sw_state,SWITCH_1	; "Prev"
		 goto	set_pan_start_state
		btfsc	sw_state,SWITCH_3	; " --> "
		 goto	pan_step_increment
		btfsc	sw_state,SWITCH_2	; " <-- "
		 goto	pan_step_decrement
		goto	main_loop
				;
do_pan_time_state	; 0x0E
		call	output_hold_time	; display hold time as pan total time
		;
		;	holding_txt			; "Total Pan Time=     "
		;	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
		call	read_sw_state
		btfsc	sw_state,SWITCH_4	; "ENTER"
		 goto	set_save_state
		btfsc	sw_state,SWITCH_1	; "Prev"
		 goto	set_pan_final_state
		btfsc	sw_state,SWITCH_3	; " up^ "
		 goto	hold_increment
		btfsc	sw_state,SWITCH_2	; " !dn "
		 goto	hold_decrement
		goto	main_loop
				;

do_error_state		; 0x0E - 0x0F
		goto	main_loop
;
		;	greeting_txt	; "  Digital Pan Head  "
		;	version_txt		; " R.H.L.   Ver x.x.x "
		;	run_prog_txt	; "  Run Current Prog  "
		;	prog_entry_txt	; " Program Entry Mode "
		;	elev_start_txt	; "Elev Start Pos=     "
		;	elev_step_txt	; "Elev Step Size=     "
		;	elev_stepct_txt	; "Elev #of Steps=     "
		;	pan_start_txt	; " Pan Start Pos=     "
		;	pan_step_txt	; "Pan Step Size=     "
		;	pan_stepct_txt	; " Pan Final Pos=     "
		;	settle_txt		; " Settling Time=     "
		;	holding_txt		; "  Holding Time=     "
		;	save_as_txt		; "  Save as Prog#     "
		;	replace_txt		; "Replace Current Prog"
		;	get_prog_txt	; "      Get Prog#     "
		;
		; ********************************
		; *  soft-button prompt strings  *
		; ********************************
		;
		;	get_run_button_txt	; "Prog  Get  Man  Run "
		;	prog_button_txt	    ; "Prev  Sys  1D    2D "
		;	pan_button_txt		; "Prev  <--  --> ENTER"
		;	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
		;	save_button_txt		; "Prev  !dn  up^  SAVE"
		;	load_button_txt		; "Prev  !dn  up^  LOAD"
		;	yes_no_button_txt	; "Prev  ---  NO   YES "
		;
		; ****************************
		; *  state setting routines  *
		; ****************************
		;	RUN state
		;	run_prog_txt		; "  Run Current Prog  "
		;	get_run_button_txt	; "Prog  Get  Man  Run "
set_run_state		; 0x00
		call	home_position
		movlw	run_state
		goto	set_state_exit
		;
		;	PROG state
		;	prog_entry_txt		; " Program Entry Mode "
		;	prog_button_txt	    ; "Prev  Sys  1D    2D "
set_prog_state		; 0x01
		movlw	prog_state
		goto	set_state_exit
		;
set_prog_slo_pan
		clrf	prog_type
		bsf		prog_type,PROG_SLO_PAN_FLG	; set slo_pan prog flag
		goto	set_pan_start_state
		;
set_prog_1d
		clrf	prog_type
		bsf		prog_type,PROG_1D_FLG	; set 1d prog flag
		goto	set_pan_start_state
		;
set_prog_2d
		clrf	prog_type
		bsf		prog_type,PROG_2D_FLG	; set 2d prog flag
		;	ELEV_START state
		;	elev_start_txt		; "Elev Start Pos=     "
		;	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
set_elev_start_state	; 0x02
		movlw	elev_start_state
		goto	set_state_exit
		;
		;	ELEV_STEP state
		;	elev_start_txt		; "Elev FirstStep=     "
		;	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
set_elev_step_state	; 0x03
		movlw	elev_step_state
		goto	set_state_exit
		;
		;	ELEV_STEPCT state
		;	elev_final_txt		; "Elev #of Steps=     "
		;	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
set_elev_stepct_state	; 0x04
		movlw	elev_stepct_state
		goto	set_state_exit
		;
		;	PAN_START state
		;	pan_start_txt		; " Pan Start Pos=     "
		;	pan_button_txt		; "Prev  <--  --> ENTER"
set_pan_start_state	; 0x05
		call	elev_home
		movlw	pan_start_state
		goto	set_state_exit
		;
set_pan_next_state
		btfsc	prog_type,PROG_SLO_PAN_FLG
		 goto	set_pan_final_state
		;	PAN_STEP state
		;	pan_step_txt		; " Pan Step Size=     "
		;	pan_button_txt		; "Prev  <--  --> ENTER"
set_pan_step_state	; 0x06
		movlw	pan_step_state
		goto	set_state_exit
		;
		;	PAN_STEPCT state
		;	pan_stepct_txt		; " Pan #of steps=     "
		;	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
set_pan_stepct_state	; 0x07
		movlw	pan_stepct_state
		goto	set_state_exit
		;
		;	SETTLE state
		;	settle_txt			; " Settling Time=     "
		;	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
; set_settle_state	; 0x08
;		movlw	settle_state
;		goto	set_state_exit
		;
		;	HOLD state
		;	holding_txt			; "  Holding Time=     "
		;	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
set_hold_state		; 0x09
		call	home_position
		movlw	hold_state
		goto	set_state_exit
		;
		;	GET state
		;	get_prog_txt		; "      Get Prog#     "
		;	load_button_txt		; "Prev  !dn  up^  LOAD"
set_get_state		; 0x0A
		movlw	get_state
		goto	set_state_exit
		;
		;	SAVE state
		;	save_as_txt			; "  Save as Prog#    ?"
		;	save_button_txt		; "EXIT  !dn  up^  SAVE"
set_save_state		; 0x0B
		movlw	save_state
		goto	set_state_exit
		;
		;	CONFIRM state
		;	replace_txt			; "Replace Current Prog"
		;	yes_no_button_txt	; "Prev  ---  NO   YES "
set_confirm_state	; 0x0C
		movlw	confirm_state
		goto	set_state_exit
		;
		;	PAN_FINAL state
		;	pan_final_txt		; " Pan Final Pos=     "
		;	pan_button_txt		; "Prev  <--  --> ENTER"
set_pan_final_state	; 0x0D
		movlw	pan_final_state
		goto	set_state_exit
		;
		;	PAN_START state
		;	pan_start_txt		; "Total Pan Time=     "
		;	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
set_pan_time_state	; 0x0E
		movlw	pan_time_state
		goto	set_state_exit
		;
		;	ERROR state
		;	error_txt			; "  EXIT ERROR STATE  "
		;	yes_button_txt		; " ---  ---  ---  YES "
set_error_state		; 0x0F
		movlw	error_state
		goto	set_state_exit
		;
		;
;		*******************************************************************
;		***  Save new state, output new screen, wait for button release ***
;		*******************************************************************
set_state_exit
		movwf	sys_state				; save new state
		movlw	HIGH set_state_text_p2	; set up PC high bits
		movwf	PCLATH
		call	set_state_text_p2		; output prompt text
			;
button_release
;		movlw	HIGH read_sw_state	; set up PC high bits
;		movwf	PCLATH
		btfss	keypad_lock,6
		 goto	 button_rel_1
		clrf	sw_state
		bcf		keypad_lock,6
		goto	main_loop
button_rel_1
		call	read_sw_state
		btfss	STATUS,Z		; switch released yet ?
		 goto	button_rel_1	; no, wait for release
		goto	main_loop
				;
				;
				; ******************************************
				; ******************************************
				; *****  ACTIONS for PUSHBUTTON Calls  *****
				; ******************************************
				; ******************************************
				;
				; **********************************************
				; **********************************************
				; ** manage /display elev_start[h:l] variable **
				; **********************************************
				; **********************************************
				;
				; ***********************************************
				; ** increment elev_start_variable and step up **
				; ***********************************************
elev_start_increment
		movlw	JOG_DELAY		; set delay to jog
		movwf	timeout
		movlw	JOG_COUNT_VAL	; set count to fast jog
		movwf	jog_count
e_start_incr_1
		movlw	#1
		bcf		STATUS,C
		addwf	elev_start_l,f
		btfsc	STATUS,C
		 incf	elev_start_h,f
		call	output_elev_start
		movlw	0						; start position is zero steps
		call	set_elev_step_position	; set elevation motor to start position
				;
e_start_incr_2
		call	read_sw_state
		btfsc	STATUS,Z		; switch released yet ?
		 goto	main_loop		; yes, return to main loop
								;
		movf	timeout,w		; test timeout
		btfss	STATUS,Z		; has delay timed out
		 goto	e_start_incr_2	; no, wait for timeout
								;
		movf	jog_count,f		; high speed jog, yet
		btfsc	STATUS,Z		; has low speed count expired ?
		 goto	e_start_incr_3	; yes, do fast jog
		decf	jog_count,f		; no, count off steps to fast
								;
		movlw	STEP_DELAY1		; set jog repetition delay
		movwf	timeout
		goto	e_start_incr_1
e_start_incr_3
		movlw	STEP_DELAY2		; set fast jog repetition delay
		movwf	timeout
		goto	e_start_incr_1
				;
				; *************************************************
				; ** decrement elev_start_variable and step down **
				; *************************************************
elev_start_decrement
		movlw	JOG_DELAY		; set delay to jog
		movwf	timeout
		movlw	JOG_COUNT_VAL	; set count to fast jog
		movwf	jog_count
e_start_decr_1
		movlw	#1
		bsf		STATUS,C
		subwf	elev_start_l,f
		btfss	STATUS,C
		 decf	elev_start_h,f
		call	output_elev_start
		movlw	0						; start position is zero steps
		call	set_elev_step_position	; set elevation motor to start position
				;
e_start_decr_2
		call	read_sw_state
		btfsc	STATUS,Z		; switch released yet ?
		 goto	main_loop		; yes, return to main loop
								;
		movf	timeout,w		; test timeout
		btfss	STATUS,Z		; has delay timed out
		 goto	e_start_decr_2	; no, wait for timeout
								;
		movf	jog_count,f		; high speed jog, yet
		btfsc	STATUS,Z		; has low speed count expired ?
		 goto	e_start_decr_3	; yes, do fast jog
		decf	jog_count,f		; no, count off steps to fast
								;
		movlw	STEP_DELAY1		; set jog repetition delay
		movwf	timeout
		goto	e_start_decr_1
e_start_decr_3
		movlw	STEP_DELAY2		; set fast jog repetition delay
		movwf	timeout
		goto	e_start_decr_1
				;
				; ******************************************************
				; ** output elev_start value as 16-bit signed decimal **
				; ******************************************************
output_elev_start
	IF	DISP24
		movlw	0x91			; move cursor to display location
	ELSE
		movlw	0x8F			; move cursor to display location
	ENDIF
		call	SEND_CMD
		movf	elev_start_h,w	; put value in minuend
		movwf	minuend_h
		movf	elev_start_l,w
		movwf	minuend_l
		goto	OUTSD16
				;
				; *********************************************
				; *********************************************
				; ** manage /display elev_step[h:l] variable **
				; *********************************************
				; *********************************************
				;
				; **********************************************
				; ** increment elev_step_variable and step up **
				; **********************************************
elev_step_increment
		movlw	JOG_DELAY		; set delay to jog
		movwf	timeout
		movlw	JOG_COUNT_VAL	; set count to fast jog
		movwf	jog_count
e_step_incr_1
		movlw	#1
		bcf		STATUS,C
		addwf	elev_step_l,f
		btfsc	STATUS,C
		 incf	elev_step_h,f
		call	output_elev_step
		movlw	1						; position for first step
		call	set_elev_step_position	; set elevation motor to start position
				;
e_step_incr_2
		call	read_sw_state
		btfsc	STATUS,Z		; switch released yet ?
		 goto	main_loop		; yes, return to main loop
								;
		movf	timeout,w		; test timeout
		btfss	STATUS,Z		; has delay timed out
		 goto	e_step_incr_2	; no, wait for timeout
								;
		movf	jog_count,f		; high speed jog, yet
		btfsc	STATUS,Z		; has low speed count expired ?
		 goto	e_step_incr_3	; yes, do fast jog
		decf	jog_count,f		; no, count off steps to fast
								;
		movlw	STEP_DELAY1		; set jog repetition delay
		movwf	timeout
		goto	e_step_incr_1
e_step_incr_3
		movlw	STEP_DELAY2		; set fast jog repetition delay
		movwf	timeout
		goto	e_step_incr_1
				;
				; ************************************************
				; ** decrement elev_step_variable and step down **
				; ************************************************
elev_step_decrement
		movlw	JOG_DELAY		; set delay to jog
		movwf	timeout
		movlw	JOG_COUNT_VAL	; set count to fast jog
		movwf	jog_count
e_step_decr_1
		movlw	#1
		bsf		STATUS,C
		subwf	elev_step_l,f
		btfss	STATUS,C
		 decf	elev_step_h,f
		call	output_elev_step
		movlw	1						; position for first step
		call	set_elev_step_position	; set elevation motor to sstep position
e_step_decr_2
		call	read_sw_state
		btfsc	STATUS,Z		; switch released yet ?
		 goto	main_loop		; yes, return to main loop
								;
		movf	timeout,w		; test timeout
		btfss	STATUS,Z		; has delay timed out
		 goto	e_step_decr_2	; no, wait for timeout
								;
		movf	jog_count,f		; high speed jog, yet
		btfsc	STATUS,Z		; has low speed count expired ?
		 goto	e_step_decr_3	; yes, do fast jog
		decf	jog_count,f		; no, count off steps to fast
								;
		movlw	STEP_DELAY1		; set jog repetition delay
		movwf	timeout
		goto	e_step_decr_1
e_step_decr_3
		movlw	STEP_DELAY2		; set fast jog repetition delay
		movwf	timeout
		goto	e_step_decr_1
				;
				; *****************************************************
				; ** output elev_step value as 16-bit signed decimal **
				; *****************************************************
output_elev_step
	IF	DISP24
		movlw	0x91			; move cursor to display location
	ELSE
		movlw	0x8F			; move cursor to display location
	ENDIF
		call	SEND_CMD
		movf	elev_step_h,w	; put value in minuend
		movwf	minuend_h
		movf	elev_step_l,w
		movwf	minuend_l
		goto	OUTSD16
				;
				; ******************************************
				; ******************************************
				; ** manage /display elev_stepct variable **
				; ******************************************
				; ******************************************
				;
				; ************************************************
				; ** increment elev_stepct_variable and step up **
				; ************************************************
elev_stepct_increment
		movlw	MAX_ELEV_STEPCT		; test for maximum
		subwf	elev_stepct,w
		btfss	STATUS,C
		 incf	elev_stepct,f		; if <MAX then increment
		call	output_elev_stepct	; display it
				;
		movf	elev_stepct,w
		call	set_elev_step_position
				;
		goto	button_release
				;
				; **************************************************
				; ** decrement elev_stepct_variable and step down **
				; **************************************************
elev_stepct_decrement
		movf	elev_stepct,f		; test for zero
		btfss	STATUS,Z
		 decf	elev_stepct,f		; if >0 then decrement
		call	output_elev_stepct	; display it
				;
		movf	elev_stepct,w
		call	set_elev_step_position
				;
		goto	button_release
				;
				; *******************************************************
				; ** output elev_stepct value as 8-bit unsigned decimal **
				; *******************************************************
output_elev_stepct
	IF	DISP24
		movlw	0x92			; move cursor to display location
	ELSE
		movlw	0x8F			; move cursor to display location
	ENDIF
		call	SEND_CMD
		clrf	BCflag
		bsf		BCflag,6		; turn on leading-zero supression
		bsf		BCflag,3		; two digit output
		movf	elev_stepct,w	; get step count
		addlw	0x01			; already at first step
		goto	OUTD8B			; display as unsigned decimal
				;
				; *********************************************
				; *********************************************
				; ** manage /display pan_start[h:l] variable **
				; *********************************************
				; *********************************************
				;
				; **********************************************
				; ** increment pan_start_variable and step CW **
				; **********************************************
pan_start_increment
		movlw	JOG_DELAY		; set delay to jog
		movwf	timeout
		movlw	JOG_COUNT_VAL	; set count to fast jog
		movwf	jog_count
p_start_incr_1
		movlw	#1
		bcf		STATUS,C
		addwf	pan_start_l,f
		btfsc	STATUS,C
		 incf	pan_start_h,f
		call	output_pan_start
		movlw	0						; start position is zero steps
		call	set_pan_step_position	; set pan motor to start position
				;
p_start_incr_2
		call	read_sw_state
		btfsc	STATUS,Z		; switch released yet ?
		 goto	main_loop		; yes, return to main loop
								;
		movf	timeout,w		; test timeout
		btfss	STATUS,Z		; has delay timed out
		 goto	p_start_incr_2	; no, wait for timeout
								;
		movf	jog_count,f		; high speed jog, yet
		btfsc	STATUS,Z		; has low speed count expired ?
		 goto	p_start_incr_3	; yes, do fast jog
		decf	jog_count,f		; no, count off steps to fast
								;
		movlw	STEP_DELAY1		; set step repetition delay
		movwf	timeout
		goto	p_start_incr_1
p_start_incr_3
		movlw	STEP_DELAY2		; set fast step repetition delay
		movwf	timeout
		goto	p_start_incr_1
				;
				; **********************************************
				; ** increment pan_start_variable and step CW **
				; **********************************************
pan_start_decrement
		movlw	JOG_DELAY		; set delay to jog
		movwf	timeout
		movlw	JOG_COUNT_VAL	; set count to fast jog
		movwf	jog_count
p_start_decr_1
		movlw	#1
		bsf		STATUS,C
		subwf	pan_start_l,f
		btfss	STATUS,C
		 decf	pan_start_h,f
		call	output_pan_start
		movlw	0						; start position is zero steps
		call	set_pan_step_position	; set pan motor to start position
				;
p_start_decr_2
		call	read_sw_state
		btfsc	STATUS,Z		; switch released yet ?
		 goto	main_loop		; yes, return to main loop
								;
		movf	timeout,w		; test timeout
		btfss	STATUS,Z		; has delay timed out
		 goto	p_start_decr_2	; no, wait for timeout
								;
		movf	jog_count,f		; high speed jog, yet
		btfsc	STATUS,Z		; has low speed count expired ?
		 goto	p_start_decr_3	; yes, do fast jog
		decf	jog_count,f		; no, count off steps to fast
								;
		movlw	STEP_DELAY1		; set jog repetition delay
		movwf	timeout
		goto	p_start_decr_1
p_start_decr_3
		movlw	STEP_DELAY2		; set fast jog repetition delay
		movwf	timeout
		goto	p_start_decr_1
				;
				; *****************************************************
				; ** output pan_start value as 16-bit signed decimal **
				; *****************************************************
output_pan_start
	IF	DISP24
		movlw	0x91			; move cursor to display location
	ELSE
		movlw	0x8F			; move cursor to display location
	ENDIF
		call	SEND_CMD
		movf	pan_start_h,w	; put value in minuend
		movwf	minuend_h
		movf	pan_start_l,w
		movwf	minuend_l
		goto	OUTSD16
				;
				; ********************************************
				; ********************************************
				; ** manage /display pan_step[h:l] variable **
				; ********************************************
				; ********************************************
				;
				; **********************************************
				; ** increment pan_step_variable and step CW **
				; **********************************************
pan_step_increment
		movlw	JOG_DELAY		; set delay to jog
		movwf	timeout
		movlw	JOG_COUNT_VAL	; set count to fast jog
		movwf	jog_count
p_step_incr_1
		movlw	#1
		bcf		STATUS,C
		addwf	pan_step_l,f
		btfsc	STATUS,C
		 incf	pan_step_h,f
		call	output_pan_step
		movlw	1						; position for first step
		call	set_pan_step_position	; set pan motor to step position
				;
p_step_incr_2
		call	read_sw_state
		btfsc	STATUS,Z		; switch released yet ?
		 goto	main_loop		; yes, return to main loop
								;
		movf	timeout,w		; test timeout
		btfss	STATUS,Z		; has delay timed out
		 goto	p_step_incr_2	; no, wait for timeout
								;
		movf	jog_count,f		; high speed jog, yet
		btfsc	STATUS,Z		; has low speed count expired ?
		 goto	p_step_incr_3	; yes, do fast jog
		decf	jog_count,f		; no, count off steps to fast
								;
		movlw	STEP_DELAY1		; set step repetition delay
		movwf	timeout
		goto	p_step_incr_1
p_step_incr_3
		movlw	STEP_DELAY2		; set fast step repetition delay
		movwf	timeout
		goto	p_step_incr_1
				;
				; **********************************************
				; ** decrement pan_step_variable and step CW **
				; **********************************************
pan_step_decrement
		movlw	JOG_DELAY		; set delay to jog
		movwf	timeout
		movlw	JOG_COUNT_VAL	; set count to fast jog
		movwf	jog_count
p_step_decr_1
		movlw	#1
		bsf		STATUS,C
		subwf	pan_step_l,f
		btfss	STATUS,C
		 decf	pan_step_h,f
		call	output_pan_step
		movlw	1						; position for first step
		call	set_pan_step_position	; set pan motor to step position
p_step_decr_2
		call	read_sw_state
		btfsc	STATUS,Z		; switch released yet ?
		 goto	main_loop		; yes, return to main loop
								;
		movf	timeout,w		; test timeout
		btfss	STATUS,Z		; has delay timed out
		 goto	p_step_decr_2	; no, wait for timeout
								;
		movf	jog_count,f		; high speed jog, yet
		btfsc	STATUS,Z		; has low speed count expired ?
		 goto	p_step_decr_3	; yes, do fast jog
		decf	jog_count,f		; no, count off steps to fast
								;
		movlw	STEP_DELAY1		; set step repetition delay
		movwf	timeout
		goto	p_step_decr_1
p_step_decr_3
		movlw	STEP_DELAY2		; set fast step repetition delay
		movwf	timeout
		goto	p_step_decr_1
				;
				; ****************************************************
				; ** output pan_step value as 16-bit signed decimal **
				; ****************************************************
output_pan_step
	IF	DISP24
		movlw	0x91			; move cursor to display location
	ELSE
		movlw	0x8F			; move cursor to display location
	ENDIF
		call	SEND_CMD
		movf	pan_step_h,w	; put value in minuend
		movwf	minuend_h
		movf	pan_step_l,w
		movwf	minuend_l
		goto	OUTSD16
				;
				; *****************************************
				; *****************************************
				; ** manage /display pan_stepct_variable **
				; *****************************************
				; *****************************************
				;
				; ***********************************************
				; ** step CW and incr/decr pan_stepct_variable **
				; ***********************************************
pan_stepct_cw
		btfsc	pan_step_h,7			; test direction of step_size
		 goto	 pan_stepct_decrement	; reverse sense if negative
				;
				; ********************************************************
				; ** increment pan_stepct_variable and step to position **
				; ********************************************************
pan_stepct_increment
		movlw	MAX_PAN_STEPCT		; test for maximum
		subwf	pan_stepct,w
		btfss	STATUS,C
		 incf	pan_stepct,f		; if <MAX then increment
		call	output_pan_stepct	; display it
				;
		movf	pan_stepct,w
		call	set_pan_step_position
				;
		goto	button_release
				;
				; ************************************************
				; ** step CCW and incr/decr pan_stepct_variable **
				; ************************************************
pan_stepct_ccw
		btfsc	pan_step_h,7			; test direction of step_size
		 goto	 pan_stepct_increment	; reverse sense if negative
				;
				; ********************************************************
				; ** decrement pan_stepct_variable and step to position **
				; ********************************************************
pan_stepct_decrement
		movf	pan_stepct,f		; test for zero
		btfss	STATUS,Z
		 decf	pan_stepct,f		; if >0 then decrement
		call	output_pan_stepct	; display it
				;
		movf	pan_stepct,w
		call	set_pan_step_position
				;
		goto	button_release
				;
				; *******************************************************
				; ** output pan_stepct value as 8-bit unsigned decimal **
				; *******************************************************
output_pan_stepct
	IF	DISP24
		movlw	0x92			; move cursor to display location
	ELSE
		movlw	0x8F			; move cursor to display location
	ENDIF
		call	SEND_CMD
		clrf	BCflag
		bsf		BCflag,6		; turn on leading-zero supression
		bsf		BCflag,3		; two digit output
		movf	pan_stepct,w	; get step count
		addlw	0x01			; already at first step
		goto	OUTD8B			; display as unsigned decimal
				;
				; ******************************************
				; ** increment Settle-time before shutter **
				; ******************************************
settle_increment
;		movlw	JOG_DELAY		; set delay time to jog
;		movwf	timeout
; settle_incr_1
;		movlw	MAX_SETTLE_TIME		; test for maximum
;		subwf	settle_time,w
;		btfss	STATUS,C
;		 incf	settle_time,f		; if <MAX then increment
;		call	output_settle_time	; display it
; settle_incr_2
;		call	read_sw_state
;		btfsc	STATUS,Z		; switch released yet ?
		 goto	main_loop		; yes, return to main loop
								;
;		movf	timeout,w		; test timeout
;		btfss	STATUS,Z		; has delay timed out
;		 goto	settle_incr_2	; no, wait for timeout
								;
;		movlw	STEP_DELAY1		; set step repetition delay
;		movwf	timeout
;		goto	settle_incr_1
				;
				; ******************************************
				; ** decrement Settle-time before shutter **
				; ******************************************
settle_decrement
;		movlw	JOG_DELAY		; set delay time to jog
;		movwf	timeout
; settle_decr_1
;		movf	settle_time,f		; test for zero
;		btfss	STATUS,Z
;		 decf	settle_time,f		; if >0 then decrement
;		call	output_settle_time	; display it
; settle_decr_2
;		call	read_sw_state
;		btfsc	STATUS,Z		; switch released yet ?
		 goto	main_loop		; yes, return to main loop
								;
;		movf	timeout,w		; test timeout
;		btfss	STATUS,Z		; has delay timed out
;		 goto	settle_decr_2	; no, wait for timeout
								;
;		movlw	STEP_DELAY1		; set step repetition delay
;		movwf	timeout
;		goto	settle_decr_1
				;
				; ************************
				; ** output settle-time **
				; ************************
output_settle_time
	return
;		movlw	0x8F			; move cursor to display location
;		call	SEND_CMD
;		clrf	BCflag
;		bsf		BCflag,5		; set up decimal point
;		movf	settle_time,w	; get settle-time
;		call	OUTD8B			; display as unsigned decimal
;		movlw	's'
;		goto	OUTCH
				;
				; ***************************************
				; ** increment Hold-time after shutter **
				; ***************************************
hold_increment
		movlw	JOG_DELAY		; set delay time to jog
		movwf	timeout
hold_incr_1
		movlw	MAX_HOLD_TIME		; test for maximum
		subwf	hold_time,w
		btfss	STATUS,C
		 incf	hold_time,f			; if <MAX then increment
		call	output_hold_time	; display it
hold_incr_2
		call	read_sw_state
		btfsc	STATUS,Z		; switch released yet ?
		 goto	main_loop		; yes, return to main loop
								;
		movf	timeout,w		; test timeout
		btfss	STATUS,Z		; has delay timed out
		 goto	hold_incr_2		; no, wait for timeout
								;
		movlw	STEP_DELAY1		; set step repetition delay
		movwf	timeout
		goto	hold_incr_1
				;
				; ***************************************
				; ** increment Hold-time after shutter **
				; ***************************************
hold_decrement
		movlw	JOG_DELAY		; set delay time to jog
		movwf	timeout
hold_decr_1
		movf	hold_time,f			; test for zero
		btfss	STATUS,Z
		 decf	hold_time,f			; if >0 then decrement
		call	output_hold_time	; display it
hold_decr_2
		call	read_sw_state
		btfsc	STATUS,Z		; switch released yet ?
		 goto	main_loop		; yes, return to main loop
								;
		movf	timeout,w		; test timeout
		btfss	STATUS,Z		; has delay timed out
		 goto	hold_decr_2	; no, wait for timeout
								;
		movlw	STEP_DELAY1		; set step repetition delay
		movwf	timeout
		goto	hold_decr_1
				;
				; ************************
				; ** output hold-time **
				; ************************
output_hold_time
	IF	DISP24
		movlw	0x91			; move cursor to display location
	ELSE
		movlw	0x8F			; move cursor to display location
	ENDIF
		call	SEND_CMD
		clrf	BCflag
		bsf		BCflag,6		; leading zero supress
		bsf		BCflag,4		; set up decimal point XX.X
		movf	hold_time,w		; get settle-time
		call	OUTD8B			; display as unsigned decimal
		movlw	's'
		goto	OUTCH
				;
				; ******************************
				; ** increment Program number **
				; ******************************
prognum_increment
		movlw	MAX_PROG_NUM		; test for maximum
		subwf	prog_num,w
		btfss	STATUS,C
		 incf	prog_num,f			; if <MAX then increment
		call	output_prog_num		; display it
		goto	button_release
				;
				; ******************************
				; ** decrement Program number **
				; ******************************
prognum_decrement
		movf	prog_num,f			; test for zero
		btfss	STATUS,Z
		 decf	prog_num,f			; if >0 then decrement
		call	output_prog_num		; display it
		goto	button_release
				;
				; ***************************
				; ** output Program_number **
				; ***************************
output_prog_num
	IF	DISP24
		movlw	0x91			; move cursor to display location
	ELSE
		movlw	0x8F			; move cursor to display location
	ENDIF
		call	SEND_CMD
		clrf	BCflag
		bsf		BCflag,6		; turn on leading-zero supression
		bsf		BCflag,3		; two digit output
		movf	prog_num,w		; get program_number
		addlw	0x01			; numbers from 1 to N
		goto	OUTD8B			; display as unsigned decimal
				;
				; *********************************
				; *********************************
				; ** page_0 part of system setup **
				; *********************************
				; *********************************
sys_setup
		movlw	HIGH sys_setup_p1		; set up PC high bits
		movwf	PCLATH
		goto	sys_setup_p1
;		movlw	HIGH main		; set up PC high bits
;		movwf	PCLATH
;		goto	main2
		;
		;
		; ********************
		; ** generic_output **
		; ********************
		; *******************************************************
		; *  param_mode                                         *
		; * |    d7   |   d6   | d5  | d4 | d3 | d2 | d1 | d0 | *
		; * | supress | 16-bit | 10s | s  | ms | x  | x  | x  | *
		; *******************************************************
generic_output
		btfsc	param_mode,7	; surpress display ?
		 return					; yes, return
				;
	IF	DISP24
		movlw	0x91			; move cursor to display location
	ELSE
		movlw	0x8F			; move cursor to display location
	ENDIF
		call	SEND_CMD
								;
		btfss	param_mode,6	; output 16-bit
		 goto	 generic_out_0
				;
				; output 16-bit as "xxxxx"
		movf	INDF,w		; get value at FSR
		movwf	minuend_h	; put in high byte
		incf	FSR,f		; bump FSR
		movf	INDF,w		; get value at FSR+1
		movwf	minuend_l	; put in low byte
		decf	FSR,f		; restore FSR
		goto	OUTSD16
		return
				;
				; 8 bit numeric displays
		; ****************************************************
		; *  BCflag  -  format control                       *
		; *  | x | zero-supress | dp1 | dp2 | 2-dig | xxx |  *
		; ****************************************************
generic_out_0
		btfss	param_mode,5	; output tens seconds ?
		 goto	 generic_out_1
				;
				; output 8bit as "xx.x s"
		clrf	BCflag
		bsf		BCflag,4		; set up decimal point
		movf	INDF,w			; get variable
		call	OUTD8B			; display as unsigned decimal
		movlw	's'
		call	OUTCH
		return
								;
generic_out_1
		btfss	param_mode,4	; output seconds ?
		 goto	 generic_out_2
				;
				; output 8bit as "x.xx s"
		clrf	BCflag
		bsf		BCflag,5		; set up decimal point
		movf	INDF,w			; get variable
		call	OUTD8B			; display as unsigned decimal
		movlw	's'
		call	OUTCH
		return
								;
generic_out_2
		btfss	param_mode,3
		 goto	 generic_out_3
				;
				; output 8-bit as "xxx ms"								
		clrf	BCflag
		bsf		BCflag,6		; no DP but zero supress
		movf	INDF,w			; get variable
		call	OUTD8B			; display as unsigned integer
		movlw	'm'
		call	OUTCH
		movlw	's'
		call	OUTCH
		return
								;
generic_out_3
				;
				; simple 8 bit decimal display
		clrf	BCflag
		bsf		BCflag,6		; no DP but zero supress
		movf	INDF,w			; get variable
		call	OUTD8B			; display as unsigned integer
		return
				;
				;
				;
				;
				; ****************************************
				; ****************************************
				; *****  RUN CURRENT MOTION PROGRAM  *****
				; ****************************************
				; ****************************************
				;
				; ****************************************
				; *  drive both motors to home position  *
				; ****************************************
home_position
		call	elev_home
		call	pan_home
		return
				;
elev_home
		clrf	new_pos_h
		clrf	new_pos_l
		call	elev_new_position
		return
				;
pan_home
		movf	pan_slow_rate,w		; set pan slew rate
		movwf	pan_rate_l			; put in rate lsb
		clrf	pan_rate_h
		bsf		prog_run_flags,SPEED_RAMP_EN
		bsf		prog_run_flags,RAMP_UP_FLAG
		call	pan_new_position
		return
				;
				; *************************
				; *  run current program  *
				; *************************
				;	prog_run_flags
				;	HALT_FLG			EQU	7	; set if HALT is detected
				;	SINGLE_STEP_FLG		EQU	6	; set when single-stepping
				;	POSITION_DISP_EN	EQU	5	; set to display position in slow-pan
				;	SPEED_RAMP_EN		EQU	4	; set when Ramp-up/down is enabled
				;	RAMP_UP_FLAG		EQU	3	; set at beginning of move for ramp-up
				;
				;	prog_type
				;	PROG_2D_FLG			EQU	2
				;	PROG_1D_FLG			EQU	1
				;	PROG_SLO_PAN_FLG	EQU	0
		;
run_current_program
		movlw	high running_program_txt	; set up 2nd page
		movwf	PCLATH
		call	running_program_txt			;  "Running Current Prog"
		movlw	high run_current_program	; reset main page	
		movwf	PCLATH
				;
		bcf		prog_run_flags,SINGLE_STEP_FLG
		goto	run_program
				;
				; *********************************
				; *  Single-step current program  *
				; *********************************
step_current_program
		movlw	high stepping_program_txt
		movwf	PCLATH
		call	stepping_program_txt  ;  " Stepping Thru Prog "
		movlw	high step_current_program	; reset main page	
		movwf	PCLATH
				;
		bcf		prog_run_flags,SINGLE_STEP_FLG
		btfss	prog_type,PROG_SLO_PAN_FLG
		 bsf	 prog_run_flags,SINGLE_STEP_FLG
		goto	run_program
				;
current_program_exit
		bcf		prog_run_flags,HALT_EN		; disable halt
		bcf		prog_run_flags,HALT_FLG
		call	home_position
		goto	set_run_state
				;
run_program
		bcf		prog_run_flags,HALT_FLG
		bsf		prog_run_flags,HALT_EN		; enable halt
				;
		btfsc	prog_type,PROG_2D_FLG
		 goto	run_program_2d		; spherical stepping
				;
		btfsc	prog_type,PROG_1D_FLG
		 goto	run_program_1d		; pan stepping
				;
		btfsc	prog_type,PROG_SLO_PAN_FLG
		 goto	run_program_slo_pan		; smooth slow pan
				;
		movlw	high no_program_txt
		movwf	PCLATH
		call	no_program_txt		;  " No Program     EXIT"
		movlw	high run_program	; reset main page	
		movwf	PCLATH
				;
no_prog_1
		call	read_sw_state
		btfss	STATUS,Z
		 goto	no_prog_1
no_prog_2				;
		call	read_sw_state
		btfss	sw_state,SWITCH_4	; "EXIT" switch
		 goto	no_prog_2
		goto	set_run_state
				;
				; *********************************************
				; *****  Run continuous slow pan program  *****
				; *********************************************
run_program_slo_pan
		movlw	high slow_prog_run_initial_txt
		movwf	PCLATH
		call	slow_prog_run_initial_txt	; "  Slow Pan Running"	
		movlw	high run_program_slo_pan	; reset main page	
		movwf	PCLATH
											;
		movf	pan_slow_rate,w		; set pan start slow rate
		movwf	pan_rate_l			; put in rate lsb
		clrf	pan_rate_h
		bsf		prog_run_flags,SPEED_RAMP_EN
		bsf		prog_run_flags,RAMP_UP_FLAG
				;
		movlw	0							; go to pan start position
		call	set_pan_step_position
				;
		movlw	high slow_prog_start_prompt_txt
		movwf	PCLATH
		call	slow_prog_start_prompt_txt	; "  Slow Pan    START"	
		movlw	high run_program_slo_pan	; reset main page	
		movwf	PCLATH
				;
		call	calc_slo_pan_rate	
		call	get_step_switch				; wait for start
				;
		movlw	high slow_prog_running_prompt_txt
		movwf	PCLATH
		call	slow_prog_running_prompt_txt  ; "  Slow Pan Running"	
		movlw	high run_program_slo_pan	; reset main page	
		movwf	PCLATH
				;
		bcf		prog_run_flags,SPEED_RAMP_EN
		bcf		prog_run_flags,RAMP_UP_FLAG
		bsf		prog_run_flags,POSITION_DISP_EN	; display current pan position
		movlw	1						; go slowly to step 1 (final) position
		call	set_pan_step_pos_1

		btfsc	prog_run_flags,HALT_FLG
		 goto	 current_program_exit
				;
		bcf		prog_run_flags,POSITION_DISP_EN	; turn off pan position display
		movf	pan_fast_rate,w			; restore fast pan slew rate
		movwf	pan_rate_l
		clrf	pan_rate_h
				;
		movlw	high prog_exit_prompt_txt
		movwf	PCLATH
		call	prog_exit_prompt_txt		; "            EXIT"	
		movlw	high run_program_slo_pan		; reset main page	
		movwf	PCLATH
				;
		call	get_step_switch					; wait for exit
;		goto	set_run_state

		goto	current_program_exit
				;
				; ***************************************
				; * calculate time per step for slow pan*
				; ***************************************
                ; *     rate = total_time / #_steps     *
				; ***************************************
calc_slo_pan_rate
		clrf	pan_rate_h
		clrf	pan_rate_l
		clrf	tempc
		movf	hold_time,w		; actually "Total Pan Time=XX.X"
		movwf	tempa
calc_slopan_1					; multiply time by 1000 to get milliseconds
		movf	tempa,f
		btfsc	STATUS,Z
		 goto	calc_slopan_2					
		movlw	low THOUSAND
		addwf	pan_rate_l,f
		btfss	STATUS,C
		 goto	calc_slopan_1a
		movlw	#1				; if carry was set, add 1 to MSB
		addwf	pan_rate_h,f
		btfsc	STATUS,C
		 incf	tempc,f
calc_slopan_1a
		movlw	high THOUSAND
		addwf	pan_rate_h,f
		btfsc	STATUS,C
		 incf	tempc,f
		decfsz	tempa,f
		 goto	calc_slopan_1
				;				; Pan_rate[h:l] = total time x1000
calc_slopan_2
		clrf	tempa
		clrf	tempb
				;				; divide by step count by successive subtraction
calc_slopan_3
		movf	pan_step_l,w
		subwf	pan_rate_l,f
		btfsc	STATUS,C
		 goto	 calc_slopan_3a
		movlw	#1				; if borrow, subtract 1 from high byte
		subwf	pan_rate_h,f
		btfss	STATUS,C
		 decf	tempc,f
calc_slopan_3a
		movf	pan_step_h,w
		subwf	pan_rate_h,f
		btfss	STATUS,C
		 decf	tempc,f
		btfsc	tempc,7		; test for going negative
		 goto	calc_slopan_4
		incfsz	tempa,f
		 goto	calc_slopan_3
		incf	tempb,f
		goto	calc_slopan_3
				;
calc_slopan_4
		movf	tempb,w
		movwf	pan_rate_h
		movf	tempa,w
		movwf	pan_rate_l
				;
		return
				;
				; ************************************
				; *****  Run current 2d program  *****
				; ************************************
run_program_2d
		movlw	high run_2d_step_txt
		movwf	PCLATH
		call	run_2d_step_txt			; "HALT Elev=xx Pan=xx     "
		movlw	high run_program_2d		; reset main page	
		movwf	PCLATH
		bcf		prog_run_flags,HALT_FLG
		bsf		prog_run_flags,HALT_EN		; enble halt
				;
run_prog_2d
		movf	elev_stepct,w
		movwf	e_step_count
		clrf	e_cur_step
				;
run_pgm_2d_1
		btfsc	e_step_count,7		; elevation step underflow
		 goto	 current_program_exit
				;
		call	output_elev_stepnum
		movf	e_cur_step,w
		call	set_elev_step_position
				;
		movf	settle_time,w
		movwf	timeout
				;
run_pgm_2d_2
		movf	timeout,f
		btfss	STATUS,Z
		 goto	run_pgm_2d_2
				;
		movf	pan_stepct,w
		movwf	p_step_count
		clrf	p_cur_step
				;
run_pgm_2d_3
		btfsc	p_step_count,7		; pan step underflow
		 goto	run_pgm_2d_8
				;
		call	output_pan_stepnum
		movf	p_cur_step,w
		call	set_pan_step_position
				;
		btfss	prog_run_flags,HALT_FLG	; did program get halted
		 goto	 run_pgm_2d_3a		; no, keep going
		call	do_halt_prompt
		btfsc	prog_run_flags,HALT_FLG	; was it an abort ?
		 goto	 current_program_exit	; abort
		goto	run_pgm_2d_3		; no, finish previous motion
				;
run_pgm_2d_3a
		movf	settle_time,w		; settle time in 100ths of seconds
		movwf	timeout
				;
run_pgm_2d_4
		movf	timeout,f
		btfss	STATUS,Z
		 goto	run_pgm_2d_4
				;
		call	press_shutter
				;
		btfsc	prog_run_flags,SINGLE_STEP_FLG
		 goto	run_pgm_2d_7		; if single stepping no need for hold time
				;
		movf	hold_time,w			; hold time in hundredths of a second
		movwf	temp_ctr				; save in count-down
run_pgm_2d_5
		movf	temp_ctr,f 			; test hold time countdown
		btfsc	STATUS,Z
		 goto	run_pgm_2d_7		; if zero, do next step
				;
		movlw	D'10'				; count 10 mS for each hold count
		movwf	timeout
run_pgm_2d_6
		movf	timeout,f
		btfss	STATUS,Z
		 goto	run_pgm_2d_6
				;
		decf	temp_ctr,f
		goto	run_pgm_2d_5
									;
run_pgm_2d_7						; test single-step mode
		btfsc	prog_run_flags,SINGLE_STEP_FLG
		 call	get_step_switch
		incf	p_cur_step,f
		decf	p_step_count,f
		goto	run_pgm_2d_3
;
run_pgm_2d_8
		incf	e_cur_step,f
		decf	e_step_count,f
		goto	run_pgm_2d_1
				;             
				; ****************************************
				; *****  Run current 1-axis program  *****
				; ****************************************
run_program_1d
		movlw	high run_1d_step_txt
		movwf	PCLATH
		call	run_1d_step_txt			; "HALT         Pan=xx     "
		movlw	high run_program_1d		; reset main page	
		movwf	PCLATH
		bcf		prog_run_flags,HALT_FLG
		bsf		prog_run_flags,HALT_EN		; enble halt
				;
;		call	run_prog_1d
;	goto	run_prog_1d
				;
;		return
;		call	home_position			; return to home position
				;
;		goto	set_run_state
				;
run_prog_1d
		movf	pan_stepct,w
		movwf	p_step_count
		clrf	p_cur_step
				;
run_pgm_1d_1
		btfsc	p_step_count,7		; pan step underflow
;		 return
		 goto	 current_program_exit
				;
		call	output_pan_stepnum
		movf	p_cur_step,w
		call	set_pan_step_position
				;
run_pgm_1d_1a
		btfss	prog_run_flags,HALT_FLG	; did program get halted
		 goto	 run_pgm_1d_1b		; no, keep going
		call	do_halt_prompt
		btfsc	prog_run_flags,HALT_FLG	; was it an abort ?
		 goto	 current_program_exit	; abort
		goto	run_pgm_1d_1		; complete current step
				;
run_pgm_1d_1b
		movf	settle_time,w		; settle time in 100ths of seconds
		movwf	timeout
				;
run_pgm_1d_2
		call	check_halt_sw
		btfsc	prog_run_flags,HALT_FLG
		 goto	 run_pgm_1d_1a		; process halt
				;		 
		movf	timeout,f
		btfss	STATUS,Z
		 goto	run_pgm_1d_2
				;
		call	press_shutter
				;
		btfsc	prog_run_flags,SINGLE_STEP_FLG
		 goto	run_pgm_1d_5		; if single stepping no need for hold time
				;
		movf	hold_time,w			; hold time in hundredths of a second
		movwf	temp_ctr				; save in count-down
run_pgm_1d_3
		movf	temp_ctr,f 			; test hold time countdown
		btfsc	STATUS,Z
		 goto	run_pgm_1d_5		; if zero, do next step
				;
		movlw	D'10'				; count 10 mS for each hold count
		movwf	timeout
run_pgm_1d_4
		call	check_halt_sw
		btfsc	prog_run_flags,HALT_FLG
		 goto	 run_pgm_1d_1a		; process halt
				;
		movf	timeout,f
		btfss	STATUS,Z
		 goto	run_pgm_1d_4
				;
		decf	temp_ctr,f
		goto	run_pgm_1d_3
									;
run_pgm_1d_5						; test single-step mode
		btfsc	prog_run_flags,SINGLE_STEP_FLG
		 call	get_step_switch
				;
		btfsc	prog_run_flags,HALT_FLG
		 goto	 run_pgm_1d_1a		; process halt
				;
		incf	p_cur_step,f
		decf	p_step_count,f
		goto	run_pgm_1d_1
				;
				; ****************************
				; *****  Do halt prompt  *****
				; ****************************
do_halt_prompt
		movlw	HIGH do_halt_prompt_text
		movwf	PCLATH
		call	do_halt_prompt_text	; "Halting Current Program "
									; "      Continue     ABORT"
		movlw	high do_halt_prompt		
		movwf	PCLATH					;
do_halt_prompt_1
		btfss	rbufptr,7		; test for RS-232 command
		 goto	do_halt_prompt_1a
		movlw	HIGH rcv_cmd_p1	; set up PC high bits
		movwf	PCLATH
		call	rcv_cmd_p1
		movlw	HIGH main_loop	; restore PC high bits
		movwf	PCLATH
do_halt_prompt_1a
		call	read_sw_state		; wait for halt switch release
		btfsc	sw_state,SWITCH_1	; is HALT switch still pressed
		 goto	 do_halt_prompt_1
do_halt_prompt_1b
		btfss	rbufptr,7		; test for RS-232 command
		 goto	do_halt_prompt_1c
		movlw	HIGH rcv_cmd_p1	; set up PC high bits
		movwf	PCLATH
		call	rcv_cmd_p1
		movlw	HIGH main_loop	; restore PC high bits
		movwf	PCLATH
do_halt_prompt_1c
		call	read_sw_state		
		btfsc	sw_state,SWITCH_4	; is ABORT switch pressed
		 return						; return with halt flag still set
		btfss	sw_state,SWITCH_2	; is CONTINUE switch pressed
		 goto	 do_halt_prompt_1b	; no, look for a switch press
				;
				; continue mode
		bcf		prog_run_flags,HALT_FLG	; clear the halt flag
		clrf	sw_state
				;
		movlw	high do_continue_text
		movwf	PCLATH
		call	do_continue_text	; "   Continuing Program   "
		movlw	high do_halt_prompt		
		movwf	PCLATH
				;
		btfss	prog_type,PROG_1D_FLG
		 goto	 do_halt_prompt_2
				;
		movlw	high run_1d_step_txt
		movwf	PCLATH
		call	run_1d_step_txt		; "HALT         Pan=xx     "
		movlw	high do_halt_prompt		
		movwf	PCLATH
		call	output_pan_stepnum
		return
				;
do_halt_prompt_2
		btfss	prog_type,PROG_2D_FLG
		 goto	 do_halt_prompt_3
		movlw	high run_2d_step_txt
		movwf	PCLATH
		call	run_2d_step_txt		; "HALT Elev=xx Pan=xx     "
		movlw	high do_halt_prompt		
		movwf	PCLATH
		call	output_elev_stepnum
		call	output_pan_stepnum
		return
				;
do_halt_prompt_3
		return
				;
				; ***************************************
				; *****  Manual single-step switch  *****
				; ***************************************		
get_step_switch

get_step_sw0
		call	check_halt_sw
		btfsc	prog_run_flags,HALT_FLG
		 goto	 get_step_sw2		; process halt
									;
		call	read_sw_state		; check that "STEP" button is released
;		btfsc	sw_state,SWITCH_1	; is HALT switch pressed
;		 goto	get_step_sw2
		movf	sw_state,f			; test switch state
		btfss	STATUS,Z			; check that "STEP" button is released
		 goto	get_step_switch 	; if not, wait for release
				;
		btfss	EXT_SW_PORT,EXT_SW	; test external switch input
		 goto	get_step_switch		; if active, wait till it's released
									;
		btfsc	prog_run_flags,SINGLE_STEP_FLG
		 call	 prog_step_prompt_txt	; output "STEP" prompt
									; wait for a switch press
get_step_sw1
		btfss	rbufptr,7		; test for RS-232 command
		 goto	get_step_sw1a
		movlw	HIGH rcv_cmd_p1	; set up PC high bits
		movwf	PCLATH
		call	rcv_cmd_p1
		movlw	HIGH main_loop	; restore PC high bits
		movwf	PCLATH
		btfsc	sw_state,SWITCH_1
		 bsf	 prog_run_flags,HALT_FLG
get_step_sw1a
		call	check_halt_sw
		btfsc	prog_run_flags,HALT_FLG
		 goto	 get_step_sw2		; process halt
									;
		call	read_sw_state		; test front panel switch
		btfsc	sw_state,SWITCH_4	; is "ENTER" / "STEP" switch pressed
		 goto	 get_step_sw2						; yes, exit to next step
									;
		btfsc	EXT_SW_PORT,EXT_SW	; test external switch input
		 goto	get_step_sw1		; if not closed, check both switches again
get_step_sw2
		btfsc	prog_run_flags,SINGLE_STEP_FLG
		 call	 prog_nostep_prompt_txt	; output "    " prompt
		clrf	sw_state
		return
				;
				; *************************************
				; ** output current elevation step # **
				; *************************************
output_elev_stepnum
	IF	DISP24
		movlw	0xCA			; move cursor to display location
	ELSE
		movlw	0xC7			; move cursor to display location
	ENDIF
		call	SEND_CMD
		clrf	BCflag
		bsf		BCflag,6		; turn on leading-zero supression
		bsf		BCflag,3		; two digit output
		movf	e_cur_step,w	; get step count
		addlw	1
		goto	OUTD8B			; display as unsigned decimal
				;
				; *******************************
				; ** output current pan step # **
				; *******************************
output_pan_stepnum
	IF	DISP24
		movlw	0xD1			; move cursor to display location
	ELSE
		movlw	0xCC			; move cursor to display location
	ENDIF
		call	SEND_CMD
		clrf	BCflag
		bsf		BCflag,6		; turn on leading-zero supression
		bsf		BCflag,3		; two digit output
		movf	p_cur_step,w	; get step count
		addlw	1
		goto	OUTD8B			; display as unsigned decimal
				;
				; **************************
				; ** press shutter button **
				; **************************
press_shutter
				bsf		SHUTTER_REG,SW1		; turn on 1st switch
				;
				movf	shutter_T1,w		; wait for 1st delay time
				movwf	timeout
press_shutter1
				movf	timeout,f
				btfss	STATUS,Z
				 goto	press_shutter1
				;
				bsf		SHUTTER_REG,SW2		; turn on 2nd switch
;				bsf		SHUTTER_REG,LED_OUT		; turn on LED output
				;
				movf	shutter_T2,w		; wait for 2nd delay time
				movwf	timeout
press_shutter2
				movf	timeout,f
				btfss	STATUS,Z
				 goto	press_shutter2
				;
				bcf		SHUTTER_REG,SW2		; turn off 2nd switch
				bcf		SHUTTER_REG,SW1		; turn on 1st switch
;				bcf		SHUTTER_REG,LED_OUT		; turn on LED output
				;
				return
				;
				; *****************************************************
				; *****************************************************
				; *****  ELEVATION MOTOR MOTION CONTROL ROUTINES  *****
				; *****************************************************
				; *****************************************************
				;
				; **************************************
				; **     set_elev_step_position (w)   **
				; **************************************
				; ** go to current elev step position **
				; **  Start_pos + #steps * Step_size  **
				; **       call with step # in w      **
				; **    w=0 goes to Start_position    **
				; **   w=1 goes to start + stepsize   **
				; **************************************
set_elev_step_position
		call	calc_elev_position	; calculate new position
		goto	elev_new_position	; move pan-head to new position
				;
				; **************************************************
				; ** calculate elev position for elev_stepct in w **
				; **************************************************
calc_elev_position
		movwf	tempa				; copy current step count
		movf	elev_start_h,w		; put start position into new
		movwf	new_pos_h
		movf	elev_start_l,w
		movwf	new_pos_l
calc_elev_pos_1
		movf	tempa,f				; get temp step counter
		btfsc	STATUS,Z			; is it zero
		 return						; yes, return
		movf	new_pos_l,w			; add one step_size to new position
		addwf	elev_step_l,w
		btfsc	STATUS,C
		 incf	new_pos_h,f
		movwf	new_pos_l
		movf	new_pos_h,w
		addwf	elev_step_h,w
		movwf	new_pos_h
		decf	tempa,f
		goto	calc_elev_pos_1
				;
				; ******************************************
				; ** move elevation motor to new_position **
				; ******************************************
elev_new_position
		movf	new_pos_h,w
		movwf	tempm_h
		movf	elev_cur_pos_l,w
		subwf	new_pos_l,w
		btfss	STATUS,C
		 decf	tempm_h,f
		movwf	tempm_l
								;
		movf	elev_cur_pos_h,w
		subwf	tempm_h,w
		btfsc	STATUS,Z
		 goto	elev_new_pos_1
		movwf	tempm_h
		btfss	tempm_h,7			; is it negative
		 goto	elev_new_pos_2
		goto	elev_new_pos_3
elev_new_pos_1
		movf	tempm_l,f
		btfsc	STATUS,Z
		 return			; exit if elev_cur_pos == new_pos
elev_new_pos_2
		call	elev_sstep_up
		goto	elev_new_pos_4
elev_new_pos_3
		call	elev_sstep_dn
elev_new_pos_4
		movf	elev_fast_rate,w
		movwf	timeout2
elev_new_pos_5
		movf	timeout2,f
		btfss	STATUS,Z
		 goto	elev_new_pos_5
		;
		call	check_halt_sw
		btfsc	prog_run_flags,HALT_FLG
		 return
		goto	elev_new_position
				;
				; ***************************************************************
				; ** increment elev_cur_pos and single-step elevation motor UP **
				; ***************************************************************
elev_sstep_up
		movlw	#1
		bcf		STATUS,C
		addwf	elev_cur_pos_l,f
		btfsc	STATUS,C
		 incf	elev_cur_pos_h,f
		incf	e_phase,f
		goto	elev_step
				;
				; ***************************************************************
				; ** decrement elev_cur_pos and single-step elevation motor DN **
				; ***************************************************************
elev_sstep_dn
		movlw	#1
		bsf		STATUS,C
		subwf	elev_cur_pos_l,f
		btfss	STATUS,C
		 decf	elev_cur_pos_h,f
		decf	e_phase,f
				;
elev_step
		movf	e_phase,w		; get current elevation phase
		call	micro_step		; get micro_step value for this phase angle
		movwf	step_val
		call	make_tmasks		; generate masks to enable elevation motor
		bsf		m_ctrl_flg,0	; set update flag
		return
				; ***********************************************
				; ***********************************************
				; *****  PAN MOTOR MOTION CONTROL ROUTINES  *****
				; ***********************************************
				; ***********************************************
				;
				; *************************************
				; **     set_pan_step_position (w)   **
				; *************************************
				; ** go to current pan step position **
				; ** Start_pos + #steps * Step_size  **
				; **      call with step # in w      **
				; **    w=0 goes to Start_position   **
				; **   w=1 goes to start + stepsize  **
				; *************************************
set_pan_step_position
		movwf	tempa				; save step #
		bcf		prog_run_flags,POSITION_DISP_EN		; turn off pan position display
		movf	pan_slow_rate,w		; set pan slew rate
		movwf	pan_rate_l			; put in rate lsb
		clrf	pan_rate_h
		bsf		prog_run_flags,SPEED_RAMP_EN	; enable ramping
		bsf		prog_run_flags,RAMP_UP_FLAG
		movf	tempa,w				; restore step #
									; entry point for slow panning
set_pan_step_pos_1
		call	calc_pan_position	; calculate new position		
		goto	pan_new_pos_0		; move pan-head to new position
				;
				; ************************************************
				; ** calculate pan position for pan_stepct in w **
				; ************************************************
calc_pan_position
		movwf	tempa				; copy current step count
		movf	pan_start_h,w		; put start position into new
		movwf	new_pos_h
		movf	pan_start_l,w
		movwf	new_pos_l
calc_pan_pos_1
		movf	tempa,f				; get temp step counter
		btfsc	STATUS,Z			; is it zero
		 return						; yes, return
		movf	new_pos_l,w			; add one step_size to new position
		addwf	pan_step_l,w
		btfsc	STATUS,C
		 incf	new_pos_h,f
		movwf	new_pos_l
		movf	new_pos_h,w
		addwf	pan_step_h,w
		movwf	new_pos_h
		decf	tempa,f
		goto	calc_pan_pos_1
				;
				; ************************************
				; ** move pan motor to new_position **
				; ************************************
				;	prog_run_flags
				;	HALT_FLG			EQU	7	; set if HALT is detected
				;	SINGLE_STEP_FLG		EQU	6	; set when single-stepping
				;	POSITION_DISP_EN	EQU	5	; set to display position in slow-pan
				;	SPEED_RAMP_EN		EQU	4	; set when Ramp-up/down is enabled
				;	RAMP_UP_FLAG		EQU	3	; set at beginning of move for ramp-up
				;
pan_new_position
		movf	pan_slow_rate,w		; set pan slew rate
		movwf	pan_rate_l			; put in rate lsb
		clrf	pan_rate_h
		bsf		prog_run_flags,SPEED_RAMP_EN	; enable ramping
		bsf		prog_run_flags,RAMP_UP_FLAG
pan_new_pos_0
		movf	pan_rate_h,w
		movwf	tempc
		movf	new_pos_h,w
		movwf	tempm_h
		movf	pan_cur_pos_l,w
		subwf	new_pos_l,w
		btfss	STATUS,C
		 decf	tempm_h,f
		movwf	tempm_l
								;
		movf	pan_cur_pos_h,w
		subwf	tempm_h,f
		btfsc	STATUS,Z
		 goto	pan_new_pos_1
		btfss	tempm_h,7			; is it negative
		 goto	pan_new_pos_2
		goto	pan_new_pos_3
pan_new_pos_1
		movf	tempm_l,f
		btfsc	STATUS,Z
		 return			; exit if pan_cur_pos == new_pos
pan_new_pos_2
				; (New - Current) > 0 : Move CW
		call	pan_sstep_cw
		goto	pan_new_pos_4
pan_new_pos_3
				; (New - Current) < 0 : Move CCW
		comf	tempm_h,f		; make abs val of offset
		comf	tempm_l,f
		call	pan_sstep_ccw
pan_new_pos_4
		btfsc	prog_run_flags,SPEED_RAMP_EN
		 goto	 pan_new_pos_6	; check for ramp
		;
		; rate management for slow pan
pan_new_pos_5
		movf	tempc,f			; if pan_rate_h >0 then count off msbs
		btfsc	STATUS,Z
		 goto	pan_new_pos_5b
		movlw	0xFF
		movwf	timeout2
pan_new_pos_5a
		movf	timeout2,f
		btfss	STATUS,Z
		 goto	pan_new_pos_5a
		decf	tempc,f
		goto	pan_new_pos_5
pan_new_pos_5b
		movf	pan_rate_h,w
		movwf	tempc
		goto	pan_new_pos_9
		;
		; ramp rate management
pan_new_pos_6
		btfss	prog_run_flags,RAMP_UP_FLAG	; are we ramping up
		 goto	 pan_new_pos_7
		;
		; ramping up to slew rate
		decf	pan_rate_l,f
		movf	pan_fast_rate,w		; get fast pan slew rate
		subwf	pan_rate_l,w		; pan rate - fast rate
		btfsc	STATUS,C			; test rate
		 goto	 pan_new_pos_7		; still accelerating
		;
		; end of ramp up
		movf	pan_fast_rate,w		; get fast pan slew rate
		movwf	pan_rate_l
		bcf		prog_run_flags,RAMP_UP_FLAG	; turn off ramp-up
		;
		; check to see if close enough to start ramping down
pan_new_pos_7
		movf	tempm_h,f		; how many more steps ?
		btfss	STATUS,Z		; >255
		 goto	 pan_new_pos_9
		movf	pan_ramp_dist,w
		subwf	tempm_l,w		; (remaining steps - rampdown distance)
		btfsc	STATUS,C		; > 0
		 goto	 pan_new_pos_9	; not in deceleration range
		;
		; speed ramp-down
		bcf		prog_run_flags,RAMP_UP_FLAG	; turn off any existing ramp-up
	incf	pan_rate_l,f
pan_new_pos_9
		movf	pan_rate_l,w
		movwf	timeout2
pan_new_pos_9a
		movf	timeout2,f
		btfss	STATUS,Z
		 goto	pan_new_pos_9a
pan_new_pos_e
		call	check_halt_sw
		btfsc	prog_run_flags,HALT_FLG
		 return
		btfss	prog_run_flags,POSITION_DISP_EN	; flag to display current pan position
		 goto	pan_new_pos_0
	IF	DISP24
		movlw	0xCB
	ELSE
		movlw	0xC9
	ENDIF
		call	SEND_CMD
		movf	pan_cur_pos_h,w	; get current pan position
		movwf	minuend_h
		movf	pan_cur_pos_l,w
		movwf	minuend_l
		call	OUTSD16			; display position
		goto	pan_new_pos_0
				;
				; ********************************************************
				; ** increment pan_cur_pos and single-step pan motor CW **
				; ********************************************************
pan_sstep_cw
		movlw	#1
		bcf		STATUS,C
		addwf	pan_cur_pos_l,f
		btfsc	STATUS,C
		 incf	pan_cur_pos_h,f
		incf	p_phase,f
		goto	pan_step
				;
				; *********************************************************
				; ** decrement pan_cur_pos and single-step pan motor CCW **
				; *********************************************************
pan_sstep_ccw
		movlw	#1
		bsf		STATUS,C
		subwf	pan_cur_pos_l,f
		btfss	STATUS,C
		 decf	pan_cur_pos_h,f
		decf	p_phase,f
				;
pan_step
		movf	p_phase,w		; get current pan_motor phase
		call	micro_step		; get micro_step value for this phase angle
		movwf	step_val
		swapf	tmask2,f		; swap nybbles to work on pan bits
		swapf	tmask1,f
;		swapf	tmask0,f
		call	make_tmasks		; generate masks to enable motor
		swapf	tmask2,f		; swap nybbles to put man in msbs
		swapf	tmask1,f
;		swapf	tmask0,f
		bsf		m_ctrl_flg,0	; set update flag
		return
					;
					; *********************************************
					; ****  make motor bit masks from step_val ****
					; ****    creates bit masks in bits[3:0]   ****
					; *********************************************
make_tmasks
		btfsc	m_ctrl_flg,0	; make sure we arent't still updating previous step
		 goto	 make_tmasks	; wait until prev update completes
								;
		movlw	0x0f			; initially turn off all motor outputs
		iorwf	tmask2,f
		iorwf	tmask1,f
;		iorwf	tmask0,f
								; set up bits for winding A
		movlw	MTR_A_PLUS		; enable for +A winding
		btfsc	step_val,3		; test winding A polarity
		 movlw	 MTR_A_MINUS	; enable for -A winding
		btfsc	step_val,2		; winding A 100% pwr ?
		 andwf	 tmask2,f		; enable output in 100% mask
		btfsc	step_val,1		; winding A 50% pwr ?
		 andwf	 tmask1,f		; enable output in 50% mask
;		btfsc	step_val,0		; winding A 25% (75% is 50%+25%)
;		 andwf	 tmask0,f		; enable output in 25% mask
								;
								; set up bits for winding B
		movlw	MTR_B_PLUS		; enable for +B winding
		btfsc	step_val,7		; test winding B polarity
		 movlw	 MTR_B_MINUS	; enable for -B winding
		btfsc	step_val,6		; winding B 100% pwr ?
		 andwf	 tmask2,f		; enable output in 100% mask
		btfsc	step_val,5		; winding B 50% pwr ?
		 andwf	 tmask1,f		; enable output in 50% mask
;		btfsc	step_val,4		; winding B 25% (75% is 50%+25%)
;		 andwf	 tmask0,f		; enable output in 25% mask
								;
		return

					;
					; *****************************
					; ****  Read switch state  ****
					; *****************************
read_sw_state
		btfss	rbufptr,7		; test for RS-232 command
		 goto	 read_sw_0
				;
				; jump to RS-232 receive handler
				;
		movlw	HIGH rcv_cmd_p1	; set up PC high bits
		movwf	PCLATH
		call	rcv_cmd_p1
		movlw	HIGH main_loop	; restore PC high bits
		movwf	PCLATH
				;
read_sw_0
		btfsc	keypad_lock,6
		 goto	read_sw_3
		btfsc	keypad_lock,7
		 goto	read_sw_2
		;
		swapf	SW_PORT,w	; get current switch state
		xorlw	0x0f		; compliment lsbs
		andlw	0x0f		; mask msbs
		movwf	sw_temp		; save switch state
							;
		movlw	#2
		movwf	timeout2
read_sw_1
		movf	timeout2,f
		btfss	STATUS,Z
		 goto	read_sw_1	
						;
		movf	sw_state,w	; return with switch state
		xorwf	sw_temp,w
		btfsc	STATUS,Z
		 goto	read_sw_3
		movf	sw_temp,w
		movwf	sw_state
		goto	read_sw_state
read_sw_2
		clrf	 sw_state
read_sw_3
		movf	sw_state,w
		btfsc	STATUS,Z
		 bcf	 keypad_lock,6
		return
		;
					;
					; *********************************
					; ****  Check for HALT button  ****
					; *********************************
check_halt_sw
		btfss	 prog_run_flags,HALT_EN	; checking for HALT
		 return							; no, return
		btfss	SW_PORT,SWITCH_1 + 4	; HALT switch pressed ?
		 bsf	 prog_run_flags,HALT_FLG
		return
		;
		; *************************
		; *  page 0 text strings  *
		; *************************
		;
prog_step_prompt_txt	; "STEP"
	IF	DISP24
		movlw	0xD4
	ELSE
		movlw	0xD0
	ENDIF
		call	SEND_CMD
		movlw	'S'
		call	OUTCH
		movlw	'T'
		call	OUTCH
		movlw	'E'
		call	OUTCH
		movlw	'P'
		call	OUTCH
		return
		;
prog_nostep_prompt_txt	; "    "
	IF	DISP24
		movlw	0xD4
	ELSE
		movlw	0xD0
	ENDIF
		call	SEND_CMD
		movlw	' '
		call	OUTCH
		movlw	' '
		call	OUTCH
		movlw	' '
		call	OUTCH
		movlw	' '
		call	OUTCH
		return
					;					
					; ***********************************
					; ****  EEPROM vectors to page 1 ****
					; ***********************************
					;
check_program
		movlw	HIGH check_program_p1	; set PC high bits
		movwf	PCLATH
		call	check_program_p1	; check program type
		movwf	tempa				; save it
		movlw	HIGH main			; restore PC high bits
		movwf	PCLATH
		movf	tempa,w				; get prog type back
		return
					;					
					; *******************************************
					; ****  Load motion program from EEPROM  ****
					; *******************************************
load_program
		movlw	HIGH load_program_p1	; set PC high bits
		movwf	PCLATH
		call	load_program_p1		; load program
		movlw	HIGH main			; restore PC high bits
		movwf	PCLATH
		return
					;					
					; *****************************************
					; ****  Save motion program to EEPROM  ****
					; *****************************************
save_program
		movlw	HIGH save_program_p1	; set PC high bits
		movwf	PCLATH
		call	save_program_p1		; load program
		movlw	HIGH main			; restore PC high bits
		movwf	PCLATH
		return
					;
					; ***************************************
					; ****  Start of LCD initialization  ****
					; ***************************************
DISPLAY_INIT					;
		bcf		SHDW_A,RW		; LCD R/W low in shadow (write)
		bcf		SHDW_A,RS		; LCD R/W low in shadow (cmd register)
		movf	SHDW_A,w		; copy shadow to LCD_CONTROL_PORT (PORTC)
		movwf	LCD_CNTL		;
					;
		CALL	LCD_OUT_MODE	; put RB3-RB0 in output mode
					;			
		MOVLW   0x03           ; Display init Command (4-bit)
		MOVWF   LCD_DATA        ;
					; 
		bsf		SHDW_A,E		; pulse E high
		movf	SHDW_A,w		; get shadow register
		movwf	LCD_CNTL
		nop
		nop
		nop						; 
		bcf		SHDW_A,E		; clear E
		movf	SHDW_A,w		; get shadow register
		movwf	LCD_CNTL		;
		nop
		nop
		movlw	LCD_INIT_DELAY
		movwf	timeout
LCD_DELAY
		movf	timeout,f
		btfss	STATUS,Z
		 goto	LCD_DELAY
								;
CMD_SEQ							;
		MOVLW   0x03           ; Display init Command (4-bit)
		MOVWF   LCD_DATA        ; put in LCD data register
					;
		bsf		SHDW_A,E		; pulse E high
		movf	SHDW_A,w		; get shadow register
		movwf	LCD_CNTL
		nop
		nop
		nop
		bcf		SHDW_A,E		; clear E
		movf	SHDW_A,w		; get shadow register
		movwf	LCD_CNTL
		nop
		nop
		nop
					;
		MOVLW   0x03           ; Display init Command (4-bit)
		MOVWF   LCD_DATA        ; 
					;
		bsf		SHDW_A,E		; pulse E high
		movf	SHDW_A,w		; get shadow register
		movwf	LCD_CNTL
		nop
		nop
		nop
		bcf		SHDW_A,E		; clear E
		movf	SHDW_A,w		; get shadow register
		movwf	LCD_CNTL
		nop
		nop
		nop
					; Busy Flag should be valid after this point
					;
		MOVLW   0x020           ; function set Command (4-bit mode)
		CALL    SEND_CMD        ;
					;
		MOVLW   0x028       	;
		CALL    SEND_CMD        ;
					;
		MOVLW   DISP_ON         ; Turn on display
		CALL    SEND_CMD        ;
		MOVLW   CLR_DISP        ; Clear contents of display
		CALL    SEND_CMD        ;
		MOVLW   ENTRY_INC       ; Set cursor to auto-increment
		CALL    SEND_CMD        ;
		MOVLW   DD_RAM_ADDR     ;
		CALL    SEND_CMD        ;
		RETURN			;
					;
					;
					;
					; **********************************
					; **  The LCD Module Subroutines  **
					; **********************************
					;
					; **********************************
					; **  re-initialize display  **
					;
INIT_DISPLAY
		MOVLW   DISP_ON		; Display On, Cursor On
		CALL    SEND_CMD	; Send This command to the Display Module
		MOVLW   CLR_DISP	; Clear the Display
		CALL    SEND_CMD	; Send This command to the Display Module
		MOVLW   ENTRY_INC	; Set Entry Mode Inc., No shift
		CALL    SEND_CMD	; Send This command to the Display Module
		RETURN
		;
;
;		**********************************************************************
;		** Output the byte in accum "W" as 2 hex digits followed by a space **
;		**     800uS                                                        **
;		**********************************************************************
OUTHS		
		call	OUTHX		; output byte as hex
		goto	OUTS		; output space
;					;
;		**************************************************
;		** Output the byte in accum "W" as 2 hex digits **
;		**************************************************
OUTHX		
		movwf	tempa		; need to save value in tempa
		swapf	tempa,0		; get upper 4-bits into lsbs of W
		call	OUTHD		; output most-significant hex digit
		movf	tempa,0		; get the original byte back
;					; fall through to output ls-digit
;
;		***********************************************************
;		** Output a hex digit from the LSBs of the contents of W **
;		***********************************************************
OUTHD		
		andlw	0x0f		; mask off the MSBs
		movwf	tempb		; save copy
		movlw	0x0a		; subtract 10 decimal
	 	subwf	tempb,1		; see if it is decimal
		movf	tempb,0
		btfss	tempb,7		; did it underflow ?
OUTHD1	 	
		addlw	0x07		; shift to A-F if 10 or greater
		nop
		addlw	0x03A		; add hex 30 plus the original 10 decimal
		goto	OUTCH		; output '0-9' or 'A-F' and return
;
;		********************************************
;		** output a carriage return and line feed **
;		********************************************
OUTCRLF		
		movlw	0x0d		; put <CR> in W
		call	OUTCH		; output it
		movlw	0x0d		; put <LF> in W
		goto	OUTCH		; output it and return

;		******************************************
;		** Output two spaces / Output one space **
;		******************************************
OUT2S	call	OUTS
OUTS	movlw	0x20		; put ASCII )x20 <space> in W
;					; fall through to character output
;
;		********************************************
;		**  Output the character in W to the LCD  **
;		********************************************
OUTCH	nop
		;
		;
		;
		; **************************************************************
		; * SEND_CHAR - Sends character to LCD                         *
		; *                                                            *
		; * This routine splits the character into the upper and lower * 
		; * nibbles and sends them to the LCD, upper nibble first.     *
		; *   (The data is transmitted on the PORTB<7:4> pins)         *
		; *   (The data is transmitted on the PORTB<3:0> pins)(rev-b)  *
		; *                                                            *
		; *   Call with character to be sent in accumulator (w)        *
		; **************************************************************
SEND_CHAR
		MOVWF   CHAR            ; Character to be sent is in W, save in CHAR
		CALL    BUSY_CHECK      ; Wait for LCD to be ready
		CALL	LCD_OUT_MODE	; put RC3-RC0 in output mode
		SWAPF   CHAR, w			; Get upper nibble into LSBs
		ANDLW   0x0F			; Get mask nibble
		MOVWF   LCD_DATA		; Send data to LCD
		BCF     SHDW_A, RW		; Set LCD to be write mode
		BSF     SHDW_A, RS		; Set LCD register select to data port
								;
		movf	SHDW_A,w		; get shadow register
		movwf	LCD_CNTL
		bsf		SHDW_A,E		; pulse E high
		movf	SHDW_A,w		; get shadow register
		movwf	LCD_CNTL
		nop
		nop
		nop
		bcf		SHDW_A,E		; clear E
		movf	SHDW_A,w		; get shadow register
		movwf	LCD_CNTL
		nop
		nop
		nop
								;
		MOVF    CHAR, w         ; Get lower nibble
		ANDLW   0x0F			; mask out upper nibble
		MOVWF   LCD_DATA        ; Send data to LCD
					;
		bsf		SHDW_A,E		; pulse E high
		movf	SHDW_A,w		; get shadow register
		movwf	LCD_CNTL
		nop
		nop
		nop
		bcf		SHDW_A,E		; clear E
		movf	SHDW_A,w		; get shadow register
		movwf	LCD_CNTL
		nop
		nop
		nop
					;	
		CALL	LCD_IN_MODE		; return port to input mode
		RETURN			;
					;
					;
		; ************************************************************
		; * SEND_CMD - Sends command to LCD                          *
		; *                                                          *
		; * this routine splits the command into the upper and lower * 
		; * nibbles and sends them to the LCD, upper nibble first.   *
		; * The data is transmitted on the PORTC<3:0> pins)          *
		; *                                                          *
		; *   Call with character to be sent in accumulator (w)      * 
		; ************************************************************
SEND_CMD
		MOVWF   CHAR            ; Character to be sent is in W
		CALL    BUSY_CHECK      ; Wait for LCD to be ready
		CALL	LCD_OUT_MODE	; put RC3-RC0 in output mode
		SWAPF   CHAR, w			; swap upper nibble into LSBs
		ANDLW   0x0F			; mask out upper nibble
		MOVWF   LCD_DATA        ; Send data to LCD
		BCF     SHDW_A,RW		; Set LCD to write mode
		BCF     SHDW_A,RS		; Set LCD register select to command port
					;
		movf	SHDW_A,w		; get shadow register
		movwf	LCD_CNTL		; 
		bsf		SHDW_A,E		; pulse E high
		movf	SHDW_A,w		; get shadow register
		movwf	LCD_CNTL
		nop
		nop
		nop
		bcf		SHDW_A,E		; clear E
		movf	SHDW_A,w		; get shadow register
		movwf	LCD_CNTL
		nop
		nop
		nop						;
		MOVF    CHAR, w         ; Get lower nibble
		ANDLW   0x0F			; mask out upper nibble
		MOVWF   LCD_DATA        ; Send data to LCD
					;
		bsf		SHDW_A,E		; pulse E high
		movf	SHDW_A,w		; get shadow register
		movwf	LCD_CNTL
		nop
		nop
		nop 
		bcf		SHDW_A,E		; clear E
		movf	SHDW_A,w		; get shadow register
		movwf	LCD_CNTL
		nop
		nop
		nop
								;
		CALL	LCD_IN_MODE		; return port to input mode
		RETURN			;
					;
					;
		; ****************************************************************
		; * BUSY_CHECK - checks the LCD busy flag, returns when not busy *
		; *             ( destroys contents of accumulator )             *
		; ****************************************************************
BUSY_CHECK
		CALL	LCD_IN_MODE		; Make sure port is configured as input
					;
		BCF     SHDW_A, RS		; Set LCD for Command mode
		BSF     SHDW_A, RW		; Setup to read busy flag
BSYCHK1
		movf	SHDW_A,w		; get shadow register
		movwf	LCD_CNTL		; write to PORT-A	
		bsf		SHDW_A,E		; pulse E high
		movf	SHDW_A,w		; get shadow register
		movwf	LCD_CNTL
		nop
		nop
		nop
		SWAPF   LCD_DATA, w     ; swap lower nibble into MSBs (busy flag, DDRam address)
		ANDLW   0xF0            ; Mask out lower nibble
		MOVWF   TEMP			; save in TEMP
		bcf		SHDW_A,E		; clear E
		movf	SHDW_A,w		; get shadow register
		movwf	LCD_CNTL
		nop
		nop			; slight delay
		nop
		bsf		SHDW_A,E		; pulse E high
		movf	SHDW_A,w		; get shadow register
		movwf	LCD_CNTL
		nop
		nop
		nop
		MOVF	LCD_DATA, w     ; Read lower nibble busy
		ANDLW   0x0F            ; Mask out upper nibble
		IORWF   TEMP,F			; Combine nibbles
		bcf		SHDW_A,E		; clear E
		movf	SHDW_A,w		; get shadow register
		movwf	LCD_CNTL
		nop
		nop
		nop
		BTFSC   TEMP,7          ; Check busy flag, high = busy
		GOTO    BSYCHK1			; If busy, check again
		BCF     LCD_CNTL, RW   
		call	LCD_OUT_MODE     
		RETURN
		;
		; ***************************************************************************
		; * LCD_OUT_MODE - configures lower 4 bits of the LCD data port as outputs  *
		;		( doesn't affect accumulator (w) )                                  *
		; ***************************************************************************
LCD_OUT_MODE
		BSF     STATUS, RP0		; Select Register page 1
;
		BCF		LCD_DATA_TRIS,3			; make RB3 an output
		BCF		LCD_DATA_TRIS,2			; make RB2 an output
		BCF		LCD_DATA_TRIS,1			; make RB1 an output
		BCF		LCD_DATA_TRIS,0			; make RB0 an output
;
		BCF     STATUS, RP0     ; Select Register page 0
		RETURN
		;
		; **************************************************************
		; * LCD_IN_MODE - configures lower 4 bits of Port-B as inputs  *
		;		( doesn't affect accumulator (w) )                     *
		; **************************************************************
LCD_IN_MODE
		BSF     STATUS, RP0     ; Select Register page 1
;
		BSF		LCD_DATA_TRIS,3			; make RB3 an input
		BSF		LCD_DATA_TRIS,2			; make RB2 an input
		BSF		LCD_DATA_TRIS,1			; make RB1 an input
		BSF		LCD_DATA_TRIS,0			; make RB0 an input
;
		BCF     STATUS, RP0     ; Select Register page 0
		RETURN
					;
		;

		;
		; ********************************
		; *   DECIMAL DISPLAY ROUTINES   *
		; ********************************
		;
		; **************************************
		; *   Output 8-bit signed number in W  *
		; **************************************
OUTSD8
		movwf	LO_TEMP
		btfss	LO_TEMP,7		; is MSB set
		goto	outpls			; no, output "+" and unsigned value
;			negative number
		movlw	'-'
		call	OUTCH
		comf	LO_TEMP,1
		movlw	1
		addwf	LO_TEMP,0
		goto	OUTD8B
;
outpls	movlw	'+'
		call	OUTCH
		movf	LO_TEMP,0		; get number back again
		;
		; **********************************************
		; *      Output 8-bit unsigned number in W     *
		; **********************************************
		; *  BCflag  -  format control                 *
		; *  | x | zero-supress | dp1 | dp2 | 2-dig | xxx |  *
		; **********************************************
OUTD8B
		movwf	LO_TEMP			; save in LO_TEMP
		bcf		BCflag,7		; blank checker for preceeding zeros
;	
		movlw	d'100'			; check amount of 100s
		movwf	tempb			; ==> Decimal Range 0 - 255 <=> 8 bit
		call	cnv08			; call conversion sub-routine
		btfss	BCflag,3		; test for 2-digit output
		 call	OUTCH
;
		movlw	'.'				; prepare DP
		btfsc	BCflag,5		; test for embedded DP in front of "tens"
		 call	OUTCH			; if flag is set output DP  (X.XX)
;
		btfsc	BCflag,4		; test for embedded DP in front of "ones"
		 bsf	BCflag,7		; spoil leading zero supression for 0.X

		movlw	d'10'			; check amount of 10s
		movwf	tempb
		call	cnv08			; call conversion sub-routine
		call	OUTCH
;
		movlw	'.'				; prepare DP
		btfsc	BCflag,4		; test for embedded DP
		 call	OUTCH			; if flag is set output DP  (XX.X)
;
		movlw	d'1'			; check amount of 1s
		movwf	tempb
		bsf		BCflag,7		; remove blank checker in case of zero
		call	cnv08			; call conversion sub-routine
		call	OUTCH
		RETURN
;	
cnv08
		clrf	tempa			; counter
		movfw	tempb			; decrement-value	
cnv08_1	subwf	LO_TEMP,0		; TEST: LO_TEMP-TEMP2 >= 0 ?
		skpc					; skip, if true
		goto	cnv08_2			; result negativ, exit
		incf	tempa,1			; count
		movfw	tempb			; decrement-value
		subwf	LO_TEMP,1		; STORE: LO_TEMP = LO_TEMP - TEMP2
		bsf		BCflag,7		; invalidate flag
		goto	cnv08_1			; repeat
cnv08_2
		movlw	'0'				; writes Number to LCD
		addwf	tempa,0			; '0' is ascii offset, add counter
		btfss	BCflag,6		; is zero-supression on
		 return					; no, you are done
		btfss	BCflag,7		; check zero-supress flagflag
		 movlw	' '				; clear preceeding zeros
		RETURN
		;
		;
		; **********************************
		; *  signed 16 bit decimal output  *
		; *      value in minuend[h:l]     *
		; **********************************
OUTSD16
		btfss	minuend_h,7		; is MSB set
		goto	outsd16p		; no, output "+" and unsigned value
;			negative number
		movlw	'-'
		call	OUTCH
		comf	minuend_h,f
		comf	minuend_l,f
		movlw	1
		addwf	minuend_l,f
		btfsc	STATUS,C
		 incf	minuend_h,f
		goto	OUTD16
;
outsd16p	
		movlw	'+'
		call	OUTCH
		;
		; ************************************
		; *  unsigned 16 bit decimal output  *
		; *       value in minuend[h:l]      *
		; ************************************
OUTD16
				; calculate thousands
		movlw	high(THOUSAND)
		movwf	subtrahend_h
		movlw	low(THOUSAND)
		movwf	subtrahend_l
		call	calc_digit
				; calculate hundreds
		movlw	high(HUNDRED)
		movwf	subtrahend_h
		movlw	low(HUNDRED)
		movwf	subtrahend_l
		call	calc_digit
				; calculate tens
		movlw	high(TEN)
		movwf	subtrahend_h
		movlw	low(TEN)
		movwf	subtrahend_l
		call	calc_digit
				; display ones
		movf	minuend_l,w		;
		goto	OUTHD			; output digit and return
				;
				; calculate display digit by repeated subtraction
				;
calc_digit
		clrf	digit_ctr		; set up counter
calc_dig1
		movf	subtrahend_l,w	; get subtrahend LSB
		subwf	minuend_l,w		; subtract from minuend LSB
		movwf	tempm_l			; save in temp minuend LSB
								;
		movf	subtrahend_h,w	; get subtrahend MSB
		btfss	STATUS,C		; test if previous subtraction generated a carry
		 incfsz	subtrahend_h,w	; if subtractive carry (C=0) then subtract 1 more
		  subwf	minuend_h,w		; subtract from minuend unless it was zero
		movwf	tempm_h			; save in temp minuend MSB
								;
		btfss	STATUS,C		; test if final subtraction generated a carry
		goto	calc_dig2		; if subtractive carry(C=0) then output digit
								;
		movf	tempm_h,w		; if not yet underflowing copy temp to minuend
		movwf	minuend_h		;
		movf	tempm_l,w		;
		movwf	minuend_l		;
								;
		incfsz	digit_ctr,f		; bump digit count - shouldn't reach zero
		goto	calc_dig1		; if non-zero repeat process until underflow
								;
calc_dig2						; underflow attained
		movf	digit_ctr,w		; get digit count
		goto	OUTHD			; output digit and return
								;
								;
		; *************************************
		; *  output character from async UART  *
		; *************************************
OUT_UART
		btfss	PIR1,TXIF	; is transmit buffer empty ?
		  goto	OUT_UART	; if not, wait until it is				;
		movwf	TXREG		; send character
		return
		; ****************************************
		; *  Initialize and start A/D converter  *
		; ****************************************
; init_adcon
;		movlw	0x80		; set 32 clocks, chan = 0
;		movwf	ADCON0
;		bsf		STATUS,RP0	; go to 2nd page
;		movlw	0x02		; set 5-chan Vdd-ref left-justified
;		movwf	ADCON1
;		bcf		STATUS,RP0	; back to main page
;		bsf		ADCON0,ADON	; turn on A/D converter
;		bsf		ADCON0,GO	; start conversion
;		return
		; **********************************
		; *  Initialize and start TIMER-2  *
		; **********************************
init_timer2
		movlw	0x78		; set postscale=16, pre-scale=1
		movwf	T2CON
;		bsf		T2CON,0		; make pre:scale 4:1
		bcf		PIR1,TMR2IF	; insure the interrupt flag is clear
		bsf		STATUS,RP0	; go to 2nd page
		movlw	T2_period	; get the interrupt period
		movwf	PR2			; set into the period register
		bsf		PIE1,TMR2IE	; enable timer-2 interrupts
		bcf		STATUS,RP0	; back to main page
		bsf		T2CON,TMR2ON	; activate the timer
		return
		;
;
		; *************************************
		; *  Initialize and start async UART  *
		; *************************************
init_uart
		bsf		STATUS,RP0	; move to page 1
							;
		movlw	BAUDSBRG	; baudrate calculated from Fosc
		movwf	SPBRG		; set into baud-rate generator
							;
		movlw	0x00		; BRGH=low, ASYNC, 8-bit mode
		movwf	TXSTA		; put in Tx-status register
		movlw	0x20		; BRGH=low, ASYNC, 8-bit mode, TXEN
		movwf	TXSTA		; put in Tx-status register
							;
		bcf		STATUS,RP0	; go back to base page
							;
		movlw	0x90		; turn on serial port - rcv continuous, 8-bit
		movwf	RCSTA		; set into Rx-status register
							;
		movf	RCREG,w		; flush receive buffer
					;
		bcf		PIR1,RCIF	; clear rcv interrupt flag
		bsf		STATUS,RP0	; move to page 1
		bsf		PIE1,RCIE	; enable RCV interrupts
		bcf		STATUS,RP0	; go back to base page
					;
		return
		;
		; ********************************************
		; *  Initialize ports and control registers  *
		; ********************************************
		;
		;	PORT-A	(EXT SWITCH input)
		;	| 0 | 0 | 0 | 0 | 0 | 0 | 1 | 0 |	= 0x02
		;	|ra7|ra6|ra5|ra4|ra3|ra2|ra1|ra0|
		;	| - | - | - | - | - | - |ext| - |	
		;
		;	PORT-B	(input switch and LCD display data interface)
		;	| 1 | 1 | 1 | 1 | 0 | 0 | 0 | 0 |	= 0xF0
		;	|rb7|rb6|rb5|rb4|rb3|rb2|rb1|rb0|
		;	|sw3|sw2|sw1|sw0|da3|da2|da1|da0|
		;		initialize as all outputs
		;
		;	PORT-C  (Serial port and LCD control)
		;	| 1 | 0 | 0 | 0 | 0 | 0 | 0 | 0 |	= 0x80
		;	|rc7|rc6|rc5|rc4|rc3|rc2|rc1|rc0|
		;	|RXi|TXo| - | - | - | E |R/W|RS0|
		;
		;	PORT-D	(Stepping motor driver outputs)
		;	| 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0 |	= 0x00
		;	|rd7|rd6|rd5|rd4|rd3|rd2|rd1|rd0|
		;	|pa+|pa-|pb+|pb-|ea+|ea-|eb+|eb-|
		;		initialize as all outputs
		;
		;	PORT-E  (Shutter outputs)
		;	| 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0 |	= 0x00
		;	| - | - | - | - |re3|re2|re1|re0|
		;	| - | - | - | - | - |LED|SH1|SH0|
		;
init_config
		clrf	PORTA
		clrf	PORTB		; make all outputs low
		clrf	PORTC		;
		movlw	0xff		; motor outputs off (high)
		movwf	PORTD
		clrf	PORTE	
					;
		bsf		STATUS,RP0	; go to 2nd page
					;
		movlw	0x02		; make PA[5:0] into input pins
		movwf	TRISA		; put in Port-A's DDR
					;
		movlw	0xF0		; make PB[3:0] into output pins
		movwf	TRISB		; put in Port-B's DDR
					; 
		movlw	0x80		; makeRC[6:0] into outputs
		movwf	TRISC		; put in Port-C's DDR
					;
		movlw	0x00		; makeRD[7:0] into outputs
		movwf	TRISD		; put in Port-D's DDR
					;
		movlw	0x00		; makeRE[2:0] into outputs
		movwf	TRISE		; put in Port-E's DDR
					;
		movlw	0x07		;
		movwf	CMCON		; turn off all comparator inputs
		movlw	0x07		;
		movwf	ADCON1		; turn off all A/D inputs
					;
		bcf		STATUS,RP0	; return to base page
		return
		;
		; *********************
		; *  stepping tables  *
		; *********************
		ORG     0x07E0
;
micro_step
		movwf	temp_tbl			; save a temporary copy
		movlw	HIGH micro_step_tbl	; set up PC high bits
		movwf	PCLATH
		movf	temp_tbl,w
micro_step_tbl
;		andlw	0x1F
		andlw	0x0F
		addwf	PCL,f
	;	32 entry micro stepping table
	;	|			B - winding			|			A - winding			|
	;	| polarity | 100% |  50% |  25% | polarity | 100% |  50% |  25% |
	;
	;		coded value		step #	B pwr	A pwr
	;
		retlw	0x40	; step 00	+100%	off		half_step 00
;		retlw	0x41	; step 01	+100%	+25%
		retlw	0x42	; step 02	+100%	+50%
;		retlw	0x43	; step 03	+100%	+75%
		retlw	0x44	; step 04	+100%	+100% 	half_step 01
;		retlw	0x34	; step 05	+75%	+100%
		retlw	0x24	; step 06	+50%	+100%
;		retlw	0x14	; step 07	+25%	+100%
		;
		retlw	0x04	; step 08	off		+100%	half_step 02
;		retlw	0x94	; step 09	-25%	+100%
		retlw	0xA4	; step 0A	-50%	+100%
;		retlw	0xB4	; step 0B	-75		+100%
		retlw	0xC4	; step 0C	-100%	+100% 	half_step 03
;		retlw	0xC3	; step 0D	-100%	+75%
		retlw	0xC2	; step 0E	-100%	+50%
;		retlw	0xC1	; step 0F	-100%	+25%
		;
		retlw	0xC0	; step 10	off		-100%	half_step 04
;		retlw	0xC9	; step 11	-100%	-25%
		retlw	0xCA	; step 12	-100%	-50%
;		retlw	0xCB	; step 13	-100%	-75%
		retlw	0xCC	; step 14	-100%	-100%	half_step 05
;		retlw	0xBC	; step 15	-75%	-100%
		retlw	0xAC	; step 16	-50%	-100%
;		retlw	0x9C	; step 17	-25%	-100%
		;
		retlw	0x0C	; step 18	off 	-100%	half_step 06
;		retlw	0x1C	; step 19	+25%	-100%
		retlw	0x2C	; step 1A	+50%	-100%
;		retlw	0x3C	; step 1B	+75%	-100%
		retlw	0x4C	; step 1C	-100%	+100% 	half_step 07
;		retlw	0x4B	; step 1D	+100%	-75%
		retlw	0x4A	; step 1E	+100%	-50%
;		retlw	0x49	; step 1F	+100%	-25%

; simple_step
;		movwf	temp_tbl		; save a temporary copy
;		movlw	HIGH simple_step_tbl	; set up PC high bits
;		movwf	PCLATH
;		movf	temp_tbl,w
; simple_step_tbl
;		andlw	0x07
;		addwf	PCL,f
		;
;		retlw	0x08
;		retlw	0x02
;		retlw	0x04
;		retlw	0x01
;		retlw	0x08
;		retlw	0x02
;		retlw	0x04
;		retlw	0x01
		;
; pair_step
;		movwf	temp_tbl		; save a temporary copy
;		movlw	HIGH pair_step_tbl	; set up PC high bits
;		movwf	PCLATH
;		movf	temp_tbl,w
; pair_step_tbl
;		andlw	0x07
;		addwf	PCL,f
;		;
;		retlw	0x0A
;		retlw	0x06
;		retlw	0x05
;		retlw	0x09
;		retlw	0x0A
;		retlw	0x06
;		retlw	0x05
;		retlw	0x09
		;
; half_step
;		movwf	temp_tbl			; save a temporary copy
;		movlw	HIGH half_step_tbl	; set up PC high bits
;		movwf	PCLATH
;		movf	temp_tbl,w
; half_step_tbl
;		andlw	0x07
;		addwf	PCL,f
;		;
;		retlw	0x08
;		retlw	0x0A
;		retlw	0x02
;		retlw	0x06
;		retlw	0x04
;		retlw	0x05
;		retlw	0x01
;		retlw	0x09
;
;
;		*****************************************************
;		*****************************************************
;		******           CODE-SPACE PAGE ONE          *******
;		******         SYSTEM PARAMETER ENTRY         *******
;		******        SERIAL INTERFACE SUPPORT        *******
;		*****************************************************
;		*****************************************************
;
		ORG		0x0800
				;
				; ****************************************
				; ****************************************
				; *****  PROGRAM_SYSTEM PARAMETERS  *****
				; ****************************************
				; ****************************************
; shutter_T1	 EQU	0x40	; delay from shutter "1st press" to "full press"
; shutter_T2	 EQU	0x41	; duration of shutter "full press"
								;
; settle_time	 EQU	0x42	; delay from motion to shutter
								;
; pan_slew_rate	 EQU	0x43	; pan fast slew rate timeout
; pan_slow_rate	 EQU	0x44	; pan start rate timeout
; pan_ramp_dist	 EQU	0x45	; pan deceleration steps
								;
; elev_slew_rate EQU	0x46	; elevation fast slew rate timeout
; elev_slow_rate EQU	0x47	; elevation start rate timeout
; elev_ramp_dist EQU	0x48	; elevation deceleration steps

; 0x00
		; " Program Sys Params " 
		; " ---  ---  ---  YES "
; 0x01
		; "Shutter 1 Time=     "	shutter_T1
		; "Prev  !dn  up^ ENTER"
; 0x02
		; "Shutter 2 Time=     "	shutter_T2
		; "Prev  !dn  up^ ENTER"
; 0x03
		; " Settling Time=     "	settle_time
		; "Prev  !dn  up^ ENTER"
; 0x04
		; " Pan Fast rate=     "	pan_fast_rate
		; "Prev  !dn  up^ ENTER"
; 0x05
		; " Pan Slow rate=     "	pan_slow_rate
		; "Prev  !dn  up^ ENTER"
; 0x06
		; " Pan Slew rate=     "	pan_ramp_dist
		; "Prev  !dn  up^ ENTER"
; 0x07
		; "Elev Fast rate=     "	elev_fast_rate
		; "Prev  !dn  up^ ENTER"
; 0x08
		; "Elev Slow rate=     "	elev_slow_rate
		; "Prev  !dn  up^ ENTER"
; 0x09
		; "Elev Slew rate=     "	elev_ramp_dist
		; "Prev  !dn  up^ ENTER"
; 0x0A
		; " Save System Params "
		; "Prev  ---  NO   YES "
;
;		*************
;		unimplemented
;		*************
; 0x0?
		; "Elev  UP limit=     "	up_limit[h:l]
		; "Prev  !dn  up^ ENTER"
; 0x0?
		; "Elev  DN limit=     "	dn_limit[h:l]
		; "Prev  !dn  up^ ENTER"
; 0x0?
		; " Pan CCW limit=     "	ccw_limit[h:l]
		; "Prev  <--  --> ENTER"
; 0x0?
		; " Pan  CW limit=     "	cw_limit[h:l]
		; "Prev  <--  --> ENTER"

		;
sys_setup_p1
		clrf	sys_state
		goto	set_sys_state
		;
sys_loop
		nop
do_generic
		call	generic_output_p1	
		call	read_sw_state_p1
		btfsc	sw_state,SWITCH_4	; "ENTER"
		 goto	set_next_sys_state
		btfsc	sw_state,SWITCH_1	; "Prev"
		 goto	set_prev_sys_state
		btfsc	sw_state,SWITCH_3	; " up^ "
		 goto	generic_increment
		btfsc	sw_state,SWITCH_2	; " !dn "
		 goto	generic_decrement
		goto	sys_loop
						;
set_prev_sys_state
		movf	sys_state,f
		btfss	STATUS,Z
		 decf	sys_state,f
		goto	set_sys_state
						;
set_next_sys_state
		movlw	MAX_SYS_STATE
		subwf	sys_state,w
		btfsc	STATUS,C
		 goto	set_sys_end_state
		incf	sys_state,f
;		goto	set_sys_state
set_sys_state		
		movlw	HIGH sys_state_set_table		; set up PC high bits
		movwf	PCLATH
		movf	sys_state,w				; get current state
		andlw	0x0F
		addwf	PCL,f
sys_state_set_table
;		goto	set_sys_prog_state		; 0x00		; " Program Sys Params "
;		goto	set_e_step_mode_state	; 0x01		; "Elev Step mode=     "	e_m_control
;		goto	set_p_step_mode_state	; 0x02		; " Pan Step mode=     "	p_m_control
;		goto	set_e_slew_state		; 0x03		; "Elev Slew rate=     "	elev_slew_rate
;		goto	set_p_slew_state		; 0x04		; " Pan Slew rate=     "	pan_slew_rate
;		goto	set_sys_settle_state	; 0x05		; " Settling Time=     "	settle_time
;		goto	set_shutter_t1_state	; 0x06		; "Shutter 1 Time=     "	shutter_T1
;		goto	set_shutter_t2_state	; 0x07		; "Shutter 2 Time=     "	shutter_T2
;		goto	set_save_sys_state		; 0x08		; " Save System Params "
;		goto	set_error_state			; 0x09
;		goto	set_error_state			; 0x0A
;		goto	set_error_state			; 0x0B
;		goto	set_error_state			; 0x0C
;		goto	set_error_state			; 0x0D
;		goto	set_error_state			; 0x0E
;		goto	set_error_state			; 0x0F
;
		goto	set_sys_prog_state		; 0x00		; " Program Sys Params "
		goto	set_shutter_t1_state	; 0x01		; "Shutter 1 Time=     "	shutter_T1
		goto	set_shutter_t2_state	; 0x02		; "Shutter 2 Time=     "	shutter_T2
		goto	set_sys_settle_state	; 0x03		; " Settling Time=     "	settle_time
		goto	set_p_fast_state		; 0x04		; " Pan Slew rate=     "	pan_slew_rate
		goto	set_p_slow_state		; 0x05		; " Pan Slow rate=     "    pan slow start rate
		goto	set_p_ramp_state		; 0x06		; " Pan Ramp step=     "    pan decel ramp start
		goto	set_e_fast_state		; 0x07		; "Elev Slew rate=     "	elev_slew_rate
		goto	set_e_slow_state		; 0x08		; "Elev Slow rate=     "    elev slow start rate
		goto	set_e_ramp_state		; 0x09		; "Elev Ramp step=     "    elev decel ramp start
		goto	set_save_sys_state		; 0x0A		; " Save System Params "
													;
		goto	set_error_state			; 0x0B
		goto	set_error_state			; 0x0C
		goto	set_error_state			; 0x0D
		goto	set_error_state			; 0x0E
		goto	set_error_state			; 0x0F
													;
MAX_SYS_STATE	EQU	0x0A
;								; unimplemented
;		goto	set_e_up_limit_state	; 0x09		; "Elev  UP limit=     "	up_limit[h:l]
;		goto	set_e_dn_limit_state	; 0x0A		; "Elev  DN limit=     "	dn_limit[h:l]

;		goto	set_p_ccw_limit_state	; 0x0B		; " Pan  CW limit=     "	cw_limit[h:l]
;		goto	set_p_cw_limit_state	; 0x0C		; " Pan CCW limit=     "	ccw_limit[h:l]

;		goto	set_error_state			; 0x0D
;		goto	set_error_state			; 0x0E
;		goto	set_error_statet		; 0x0F

		;
		; *******************************************************
		; *  param_mode                                         *
		; * |    d7   |   d6   | d5  | d4 | d3 | d2 | d1 | d0 | *
		; * | supress | 16-bit | 10s | s  | ms | x  | x  | x  | *
		; *******************************************************

set_sys_prog_state		; 0x00		; " Program Sys Params "
		bsf		param_mode,7		; supress output
		goto	set_sys_state_exit
									;
set_shutter_t1_state	; 0x01		; "Shutter 1 Time=     "	shutter_T1
		clrf	param_mode			; enable display
		bsf		param_mode,4		; show value as "x.xx s"
		movlw	shutter_T1
		movwf	FSR
		goto	set_sys_state_exit
									;
set_shutter_t2_state	; 0x02		; "Shutter 2 Time=     "	shutter_T2
		clrf	param_mode			; enable display
		bsf		param_mode,4		; show value as "x.xx s"
		movlw	shutter_T2
		movwf	FSR
		goto	set_sys_state_exit
set_sys_settle_state	; 0x03		; " Settling Time=     "	settle_time
		clrf	param_mode			; enable display
		bsf		param_mode,4		; show value as "x.xx s"
		movlw	settle_time
		movwf	FSR
		goto	set_sys_state_exit
set_p_fast_state		; 0x04		; " Pan Fast rate=     "	pan_fast_rate
		clrf	param_mode			; enable display
		bsf		param_mode,3		; show value as "xxx ms"
		movlw	pan_fast_rate
		movwf	FSR
		goto	set_sys_state_exit
set_p_slow_state		; 0x05		; " Pan Slow rate=     "	pan_slow_rate
		clrf	param_mode			; enable display
		bsf		param_mode,3		; show value as "xxx ms"
		movlw	pan_slow_rate
		movwf	FSR
		goto	set_sys_state_exit
set_p_ramp_state		; 0x06		; " Pan Ramp step=     "	pan_ramp_step
		clrf	param_mode			; enable display
;		bcf		param_mode,3		; show value as "xxx"
		movlw	pan_ramp_dist
		movwf	FSR
		goto	set_sys_state_exit
set_e_fast_state		; 0x07		; "Elev Slew rate=     "	elev_fast_rate
		clrf	param_mode			; enable display
		bsf		param_mode,3		; show value as "xxx ms"
		movlw	elev_fast_rate
		movwf	FSR
		goto	set_sys_state_exit
set_e_slow_state		; 0x08		; "Elev Slow rate=     "	elev_slow_rate
		clrf	param_mode			; enable display
		bsf		param_mode,3		; show value as "xxx ms"
		movlw	elev_slow_rate
		movwf	FSR
		goto	set_sys_state_exit
set_e_ramp_state		; 0x09		; "Elev Ramp step=     "	elev_ramp_step
		clrf	param_mode			; enable display
;		bcf		param_mode,3		; show value as "xxx"
		movlw	elev_ramp_dist
		movwf	FSR
		goto	set_sys_state_exit
set_save_sys_state		; 0x08		; " Save System Params "
		bsf		param_mode,7		; supress display
		goto	set_sys_state_exit
;
;		*******************************************************************
;		***  Save new sys_state, output new screen, wait for button release ***
;		*******************************************************************
set_sys_state_exit
		movlw	HIGH set_state_text_p2		; set up PC high bits
		movwf	PCLATH
		call	set_sys_state_text_p2		; output prompt text 
		movlw	HIGH set_sys_state_exit		; set up PC high bits
		movwf	PCLATH
set_sys_state_1
		call	read_sw_state_p1
		btfss	STATUS,Z		; switch released yet ?
		 goto	set_sys_state_1	; no, wait for release
		goto	sys_loop
;
set_sys_end_state
;		movlw	HIGH save_sys_param		; set up PC high bits
;		movwf	PCLATH
		call	save_sys_param_p1		; save system parameters to EEPROM
;		goto	main2
		movlw	HIGH main2		; set up PC high bits
		movwf	PCLATH
		goto	main2
;		return
				;
				; ********************************
				; ** generic increment of INDF  **
				; ********************************
generic_increment
		btfsc	param_mode,7	; surpress variable modification
		 goto	sys_loop		; yes, return to main loop
		movlw	JOG_DELAY		; set delay time to jog
		movwf	timeout
generic_incr_1
		movlw	MAX_HOLD_TIME		; test for maximum
		subwf	INDF,w
		btfss	STATUS,C
		 incf	INDF,f			; if <MAX then increment
		call	generic_output_p1	; display it
generic_incr_2
		call	read_sw_state_p1
		btfsc	STATUS,Z		; switch released yet ?
		 goto	sys_loop		; yes, return to main loop
								;
		movf	timeout,w		; test timeout
		btfss	STATUS,Z		; has delay timed out
		 goto	generic_incr_2	; no, wait for timeout
								;
		movlw	STEP_DELAY1		; set step repetition delay
		movwf	timeout
		goto	generic_incr_1
				;
				; ********************************
				; ** generic decrement of INDF  **
				; ********************************
generic_decrement
		btfsc	param_mode,7	; surpress variable modification
		 goto	sys_loop		; yes, return to main loop
		movlw	JOG_DELAY		; set delay time to jog
		movwf	timeout
generic_decr_1
		movf	INDF,f			; test for zero
		btfss	STATUS,Z
		 decf	INDF,f			; if >0 then decrement
		call	generic_output_p1	; display it
generic_decr_2
		call	read_sw_state_p1
		btfsc	STATUS,Z		; switch released yet ?
		 goto	sys_loop		; yes, return to main loop
								;
		movf	timeout,w		; test timeout
		btfss	STATUS,Z		; has delay timed out
		 goto	generic_decr_2	; no, wait for timeout
								;
		movlw	STEP_DELAY1		; set step repetition delay
		movwf	timeout
		goto	generic_decr_1
					;
					; ********************************
					; ****  Read switch state_p1  ****
					; ********************************
read_sw_state_p1
		swapf	SW_PORT,w	; get current switch state
		xorlw	0x0f		; compliment lsbs
		andlw	0x0f		; mask msbs
		movwf	sw_temp		; save switch state
							;
		movlw	#2
		movwf	timeout2
read_sw_p1_0
		movf	timeout2,f
		btfss	STATUS,Z
		 goto	read_sw_p1_0	
						;
		movf	sw_state,w	; return with switch state
		xorwf	sw_temp,w
		btfsc	STATUS,Z
		 goto	read_sw_p1_1
		movf	sw_temp,w
		movwf	sw_state
		goto	read_sw_state_p1
read_sw_p1_1
		movf	sw_state,w
		return
				;
				; ***********************
				; ** generic_output_p1 **
				; ***********************
generic_output_p1
		movlw	HIGH generic_output		; set up PC high bits
		movwf	PCLATH
		call	generic_output
		movlw	HIGH generic_output_p1		; set up PC high bits
		movwf	PCLATH
		return	
;
;
; ***********************
; ***********************
; ** EEPROM GET / SAVE **
; ***********************
; ***********************
					;					
					; ****************************************
					; ****  get program type from EEPROM  ****
					; ****************************************
					; *  return first byte at Prog_num in w  *
					; ****************************************
check_program_p1
		bcf		INTCON,GIE		; turn off interrupts
		call	get_EE_addr		; calculate EEPROM address for bank
								;
		bsf		STATUS,RP1		; go to register bank 2
		bcf		STATUS,RP0 
		movwf	EEADR			; write to EEPROM address
								;
		bsf		STATUS,RP0		; register bank 3
		bcf		EECON1,EEPGD	; set data bank
		bsf		EECON1,RD		; start read
		bcf		STATUS,RP0		; back to bank 2
		movf	EEDATA,w		; get EEPROM byte
								;
		bcf		STATUS,RP1		; return to register bank 0
		bcf		STATUS,RP0 
		bsf		INTCON,GIE		; re-enable interrupts
		return
					;					
					; *******************************************
					; ****  Load motion program from EEPROM  ****
					; *******************************************
load_program_p1
		bcf		INTCON,GIE		; turn off interrupts
					;
		movlw	CUR_PROG_PARAM_START	; get #address of start of parameter list
		movwf	FSR				; set indirect address for table
								;
		call	get_EE_addr		; calculate EEPROM address for bank
								;
		bsf		STATUS,RP1		; go to register bank 2
		bcf		STATUS,RP0 
		movwf	EEADR			; write to EEPROM address
				;
		movlw	PROG_PARAM_SIZE
		movwf	tempa			; set up copy count
load_pgm_1
		bsf		STATUS,RP0		; register bank 3
		bcf		EECON1,EEPGD	; set data bank
		bsf		EECON1,RD		; start read
		bcf		STATUS,RP0		; back to bank 2
		movf	EEDATA,w		; get EEPROM byte
		movwf	INDF			; write to PARAMETER block
								;
		incf	EEADR,f			; step to next table entry
		incf	FSR,f
		decfsz	tempa,f			; count down block length
		 goto	load_pgm_1		; copy next byte
								;
		bcf		STATUS,RP1		; return to register bank 0
		bcf		STATUS,RP0 
		bsf		INTCON,GIE		; re-enable interrupts
		return
					;					
					; *****************************************
					; ****  Save motion program to EEPROM  ****
					; *****************************************
save_program_p1
		bcf		INTCON,GIE		; turn off interrupts
					;
		movlw	CUR_PROG_PARAM_START	; get #address of start of parameter list
		movwf	FSR				; set indirect address for table
								;
		call	get_EE_addr		; calculate EEPROM address for bank
								;
		bsf		STATUS,RP1		; go to register bank 2
		bcf		STATUS,RP0 
		movwf	EEADR			; write to EEPROM address
				;
		movlw	PROG_PARAM_SIZE
		movwf	tempa			; set up copy count
save_pgm_1
		movf	INDF,w			; get data from PARAMETER block
		movwf	EEDATA			; put in EEPROM data register
		bsf		STATUS,RP0		; go to register bank 3
		bcf		EECON1,EEPGD	; set data bank
		bsf		EECON1,WREN		; enable EEPROM write
		movlw	0x55			; do write sequence
		movwf	EECON2			;
		movlw	0xAA			; do write sequence
		movwf	EECON2			;
		bsf		EECON1,WR		; set write bit
save_pgm_2
		btfsc	EECON1,WR		; test for write complete
		 goto	save_pgm_2
								;
		bcf		EECON1,WREN		; disable write
		bcf		STATUS,RP0		; back to bank 2
		incf	EEADR,f			; step to next table entry
		incf	FSR,f
		decfsz	tempa,f			; count down block length
		 goto	save_pgm_1		; copy next byte
								;
		bcf		STATUS,RP1		; return to register bank 0
		bcf		STATUS,RP0 
		bcf		PIR2,EEIF		; reset EEPROM write complete flag
		bsf		INTCON,GIE		; re-enable interrupts
		return
					;
					; calculate EEPROM address for program
get_EE_addr			
		movf	prog_num,w		; from "prog_num" * PROG_PARAM_SIZE
		movwf	tempa			; returns with address in w
		clrw
get_EE_addr_1
		movf	tempa,f
		btfsc	STATUS,Z
		 return
		addlw	PROG_PARAM_SIZE
		decf	tempa,f
		goto	get_EE_addr_1
					;
					;					
					; **********************************************
					; ****  Load System Parameters from EEPROM  ****
					; **********************************************
load_sys_param_p1
		bcf		INTCON,GIE		; turn off interrupts
					;
		movlw	SYS_PARAM_START	; get #address of start of parameter list
		movwf	FSR				; set indirect address for table
								;
		movlw	SYS_EE_ADDR		; base of system parameters
		bsf		STATUS,RP1		; go to register bank 2
		bcf		STATUS,RP0 
		movwf	EEADR			; write to EEPROM address
				;
		movlw	SYS_PARAM_SIZE
		movwf	tempa			; set up copy count
load_sys_1
		bsf		STATUS,RP0		; register bank 3
		bcf		EECON1,EEPGD	; set data bank
		bsf		EECON1,RD		; start read
		bcf		STATUS,RP0		; back to bank 2
		movf	EEDATA,w		; get EEPROM byte
		movwf	INDF			; write to PARAMETER block
								;
		incf	EEADR,f			; step to next table entry
		incf	FSR,f
		decfsz	tempa,f			; count down block length
		 goto	load_sys_1		; copy next byte
								;
		bcf		STATUS,RP1		; return to register bank 0
		bcf		STATUS,RP0 
		bsf		INTCON,GIE		; re-enable interrupts
		return
					;
					; *****************************************
					; ****  Save motion program to EEPROM  ****
					; *****************************************
save_sys_param_p1
		bcf		INTCON,GIE		; turn off interrupts
					;
		movlw	SYS_PARAM_START	; get address of start of sys parameters
		movwf	FSR				; set indirect address for table
								;
		movlw	SYS_EE_ADDR		; base of system parameters
		bsf		STATUS,RP1		; go to register bank 2
		bcf		STATUS,RP0 
		movwf	EEADR			; write to EEPROM address
				;
		movlw	SYS_PARAM_SIZE
		movwf	tempa			; set up copy count
save_sys_1
		movf	INDF,w			; get data from PARAMETER block
		movwf	EEDATA			; put in EEPROM data register
		bsf		STATUS,RP0		; go to register bank 3
		bcf		EECON1,EEPGD	; set data bank
		bsf		EECON1,WREN		; enable EEPROM write
		movlw	0x55			; do write sequence
		movwf	EECON2			;
		movlw	0xAA			; do write sequence
		movwf	EECON2			;
		bsf		EECON1,WR		; set write bit
save_sys_2
		btfsc	EECON1,WR		; test for write complete
		 goto	save_sys_2
								;
		bcf		EECON1,WREN		; disable write
		bcf		STATUS,RP0		; back to bank 2
		incf	EEADR,f			; step to next table entry
		incf	FSR,f
		decfsz	tempa,f			; count down block length
		 goto	save_sys_1		; copy next byte
								;
		bcf		STATUS,RP1		; return to register bank 0
		bcf		STATUS,RP0 
		bcf		PIR2,EEIF		; reset EEPROM write complete flag
		bsf		INTCON,GIE		; re-enable interrupts
		return
;
; ********************************
; ********************************
; ****  Serial port handlers  ****
; ********************************
; ********************************
		;
		; ****************************
		; *  process receive buffer  *
		; ****************************
		;
rcv_cmd_p1
		movlw	rxbuf		; point to start of buffer
		movwf	FSR			; set indirect address
		;
		; test for "version" command (v)
		;
		movf	INDF,w		; get first byte of buffer
		xorlw	'v'			; is it version command ?
		btfss	STATUS,Z
		 goto	rcv_cmd_01
			;
			; show current software version
		call	s_version_txt
		goto	rcv_cmd_exit
		;
		; test for "keypad lock/unlock" command (kl / ku)
		;
rcv_cmd_01
		movf	INDF,w		; get first byte of buffer
		xorlw	'k'			; is it "keypad lock/unlock" command ?
		btfss	STATUS,Z
		 goto	rcv_cmd_02
			;
		incf	FSR,f
		movf	INDF,w		; get 2nd char
		xorlw	'l'			; is it "keypad lock" command ?
		btfss	STATUS,Z
		 goto	rcv_cmd_01a
			;
			; insert keypad lock code
		bsf		keypad_lock,7	; lock out keypad press
			;
		movlw	HIGH keypad_locked_txt_p2	; set up PC high bits
		movwf	PCLATH
		call	keypad_locked_txt_p2
		movlw	HIGH rcv_cmd_exit	; restore PC high bits
		movwf	PCLATH
		goto	rcv_cmd_exit
rcv_cmd_01a
		movf	INDF,w		; get 2nd char
		xorlw	'u'			; is it "keypad unlock" command ?
		btfss	STATUS,Z
		 goto	rcv_cmd_err
			;
			; insert keypad unlock code
		bcf		keypad_lock,7	; unlock keypad
		clrf	rbufptr		; enable next message
		call	prompt_txt	; output next prompt
			;
		bsf		keypad_lock,5	; go back to main page
;		movlw	HIGH main2	; restore PC high bits
;		movwf	PCLATH
;		goto	main2
		goto	rcv_cmd_exit
		;
		; test for "get position" command (gp / ge)
		;
rcv_cmd_02
		movf	INDF,w		; get first byte of buffer
		xorlw	'g'			; is it "get" command ?
		btfss	STATUS,Z
		 goto	rcv_cmd_03
			;
		incf	FSR,f
		movf	INDF,w		; get 2nd char
		xorlw	'p'			; is it get_pan_position command ?
		btfss	STATUS,Z
		 goto	rcv_cmd_02a
			; show current pan position
		movf	pan_cur_pos_h,w	; put value in minuend
		movwf	minuend_h
		movf	pan_cur_pos_l,w
		movwf	minuend_l
		call	SER_OUTSD16
		goto	rcv_cmd_exit
rcv_cmd_02a
		movf	INDF,w		; get 2nd char
		xorlw	'e'			; is it get_elevation_position command ?
		btfss	STATUS,Z
		 goto	rcv_cmd_02b
			; show current elevation position
		movf	elev_cur_pos_h,w	; put value in minuend
		movwf	minuend_h
		movf	elev_cur_pos_l,w
		movwf	minuend_l
		call	SER_OUTSD16
		goto	rcv_cmd_exit
		;
rcv_cmd_02b
		movf	INDF,w		; get 2nd char
		xorlw	's'			; is it get_elevation_position command ?
		btfss	STATUS,Z
		 goto	rcv_cmd_err
			; show current system state variable
		clrf	minuend_h
		movf	sys_state,w
		movwf	minuend_l
		call	SER_OUTD8
		goto	rcv_cmd_exit
		;
		; test for shutter or switch cmd command (sh / sw)
		;
rcv_cmd_03
		movf	INDF,w		; get first byte of buffer
		xorlw	's'			; is it "shutter" or "switch" command ?
		btfss	STATUS,Z
		 goto	rcv_cmd_04
			;
		incf	FSR,f
		movf	INDF,w		; get 2nd char
		xorlw	'h'			; is it "shutter" command ?
		btfss	STATUS,Z
		 goto	rcv_cmd_03a
			; actuate camera shutter
		movlw	HIGH press_shutter	; set up PC high bits
		movwf	PCLATH
		call	press_shutter
		movlw	HIGH rcv_cmd_exit	; restore PC high bits
		movwf	PCLATH
		goto	rcv_cmd_exit
			;
rcv_cmd_03a
		movf	INDF,w		; get 2nd char
		xorlw	'w'			; is it "set_switch" command ?
		btfss	STATUS,Z
		 goto	rcv_cmd_err
			; actuate remote push-button
		clrf	sw_state
		incf	FSR,f
		call	getnum		; what switch ?
		movf	rcv_num_l,w
		xorlw	1
		btfsc	STATUS,Z
		 bsf	sw_state,0
		movf	rcv_num_l,w
		xorlw	2
		btfsc	STATUS,Z
		 bsf	sw_state,1
		movf	rcv_num_l,w
		xorlw	3
		btfsc	STATUS,Z
		 bsf	sw_state,2
		movf	rcv_num_l,w
		xorlw	4
		btfsc	STATUS,Z
		 bsf	sw_state,3
		bsf		keypad_lock,6
		goto	rcv_cmd_exit
		;
		; test for display or debug cmd (sh / sw)
		;
rcv_cmd_04
		movf	INDF,w		; get first byte of buffer
		xorlw	'd'			; is it "display" command ?
		btfss	STATUS,Z
		 goto	rcv_act_cmd
			;
		incf	FSR,f
		movf	INDF,w		; get 2nd char
		xorlw	'p'			; is it "display program" command ?
		btfss	STATUS,Z
		 goto	rcv_cmd_04a
			; output program
		call	ser_out_prog
		goto	rcv_cmd_exit
		;
rcv_cmd_04a
		movf	INDF,w		; get 2nd char
		xorlw	'b'			; is it "debug" command ?
		btfss	STATUS,Z
		 goto	rcv_cmd_err
			; output debug stream
		call	ser_out_debug
		goto	rcv_cmd_exit
		;
		; the following are action commands 
		; they can only be executed if
		; the keypad is locked
		;
rcv_act_cmd
		;
		btfss	keypad_lock,7
		 goto	 rcv_cmd_lock_msg
		; test for pan positioning commands (pa / pr)
		;
rcv_act_cmd_00
		movf	INDF,w		; get first byte of buffer
		xorlw	'p'			; is it "pan" command ?
		btfss	STATUS,Z
		 goto	rcv_act_cmd_01
			;
		incf	FSR,f
		movf	INDF,w		; get 2nd char
		xorlw	'a'			; is it "move absolute" command ?
		btfss	STATUS,Z
		 goto	rcv_act_cmd_00a
			; extract number
		incf	FSR,f
		call	getnum
			; move to absolute pan position
		movf	rcv_num_h,w
		movwf	new_pos_h
		movf	rcv_num_l,w
		movwf	new_pos_l
		movlw	HIGH pan_new_position	; set up PC high bits
		movwf	PCLATH
		call	pan_new_position
		movlw	HIGH rcv_cmd_exit		; restore PC high bits
		movwf	PCLATH
			;
		goto	rcv_cmd_exit
rcv_act_cmd_00a
		movf	INDF,w		; get 2nd char
		xorlw	'r'			; is it "move relative" command ?
		btfss	STATUS,Z
		 goto	rcv_cmd_err
			; extract number
		incf	FSR,f
		call	getnum
			; step to relative pan position
		movf	pan_cur_pos_h,w
		addwf	rcv_num_h,w
		movwf	new_pos_h
		movf	pan_cur_pos_l,w
		addwf	rcv_num_l,w
		movwf	new_pos_l
		btfsc	STATUS,C
		 incf	 new_pos_h,f
		movlw	HIGH pan_new_position	; set up PC high bits
		movwf	PCLATH
		call	pan_new_position
		movlw	HIGH rcv_cmd_exit		; restore PC high bits
		movwf	PCLATH
		goto	rcv_cmd_exit
		;
		; test for elevation positioning commands (ea / er)
		;
rcv_act_cmd_01
		movf	INDF,w		; get first byte of buffer
		xorlw	'e'			; is it "elevation" command ?
		btfss	STATUS,Z
		 goto	rcv_cmd_err
			;
		incf	FSR,f
		movf	INDF,w		; get 2nd char
		xorlw	'a'			; is it "move absolute" command ?
		btfss	STATUS,Z
		 goto	rcv_act_cmd_01a
			; extract number
		incf	FSR,f
		call	getnum
			; move to absolute elevation position
		movf	rcv_num_h,w
		movwf	new_pos_h
		movf	rcv_num_l,w
		movwf	new_pos_l
		movlw	HIGH elev_new_position	; set up PC high bits
		movwf	PCLATH
		call	elev_new_position
		movlw	HIGH rcv_cmd_exit		; restore PC high bits
		movwf	PCLATH
		goto	rcv_cmd_exit
rcv_act_cmd_01a
		movf	INDF,w		; get 2nd char
		xorlw	'r'			; is it "move relative" command ?
		btfss	STATUS,Z
		 goto	rcv_cmd_err
			; extract number
		incf	FSR,f
		call	getnum
			; step to relative elevation position
		movf	elev_cur_pos_h,w
		addwf	rcv_num_h,w
		movwf	new_pos_h
		movf	elev_cur_pos_l,w
		addwf	rcv_num_l,w
		movwf	new_pos_l
		btfsc	STATUS,C
		 incf	 new_pos_h,f
		movlw	HIGH elev_new_position	; set up PC high bits
		movwf	PCLATH
		call	elev_new_position
		movlw	HIGH rcv_cmd_exit		; restore PC high bits
		movwf	PCLATH
		goto	rcv_cmd_exit

		;
		; "lock keypad first" message
		;
rcv_cmd_lock_msg
		call	must_lock_keypad_txt
		goto	rcv_cmd_exit
		;
		; process invalid commands
		;
rcv_cmd_err
		movlw	0x07	; "beep"
		call	OUT_UART_p1
		movlw	'?'
		call	OUT_UART_p1
rcv_cmd_exit
		clrf	rbufptr		; enable next message
		call	prompt_txt	; output next prompt
		return
;		movlw	HIGH main_loop	; restore PC high bits
;		movwf	PCLATH
;		goto	main_loop
;
getnum16
		return
		;
		;	look for signed decimal number in receive buffer
		;	return with 16-bit hexidecimal # in "rcv_num[h:l]"
		;	if no number found, return with zero
getnum
		clrf	rsign
		clrf	rdigit3
		clrf	rdigit2
		clrf	rdigit1
		clrf	rdigit0
		clrf	rcv_num_h
		clrf	rcv_num_l
		;
getnum_00
		call	get_digit
		btfss	rcv_digit,7		; is current char a digit
		 goto	 getnum_01		; yes, process it
		movf	INDF,w
		xorlw	'-'				; is it a negative sign ?
		btfsc	STATUS,Z
		 bsf	 rsign,7
		movf	INDF,w
		xorlw	EOL_CHAR		; last character in buffer ?
		btfsc	STATUS,Z
		 return
		incf	FSR,f
		btfsc	FSR,4			; at end of buffer ?
		 return
		goto	getnum_00
		;
		; copy number into right-justified 4-digit digit-buffer
getnum_01
		call	get_digit
		btfsc	rcv_digit,7
		 goto	 getnum_02
		;
		movf	rdigit2,w
		movwf	rdigit3
		movf	rdigit1,w
		movwf	rdigit2
		movf	rdigit0,w
		movwf	rdigit1
		movf	rcv_digit,w
		movwf	rdigit0
		;
		incf	FSR,f
		goto	getnum_01
		;
		; convert digit-buffer to 16-bit hex
		;
		; add in "thousands" digit
getnum_02
		movf	rdigit3,f
		btfsc	STATUS,Z
		 goto	 getnum_03
		movlw	LOW	THOUSAND
		addwf	rcv_num_l,f
		btfsc	STATUS,C
		 incf	rcv_num_h,f
		movlw	HIGH THOUSAND
		addwf	rcv_num_h,f
		decfsz	rdigit3,f
		 goto	 getnum_02
		;
		; add in "hundreds" digit
getnum_03
		movf	rdigit2,f
		btfsc	STATUS,Z
		 goto	 getnum_04
		movlw	LOW	HUNDRED
		addwf	rcv_num_l,f
		btfsc	STATUS,C
		 incf	rcv_num_h,f
		movlw	HIGH HUNDRED
		addwf	rcv_num_h,f
		decfsz	rdigit2,f
		 goto	 getnum_03
		;
		; add in "tens" digit
getnum_04
		movf	rdigit1,f
		btfsc	STATUS,Z
		 goto	 getnum_05
		movlw	TEN
		addwf	rcv_num_l,f
		btfsc	STATUS,C
		 incf	rcv_num_h,f
		decfsz	rdigit1,f
		 goto	 getnum_04
		;
		; add in last digit
getnum_05
		movf	rdigit0,w
		addwf	rcv_num_l,f
		btfsc	STATUS,C
		 incf	rcv_num_h,f
		btfss	rsign,7			; is it negative
		 return					; no, return
		;
		; make it into a negative number
		comf	rcv_num_h,f	
		comf	rcv_num_l,f
		movlw	1
		addwf	rcv_num_l,f
		btfsc	STATUS,C
		 incf	rcv_num_h,f
		return
		;
		;	check receive buffer for digit (0x30-0x39)
		;	if valid, strip lsb's and returns (0-9)
		;	otherwise, mark as non digit bysetting msb
get_digit
		clrf	rcv_digit
		bsf		rcv_digit,7	; mark as non_digit
		movf	INDF,w		; get current char
		andlw	0xF0		; look at MSBs
		xorlw	0x30
		btfss	STATUS,Z	; is it 30-3F ?
		 return				; return as bad digit
		movf	INDF,w		; get current char
		andlw	0x0F		; mask LSBs
		movwf	rcv_digit	; save
		movlw	0x0a		; test for greater than 9
		subwf	rcv_digit,w
		btfsc	STATUS,C
		 bsf	 rcv_digit,7 ; mark as non_digit		
		return
;
;	display current program
;
ser_out_prog
		movlw	0x50
		movwf	FSR
		call	ser_out_fsr_8	; prog_type
		call	ser_out_fsr_16	; elev_start_pos
		call	ser_out_fsr_16	; elev_step_size
		call	ser_out_fsr_8	; elev_step_ct
		call	ser_out_fsr_16	; pan_start_pos
		call	ser_out_fsr_16	; pan_step_size
		call	ser_out_fsr_8	; pan_step_ct
		call	ser_out_fsr_8_3d	; hold_time
		return
;
ser_out_fsr_16
		movf	INDF,w
		movwf	minuend_h
		incf	FSR,f
		movf	INDF,w
		movwf	minuend_l
		incf	FSR,f
		call	SER_OUTSD16
		goto	SER_OUTS
;
ser_out_fsr_8
		clrf	minuend_h
		movf	INDF,w
		movwf	minuend_l
		incf	FSR,f
		call	SER_OUTD8
		goto	SER_OUTS
;
ser_out_fsr_8_3d
		clrf	minuend_h
		movf	INDF,w
		movwf	minuend_l
		incf	FSR,f
		call	SER_OUTD8_3D
		goto	SER_OUTS
;
ser_outh_fsr_16
		movf	INDF,w
		call	SER_OUTHX
		incf	FSR,f
ser_outh_fsr_8
		movf	INDF,w
		call	SER_OUTHS
		incf	FSR,f
		return
;
;	debug display of all user and device registers
;
ser_out_debug
		movlw	0x20
		movwf	FSR
ser_debug1
		movf	FSR,w
		andlw	0x0F
		btfss	STATUS,Z
		 goto	 ser_debug1a
		movlw	0x0d
		call	OUT_UART_p1
		movlw	0x0a
		call	OUT_UART_p1
		movlw	0x00
		call	SER_OUTHX
		movf	FSR,w
		call	SER_OUTHX
ser_debug1a
		movf	FSR,w
		andlw	0x03
		btfss	STATUS,Z
		 goto	 ser_debug2
		call	SER_OUTS
ser_debug2
		movf	INDF,w
		call	SER_OUTHS
		incfsz	FSR,f
		 goto	 ser_debug3
		return
ser_debug3
		movf	FSR,w
		xorlw	0x80
		btfss	STATUS,Z
		 goto	 ser_debug1
		movlw	0x0d
		call	OUT_UART_p1
		movlw	0x0a
		call	OUT_UART_p1
		movlw	0xA0
		movwf	FSR
		goto	ser_debug1


		;
		; *****************************
		; *  Serial output processes  *
		; *****************************
		;
		; **********************************
		; *  signed 16 bit decimal output  *
		; *      value in minuend[h:l]     *
		; **********************************
SER_OUTSD16
		btfss	minuend_h,7		; is MSB set
		goto	ser_outsd16p	; no, output "+" and unsigned value
;			negative number
		movlw	'-'
		call	OUT_UART_p1
		comf	minuend_h,f
		comf	minuend_l,f
		movlw	1
		addwf	minuend_l,f
		btfsc	STATUS,C
		 incf	minuend_h,f
		goto	SER_OUTD16
;
ser_outsd16p	
		movlw	'+'
		call	OUT_UART_p1
		;
		; ************************************
		; *  unsigned 16 bit decimal output  *
		; *       value in minuend[h:l]      *
		; ************************************
SER_OUTD16
				; calculate thousands
		movlw	high(THOUSAND)
		movwf	subtrahend_h
		movlw	low(THOUSAND)
		movwf	subtrahend_l
		call	ser_calc_digit
SER_OUTD8_3D	; calculate hundreds
		movlw	high(HUNDRED)
		movwf	subtrahend_h
		movlw	low(HUNDRED)
		movwf	subtrahend_l
		call	ser_calc_digit
SER_OUTD8		; calculate tens		
		movlw	high(TEN)
		movwf	subtrahend_h
		movlw	low(TEN)
		movwf	subtrahend_l
		call	ser_calc_digit
				; display ones
		movf	minuend_l,w		;
		goto	SER_OUTHD		; output digit and return
				;
				; calculate display digit by repeated subtraction
				;
ser_calc_digit
		clrf	digit_ctr		; set up counter
ser_calc_dig1
		movf	subtrahend_l,w	; get subtrahend LSB
		subwf	minuend_l,w		; subtract from minuend LSB
		movwf	tempm_l			; save in temp minuend LSB
								;
		movf	subtrahend_h,w	; get subtrahend MSB
		btfss	STATUS,C		; test if previous subtraction generated a carry
		 incfsz	subtrahend_h,w	; if subtractive carry (C=0) then subtract 1 more
		  subwf	minuend_h,w		; subtract from minuend unless it was zero
		movwf	tempm_h			; save in temp minuend MSB
								;
		btfss	STATUS,C		; test if final subtraction generated a carry
		goto	ser_calc_dig2		; if subtractive carry(C=0) then output digit
								;
		movf	tempm_h,w		; if not yet underflowing copy temp to minuend
		movwf	minuend_h		;
		movf	tempm_l,w		;
		movwf	minuend_l		;
								;
		incfsz	digit_ctr,f		; bump digit count - shouldn't reach zero
		goto	ser_calc_dig1	; if non-zero repeat process until underflow
								;
ser_calc_dig2						; underflow attained
		movf	digit_ctr,w		; get digit count
		goto	SER_OUTHD			; output digit and return
;		**********************************************************************
;		** Output the byte in accum "W" as 2 hex digits followed by a space **
;		**     800uS                                                        **
;		**********************************************************************
SER_OUTHS		
		call	SER_OUTHX		; output byte as hex
		goto	SER_OUTS		; output space
;					;
;		**************************************************
;		** Output the byte in accum "W" as 2 hex digits **
;		**************************************************
SER_OUTHX		
		movwf	tempa		; need to save value in tempa
		swapf	tempa,0		; get upper 4-bits into lsbs of W
		call	SER_OUTHD	; output most-significant hex digit
		movf	tempa,0		; get the original byte back
;					; fall through to output ls-digit
;
;		***********************************************************
;		** Output a hex digit from the LSBs of the contents of W **
;		***********************************************************
SER_OUTHD		
		andlw	0x0f		; mask off the MSBs
		movwf	tempb		; save copy
		movlw	0x0a		; subtract 10 decimal
	 	subwf	tempb,1		; see if it is decimal
		movf	tempb,0
		btfss	tempb,7		; did it underflow ?
SER_OUTHD1	 	
		addlw	0x07		; shift to A-F if 10 or greater
		nop
		addlw	0x03A		; add hex 30 plus the original 10 decimal
		goto	OUT_UART_p1		; output '0-9' or 'A-F' and return
;
;		********************
;		** Output a space **
;		********************
SER_OUTS
		movlw	0x20
OUT_UART_p1
		btfss	PIR1,TXIF	; is transmit buffer empty ?
		  goto	OUT_UART_p1	; if not, wait until it is				;
		movwf	TXREG		; send character
		return
;		
;
; *********************************
; ****  Serial output strings  ****
; *********************************
;
serial_greeting_p1
		movlw	'D'
		call	OUT_UART_p1
		movlw	'i'
		call	OUT_UART_p1
		movlw	'g'
		call	OUT_UART_p1
		movlw	'i'
		call	OUT_UART_p1
		movlw	't'
		call	OUT_UART_p1
		movlw	'a'
		call	OUT_UART_p1
		movlw	'l'
		call	OUT_UART_p1
		movlw	' '
		call	OUT_UART_p1
		movlw	'P'
		call	OUT_UART_p1
		movlw	'a'
		call	OUT_UART_p1
		movlw	'n'
		call	OUT_UART_p1
		movlw	' '
		call	OUT_UART_p1
		movlw	'H'
		call	OUT_UART_p1
		movlw	'e'
		call	OUT_UART_p1
		movlw	'a'
		call	OUT_UART_p1
		movlw	'd'
		call	OUT_UART_p1
		movlw	' '
		call	OUT_UART_p1
s_version_txt
		movlw	'V'
		call	OUT_UART_p1
		movlw	'e'
		call	OUT_UART_p1
		movlw	'r'
		call	OUT_UART_p1
		movlw	'.'
		call	OUT_UART_p1
		movlw	VERSION_H
		call	OUT_UART_p1
		movlw	'.'
		call	OUT_UART_p1
		movlw	VERSION_M
		call	OUT_UART_p1
		movlw	'.'
		call	OUT_UART_p1
		movlw	VERSION_L
		call	OUT_UART_p1
prompt_txt
		movlw	0x0d
		call	OUT_UART_p1
		movlw	0x0a
		call	OUT_UART_p1
		movlw	'>'
		call	OUT_UART_p1
		return
;
must_lock_keypad_txt
		movlw	'M'
		call	OUT_UART_p1
		movlw	'u'
		call	OUT_UART_p1
		movlw	's'
		call	OUT_UART_p1
		movlw	't'
		call	OUT_UART_p1
		movlw	' '
		call	OUT_UART_p1
		movlw	'f'
		call	OUT_UART_p1
		movlw	'i'
		call	OUT_UART_p1
		movlw	'r'
		call	OUT_UART_p1
		movlw	's'
		call	OUT_UART_p1
		movlw	't'
		call	OUT_UART_p1
		movlw	' '
		call	OUT_UART_p1
		movlw	'L'
		call	OUT_UART_p1
		movlw	'O'
		call	OUT_UART_p1
		movlw	'C'
		call	OUT_UART_p1
		movlw	'K'
		call	OUT_UART_p1
		movlw	' '
		call	OUT_UART_p1
		movlw	'k'
		call	OUT_UART_p1
		movlw	'e'
		call	OUT_UART_p1
		movlw	'y'
		call	OUT_UART_p1
		movlw	'p'
		call	OUT_UART_p1
		movlw	'a'
		call	OUT_UART_p1
		movlw	'd'
		call	OUT_UART_p1
		movlw	0x0d
		call	OUT_UART_p1
		movlw	0x0a
		call	OUT_UART_p1
		return
;
;		*****************************************************
;		*****************************************************
;		******           CODE-SPACE PAGE TWO          ******
;		*****************************************************
;		*****************************************************
;
		ORG		0x1000
		;
		; ***************************************
		; *  Text Prompt String Setup - page 2  *
		; ***************************************
set_state_text_p2
	call	clear_display
		movlw	HIGH state_prompt_table	; set up PC high bits
		movwf	PCLATH
		movf	sys_state,w				; get current state
		andlw	0x0F
		addwf	PCL,f
state_prompt_table
		goto	set_run_state_txt			; 0x00
		goto	set_prog_state_txt			; 0x01
		goto	set_elev_start_state_txt	; 0x02
		goto	set_elev_step_state_txt		; 0x03
		goto	set_elev_stepct_state_txt	; 0x04
		goto	set_pan_start_state_txt		; 0x05
		goto	set_pan_step_state_txt		; 0x06
		goto	set_pan_stepct_state_txt	; 0x07
		goto	set_settle_state_txt		; 0x08
		goto	set_hold_state_txt			; 0x09
		goto	set_get_state_txt			; 0x0A
		goto	set_save_state_txt			; 0x0B
		goto	set_confirm_state_txt		; 0x0C
		goto	set_pan_final_state_txt		; 0x0D
		goto	set_pan_time_state_txt		; 0x0E
		goto	set_error_state_txt			; 0x0F
		;
		;
		; **********************************
		; *  System Prompt String Setup    *
		; **********************************
set_sys_state_text_p2
		call	clear_display			
		movlw	HIGH sys_prompt_table	; set up PC high bits
		movwf	PCLATH			
		movf	sys_state,w				; get current state
		andlw	0x0F
		addwf	PCL,f
sys_prompt_table
;		goto	set_sys_prog_state_txt		; 0x00		; " Program Sys Params "
;		goto	set_e_step_mode_state_txt	; 0x01		; "Elev Step mode=     "	e_m_control
;		goto	set_e_slew_state_txt		; 0x02		; "Elev Slew rate=     "	elev_slew_rate
;		goto	set_e_up_limit_state_txt	; 0x03		; "Elev  UP limit=     "	up_limit[h:l]
;		goto	set_e_dn_limit_state_txt	; 0x04		; "Elev  DN limit=     "	dn_limit[h:l]
;		goto	set_p_step_mode_state_txt	; 0x05		; " Pan Step mode=     "	p_m_control
;		goto	set_p_slew_state_txt		; 0x06		; " Pan Slew rate=     "	pan_slew_rate
;		goto	set_p_cw_limit_state_txt	; 0x07		; " Pan CCW limit=     "	ccw_limit[h:l]
;		goto	set_p_ccw_limit_state_txt	; 0x08		; " Pan  CW limit=     "	cw_limit[h:l]
;		goto	set_settle_state_txt		; 0x09		; " Settling Time=     "	settle_time
;		goto	set_shutter_t1_state_txt	; 0x0A		; "Shutter 1 Time=     "	shutter_T1
;		goto	set_shutter_t2_state_txt	; 0x0B		; "Shutter 2 Time=     "	shutter_T2
;		goto	set_save_sys_state_txt		; 0x0C		; " Save System Params "
;		goto	set_error_state_txt			; 0x0D
;		goto	set_error_state_txt			; 0x0E
;		goto	set_error_state_txt			; 0x0F
		;
		;
		goto	set_sys_prog_state_txt		; 0x00		; " Program Sys Params "
											;
		goto	set_shutter_t1_state_txt	; 0x01		; "Shutter 1 Time=     "	shutter_T1
		goto	set_shutter_t2_state_txt	; 0x02		; "Shutter 2 Time=     "	shutter_T2
											;
		goto	set_settle_state_txt		; 0x03		; " Settling Time=     "	settle_time
											;
		goto	set_p_fast_rate_txt			; 0x04		; " Pan Fast Rate=     "	pan_fast_rate
		goto	set_p_slow_rate_txt			; 0x05		; " Pan Slow Rate=     "	pan_slow_rate
		goto	set_p_ramp_dist_txt			; 0x06		; " Pan Ramp Dist=     "	pan_ramp_dist
											;
		goto	set_e_fast_rate_txt			; 0x07		; "Elev Fast Rate=     "	elev_slew_rate
		goto	set_e_slow_rate_txt			; 0x08		; "Elev Slow Rate=     "	pan_slow_rate
		goto	set_e_ramp_dist_txt			; 0x09		; "Elev Ramp Dist=     "	pan_ramp_dist
											;
		goto	set_save_sys_state_txt		; 0x0A		; " Save System Params "
											;
		goto	set_error_state_txt			; 0x0B
		goto	set_error_state_txt			; 0x0C
		goto	set_error_state_txt			; 0x0D
		goto	set_error_state_txt			; 0x0E
		goto	set_error_state_txt			; 0x0F
; 0x00
		; " Program Sys Params " 
		; " ---  ---  ---  YES "
; 0x01
		; "Elev Step mode=     "	m_control,7
		; "Prev HALF FULL ENTER" 
; 0x02
		; "Elev Slew rate=     "	elev_slew_rate
		; "Prev  !dn  up^ ENTER"
; 0x03
		; "Elev  UP limit=     "	up_limit[h:l]
		; "Prev  !dn  up^ ENTER"
; 0x04
		; "Elev  DN limit=     "	dn_limit[h:l]
		; "Prev  !dn  up^ ENTER"
; 0x05
		; " Pan Step mode=     "	m_control,6
		; "Prev HALF FULL ENTER"
; 0x06
		; " Pan Slew rate=     "	pan_slew_rate
		; "Prev  !dn  up^ ENTER"
; 0x07
		; " Pan CCW limit=     "	ccw_limit[h:l]
		; "Prev  <--  --> ENTER"
; 0x08
		; " Pan  CW limit=     "	cw_limit[h:l]
		; "Prev  <--  --> ENTER"
; 0x09
		; " Settling Time=     "	settle_time
		; "Prev  !dn  up^ ENTER"
; 0x0A
		; "Shutter 1 Time=     "	shutter_T1
		; "Prev  !dn  up^ ENTER"
; 0x0B
		; "Shutter 2 Time=     "	shutter_T2
		; "Prev  !dn  up^ ENTER"
; 0x0C
		; " Save System Params "
		; "Prev  ---  NO   YES "
		;
		;
		; ********************************
		; *  table of prompt strings  *
		; ********************************
		;
		;	RUN state
set_run_state_txt
		call	run_prog_txt		; "  Run Current Prog  "
									;
		call	get_run_button_txt	; "Prog  Get  Man  Run "
									;
		goto	set_text_exit
		;
		;	PROG state
set_prog_state_txt
		call	prog_entry_txt		; " Program Entry Mode "
									;
;		call	prog_button_txt	    ; "EXIT  Pan  1D    2D "
		call	exit_txt	   		; "EXIT "
		call	pan1d2d_txt	    	;      " Pan  1D    2D "
									;
		goto	set_text_exit
		;
		;	ELEV_START state
set_elev_start_state_txt
;		call	elev_start_txt		; "Elev Start Pos=     "
		call	elev_txt			; "Elev "
		call	start_pos_txt		;      "Start Pos"
		call	equal_txt			;               "=     "
									;
		call	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
		goto	set_text_exit
		;
		;	ELEV_STEP state
set_elev_step_state_txt
;		call	elev_step_txt		; "Elev Step Size=     "
		call	elev_txt			; "Elev "
		call	step_size_txt		;      "Step Size"
		call	equal_txt			;               "=     "
									;
		call	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
		goto	set_text_exit
		;
		;	ELEV_STEPCT state
set_elev_stepct_state_txt
;		call	elev_stepct_txt		; "Elev #of Steps=     "
		call	elev_txt			; "Elev "
		call	equal_txt			;               "=     "
		call	stepct_txt			;      "# of Steps="
									;
		call	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
		goto	set_text_exit
		;
		;	PAN_START state
set_pan_start_state_txt
;		call	pan_start_txt		; " Pan Start Pos=     "
		call	pan_txt				; " Pan "
		call	start_pos_txt		;      "Start Pos"
		call	equal_txt			;               "=     "
									;
;		call	pan_button_txt		; "EXIT  <--  --> ENTER"
		call	exit_txt	   		; "EXIT "
		call	left_right_txt		;      " <--  --> "
		call	enter_txt			;                "ENTER"
									;
		goto	set_text_exit
		;
		;	PAN_STEP state
set_pan_step_state_txt
;		call	pan_step_txt		; " Pan Step Size=     "
		call	pan_txt				; " Pan "
		call	step_size_txt		;      "Step Size"
		call	equal_txt			;               "=     "
									;
		call	pan_button_txt		; "Prev  <--  --> ENTER"
		goto	set_text_exit
		;
		;	PAN_STEPCT state
set_pan_stepct_state_txt
;		call	pan_stepct_txt		; " Pan #of Steps=     "
		call	pan_txt				; " Pan "
		call	equal_txt			;               "=     "
		call	stepct_txt			;      "# of Steps="
									;
;		call	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
		call	pan_button_txt		; "Prev  <--  --> ENTER"
		goto	set_text_exit
		;
		;	HOLD state
set_hold_state_txt
;		call	holding_txt			; "  Holding Time=     "
		call	holding_txt			; "  Holding "
		call	time_txt			; "         "Time"
		call	equal_txt			;               "=     "
									;
		call	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
		goto	set_text_exit
		;
		;	GET state
set_get_state_txt
;		call	get_prog_txt		; "      Get Prog#     "
		call	get_txt				; "      Get "
;		call	equal_txt			;               "=     "
;		call	prog_txt			;           "Prog#"
									;
;		call	load_button_txt		; "EXIT  !dn  up^  LOAD"
		call	exit_txt	   		; "EXIT "
		call	dn_up_txt			;      " !dn  up^ "
		call	load_txt			;                " LOAD"
									;
		goto	set_text_exit
		;
		;	SAVE state
set_save_state_txt
		call	save_as_txt			; "  Save as Prog#     "
									;
;		call	save_button_txt		; "EXIT  !dn  up^  SAVE"
		call	exit_txt	   		; "EXIT "
		call	dn_up_txt			;      " !dn  up^ "
		call	save_txt			;                " SAVE"
									;
		goto	set_text_exit
		;
		;	CONFIRM state
set_confirm_state_txt
		call	replace_txt			; "Replace Current Prog"
		call	yes_no_button_txt	; "Prev  ---  NO   YES "
		goto	set_text_exit
		;
		;	PAN_FINAL state
set_pan_final_state_txt
;		call	pan_final_txt		; " Pan Swing Ang=     "
		call	pan_txt				; " Pan "
		call	swing_txt			;      "Swing Ang"
		call	equal_txt			;               "=     "
									;
		call	pan_button_txt		; "Prev  <--  --> ENTER"
		goto	set_text_exit
		;
		;	PAN_TIME state
set_pan_time_state_txt
;		call	pan_time_txt		; "Total Pan Time=     "
		call	total_pan_txt		; "Total Pan "
		call	time_txt			; "         "Time"
		call	equal_txt			;               "=     "
									;
		call	up_dn_button_txt	; "Prev  !dn  up^ ENTER"								;
		goto	set_text_exit
		;
		;	ERROR state
set_error_state_txt
		call	error_txt			; "  EXIT ERROR STATE  "
		call	yes_button_txt		; " ---  ---  ---  YES "
;
set_text_exit
		movlw	HIGH main	; set up PC high bits
		movwf	PCLATH	
		return
		;
		; ********************************
		; *  button-prompt consolidation *
		; ********************************
up_dn_button_txt					; "Prev  !dn  up^ ENTER"
		call	prev_txt			; "Prev "
		call	dn_up_txt			;      " !dn  up^ "
		goto	enter_txt			;                "ENTER"
		;
pan_button_txt						; "Prev  <--  --> ENTER"
		call	prev_txt			; "Prev "
		call	left_right_txt		;      " <--  --> "
		goto	enter_txt			;                "ENTER"
		;
		; ******************************
		; * System parameter menu text *
		; ******************************
; 0x00
		; " Program Sys Params " 
		; " ---  ---  ---  YES "
; 0x01
		; "Shutter 1 Time=     "
		; "Prev  !dn  up^ ENTER"
; 0x02
		; "Shutter 2 Time=     "
		; "Prev  !dn  up^ ENTER"
; 0x03
		; " Settling Time=     "
		; "Prev  !dn  up^ ENTER"
; 0x04
		; " Pan Fast Rate=     "
		; "Prev  !dn  up^ ENTER"
; 0x05
		; " Pan Slow Rate=     "
		; "Prev  !dn  up^ ENTER"
; 0x06
		; " Pan Ramp Dist=     "
		; "Prev  !dn  up^ ENTER"
; 0x07
		; "Elev Fast Rate=     "
		; "Prev  !dn  up^ ENTER"
; 0x08
		; "Elev Slow Rate=     "
		; "Prev  !dn  up^ ENTER"
; 0x09
		; "Elev Ramp Dist=     "
		; "Prev  !dn  up^ ENTER"
; 0x0A
		; " Save System Params "
		; "Prev  ---  NO   YES "

set_sys_prog_state_txt		; 0x00
		call	sys_prog_txt		; " Program Sys Params " 
		call	yes_button_txt		; " ---  ---  --- YES "
		goto	set_text_exit
;
set_shutter_t1_state_txt	; 0x01
									; "Shutter 1 Time=     "
		call	shutter1_txt		; "Shutter 1 "
		call	time_txt			; "         "Time"
		call	equal_txt			;               "=     "
									;
		call	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
		goto	set_text_exit
;
set_shutter_t2_state_txt	; 0x02
									; "Shutter 2 Time=     "
		call	shutter1_txt		; "Shutter 1 "
	IF	DISP24
		movlw	0x8A
	ELSE
		movlw	0x88
	ENDIF
		call	SEND_CMD_p2
		movlw	'2'
		call	OUTCH_p2			;         "2"
		call	time_txt			; "         "Time"
		call	equal_txt			;               "=     "
									;
		call	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
		goto	set_text_exit
;
set_settle_state_txt		; 0x03
;		call	settle_txt			; " Settling Time=     "
		call	settle_txt			; " Settling "
		call	time_txt			; "         "Time"
		call	equal_txt			;               "=     "
									;
		call	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
		goto	set_text_exit
;
set_p_fast_rate_txt			; 0x04
;									; " Pan Slew rate=     "
		call	pan_txt				; " Pan "
		call	fast_rate_txt		;      "Fast Rate"
		call	equal_txt			;               "=     "
		call	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
		goto	set_text_exit
;
set_p_slow_rate_txt			; 0x05
;									; " Pan Slow Rate=     "
		call	pan_txt				; " Pan "
		call	slow_rate_txt		;      "Slew rate"
		call	equal_txt			;               "=     "
		call	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
		goto	set_text_exit
;
set_p_ramp_dist_txt			; 0x06
;									; " Pan Slow Rate=     "
		call	pan_txt				; " Pan "
		call	ramp_dist_txt		;      "Ramp dist"
		call	equal_txt			;               "=     "
		call	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
		goto	set_text_exit
;
set_e_fast_rate_txt			; 0x07
;									; "Elev Slew rate=     "
		call	elev_txt			; "Elev "
		call	fast_rate_txt		;      "Fast Rate"
		call	equal_txt			;               "=     "
		call	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
		goto	set_text_exit
;
set_e_slow_rate_txt			; 0x08
;									; "Elev Slow Rate=     "
		call	elev_txt			; "Elev "
		call	slow_rate_txt		;      "Slow rate"
		call	equal_txt			;               "=     "
		call	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
		goto	set_text_exit
;
set_e_ramp_dist_txt			; 0x09
;									; "Elev Slow Rate=     "
		call	elev_txt			; "Elev "
		call	ramp_dist_txt		;      "Ramp dist"
		call	equal_txt			;               "=     "
		call	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
		goto	set_text_exit
;
; set_p_ccw_limit_state_txt	; 0x0A
;									; " Pan  CW limit=     "
;		call	pan_txt				; " Pan "
;		call	ccw_limit_txt		;      "CCW limit"
;		call	equal_txt			;               "=     "
;		call	pan_button_txt		; "Prev  <--  --> ENTER"
;		goto	set_text_exit
;
; set_p_cw_limit_state_txt	; 0x0B
;									; " Pan CCW limit=     "
;		call	pan_txt				; " Pan "
;		call	cw_limit_txt		;      " CW limit"
;		call	equal_txt			;               "=     "
;		call	pan_button_txt		; "Prev  <--  --> ENTER"
;		goto	set_text_exit
;
; set_e_up_limit_state_txt	; 0x0C
;									; "Elev  UP limit=     "
;		call	elev_txt			; "Elev "
;		call	up_limit_txt			;      " UP limit"
;		call	equal_txt			;               "=     "
;		call	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
;		goto	set_text_exit
;
; set_e_dn_limit_state_txt	; 0x0D
									; "Elev  DN limit=     "
;		call	elev_txt			; "Elev "
;		call	dn_limit_txt		;      " DN limit"
;		call	equal_txt			;               "=     "
;		call	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
;		goto	set_text_exit
;
set_save_sys_state_txt	; 0x0A
		call	save_sys_txt		; " Save System Params "
		call	yes_no_button_txt	; "Prev  ---  NO   YES "
		goto	set_text_exit
		; ******************************
		; *  State Label Text Strings  *
		; ******************************
		; 00 run_prog_txt	; "  Digital Pan Head  " *
		; 01 prog_entry_txt	; " Program Entry Mode " *
		; 02 elev_start_txt	; "Elev Start Pos=     "
		; 03 elev_step_txt	; "Elev Step Size=     "
		; 04 elev_final_txt	; "Elev #of Steps=     "
		; 05 pan_start_txt	; " Pan Start Pos=     "
		; 06 pan_step_txt	; " Pan Step Size=     "
		; 07 pan_final_txt	; " Pan #of Steps=     "
		; 08 settle_txt		; " Settling Time=     "
		; 09 holding_txt	; "  Holding Time=     "
		; 0A save_as_txt	; "  Save as Prog#     "
		; 0B replace_txt	; "Replace Current Prog"
		; 0C pan_final_txt	; " Pan Swing Ang=     "
		; 0D pan_time_txt	; "Total Pan Time=     "
		; 0E error_txt		; "  EXIT ERROR STATE  "
		;
		;
greeting_txt	; "  Digital Pan Head  "
		call	clear_display
		call	run_prog_txt	; "  Digital Pan Head  "
		goto	version_txt		; " R.H.L.   Ver x.x.x "
		;
run_prog_txt	; "  Digital Pan Head   "
		movlw	0x80
		call	SEND_CMD_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'D'
		call	OUTCH_p2
		movlw	'i'
		call	OUTCH_p2
		movlw	'g'
		call	OUTCH_p2
		movlw	'i'
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'l'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'P'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'H'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'd'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		return
		;
prog_entry_txt	; "  Program Entry Mode  "
		movlw	0x80
		call	SEND_CMD_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	' '
		call	OUTCH_p2
		movlw	'P'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	'g'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'm'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'E'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'y'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'M'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	'd'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		return
		;
elev_txt				; "Elev "
		movlw	0x80
		call	SEND_CMD_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	'E'
		call	OUTCH_p2
		movlw	'l'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		movlw	'v'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		return
		;
pan_txt				; " Pan "
		movlw	0x80
		call	SEND_CMD_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	' '
		call	OUTCH_p2
		movlw	'P'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		return
		;
start_pos_txt	; "Start Pos"
	IF	DISP24
		movlw	0x87
	ELSE
		movlw	0x85
	ENDIF
		call	SEND_CMD_p2
		movlw	'S'
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'P'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	's'
		call	OUTCH_p2
		return
		;
step_size_txt	; "Step Size"
	IF	DISP24
		movlw	0x87
	ELSE
		movlw	0x85
	ENDIF
		call	SEND_CMD_p2
		movlw	'S'
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		movlw	'p'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'S'
		call	OUTCH_p2
		movlw	'i'
		call	OUTCH_p2
		movlw	'z'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		return
		;
stepct_txt		; "# of Steps="
	IF	DISP24
		movlw	0x87
	ELSE
		movlw	0x85
	ENDIF
		call	SEND_CMD_p2
		movlw	'#'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	'f'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'S'
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		movlw	'p'
		call	OUTCH_p2
		movlw	's'
		call	OUTCH_p2
		movlw	'='
		call	OUTCH_p2
		return
			;
settle_txt	; " Settling "
		movlw	0x80
		call	SEND_CMD_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	' '
		call	OUTCH_p2
		movlw	'S'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		movlw	'l'
		call	OUTCH_p2
		movlw	'i'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	'g'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		return
			;
holding_txt	; "  Holding "
		movlw	0x80
		call	SEND_CMD_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'H'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	'l'
		call	OUTCH_p2
		movlw	'd'
		call	OUTCH_p2
		movlw	'i'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	'g'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		return
			;
time_txt	; "Time"
	IF	DISP24
		movlw	0x8C
	ELSE
		movlw	0x8A
	ENDIF
		call	SEND_CMD_p2
		movlw	'T'
		call	OUTCH_p2
		movlw	'i'
		call	OUTCH_p2
		movlw	'm'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		return
			;
equal_txt	; "=     "
	IF	DISP24
		movlw	0x90
	ELSE
		movlw	0x8E
	ENDIF
		call	SEND_CMD_p2
		movlw	'='
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		return
		;
get_txt	; 	"      Get "
		movlw	0x80
		call	SEND_CMD_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'L'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'd'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
;		return
;		call	SEND_CMD_p2
		movlw	'P'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	'g'
		call	OUTCH_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	'#'
		call	OUTCH_p2
		return
				;
save_as_txt	; "  Save as "
	IF	DISP24
		movlw	0x80
	ELSE
		movlw	0x82
	ENDIF
		call	SEND_CMD_p2
;		movlw	' '
;		call	OUTCH_p2
;		movlw	' '
;		call	OUTCH_p2
		movlw	'S'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'v'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	's'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
;		return
				;
; prog_txt	; "Prog#"
;	IF	DISP24
;		movlw	0x88
;	ELSE
;		movlw	0x8A
;	ENDIF
;		call	SEND_CMD_p2
		movlw	'P'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	'g'
		call	OUTCH_p2
	IF	DISP24
		movlw	'r'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'm'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	'#'
		call	OUTCH_p2
		return
		;
swing_txt	; "Swing Ang=     "
	IF	DISP24
		movlw	0x87
	ELSE
		movlw	0x85
	ENDIF
		call	SEND_CMD_p2
		movlw	'S'
		call	OUTCH_p2
		movlw	'w'
		call	OUTCH_p2
		movlw	'i'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	'g'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'A'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	'g'
		call	OUTCH_p2
		return
				;
total_pan_txt	; "Total Pan"
		movlw	0x80
		call	SEND_CMD_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	'T'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'l'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'P'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		return
		;
replace_txt	; "Replace Current Program"
		movlw	0x80
		call	SEND_CMD_p2
		movlw	'R'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		movlw	'p'
		call	OUTCH_p2
		movlw	'l'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'c'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'C'
		call	OUTCH_p2
		movlw	'u'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'P'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	'g'
		call	OUTCH_p2
	IF	DISP24
		movlw	'r'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'm'
		call	OUTCH_p2
	ENDIF
		return
				;
save_sys_txt	; " Save System Params "
		movlw	0x80
		call	SEND_CMD_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'S'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'v'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'S'
		call	OUTCH_p2
		movlw	'y'
		call	OUTCH_p2
		movlw	's'
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		movlw	'm'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'P'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'm'
		call	OUTCH_p2
		movlw	's'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		return
				;
shutter1_txt	; "Shutter 1 "
		movlw	0x80
		call	SEND_CMD_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	'S'
		call	OUTCH_p2
		movlw	'h'
		call	OUTCH_p2
		movlw	'u'
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'1'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		return
				;
ramp_dist_txt	; "Ramp Dist"
	IF	DISP24
		movlw	0x87
	ELSE
		movlw	0x85
	ENDIF
		call	SEND_CMD_p2
		movlw	'R'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'm'
		call	OUTCH_p2
		movlw	'p'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'D'
		call	OUTCH_p2
		movlw	'i'
		call	OUTCH_p2
		movlw	's'
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		return
				;
fast_rate_txt	; "Fast rate"
	IF	DISP24
		movlw	0x87
	ELSE
		movlw	0x85
	ENDIF
		call	SEND_CMD_p2
		movlw	'F'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	's'
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		movlw	' '
		goto	rate_txt
				;
slow_rate_txt	; "Slow rate"
	IF	DISP24
		movlw	0x87
	ELSE
		movlw	0x85
	ENDIF
		call	SEND_CMD_p2
		movlw	'S'
		call	OUTCH_p2
		movlw	'l'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	'w'
		call	OUTCH_p2
		movlw	' '
rate_txt
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		return
		;
up_limit_txt	; " UP limit"
		movlw	0x85
		call	SEND_CMD_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'U'
		call	OUTCH_p2
		movlw	'P'
		call	OUTCH_p2
		goto	limit_txt
		;
dn_limit_txt	; " DN limit"
		movlw	0x85
		call	SEND_CMD_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'D'
		call	OUTCH_p2
		movlw	'N'
		call	OUTCH_p2
		goto	limit_txt
		;
ccw_limit_txt	; "CCW limit"
		movlw	0x85
		call	SEND_CMD_p2
		movlw	'C'
		call	OUTCH_p2
		movlw	'C'
		call	OUTCH_p2
		movlw	'W'
		call	OUTCH_p2
		goto	limit_txt
		;
cw_limit_txt	; " CW limit"
		movlw	0x85
		call	SEND_CMD_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'C'
		call	OUTCH_p2
		movlw	'W'
		call	OUTCH_p2
limit_txt
		movlw	' '
		call	OUTCH_p2
		movlw	'l'
		call	OUTCH_p2
		movlw	'i'
		call	OUTCH_p2
		movlw	'm'
		call	OUTCH_p2
		movlw	'i'
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		return
		; 
error_txt		; "  EXIT ERROR STATE  "
		movlw	0x80
		call	SEND_CMD_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'E'
		call	OUTCH_p2
		movlw	'X'
		call	OUTCH_p2
		movlw	'I'
		call	OUTCH_p2
		movlw	'T'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'E'
		call	OUTCH_p2
		movlw	'R'
		call	OUTCH_p2
		movlw	'R'
		call	OUTCH_p2
		movlw	'O'
		call	OUTCH_p2
		movlw	'R'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'S'
		call	OUTCH_p2
		movlw	'T'
		call	OUTCH_p2
		movlw	'A'
		call	OUTCH_p2
		movlw	'T'
		call	OUTCH_p2
		movlw	'E'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		return
		;
		; ********************************
		; *  soft-button prompt strings  *
		; ********************************
		;
		;	get_run_button_txt	; "Prog  Get  Man  Run "
		;	prog_button_txt	    ; "EXIT  Pan  1D    2D "
		;	pan_button_txt		; "Prev  <--  --> ENTER"
		;	up_dn_button_txt	; "Prev  !dn  up^ ENTER"
		;	save_button_txt		; "EXIT  !dn  up^  SAVE"
		;	load_button_txt		; "Prev  !dn  up^  LOAD"
		;	yes_no_button_txt	; "Prev  ---  NO   YES "
		;	yes_button_txt		; " ---  ---  ---  YES "
		;
get_run_button_txt	
	IF	DISP24
						; "Prog  Get  Man  Run "
						; "Prog  Load    Step   Run "
		movlw	0xC0
		call	SEND_CMD_p2
		movlw	'P'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	'g'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'L'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'd'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'S'
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		movlw	'p'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'R'
		call	OUTCH_p2
		movlw	'u'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		return
		;
	ELSE
							; "Prog  Get  Man  Run "
		movlw	0xC0
		call	SEND_CMD_p2
		movlw	'P'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	'g'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'G'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'M'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'R'
		call	OUTCH_p2
		movlw	'u'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		return
		;
	ENDIF

pan1d2d_txt
	IF	DISP24
						; " Pan  1D  2D     "
						; "SloPan 1-axis 2axis"

		movlw	0xC5
		call	SEND_CMD_p2
		movlw	'S'
		call	OUTCH_p2
		movlw	'l'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	'P'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'1'
		call	OUTCH_p2
		movlw	'-'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'x'
		call	OUTCH_p2
		movlw	'i'
		call	OUTCH_p2
		movlw	's'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'2'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'x'
		call	OUTCH_p2
		movlw	'i'
		call	OUTCH_p2
		movlw	's'
		call	OUTCH_p2
		return
	ELSE
						; " Pan  1D  2D "
		movlw	0xC5
		call	SEND_CMD_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'P'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'1'
		call	OUTCH_p2
		movlw	'D'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'2'
		call	OUTCH_p2
		movlw	'D'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		return
	ENDIF
		;
prev_txt	; "Prev "
		movlw	0xC0
		call	SEND_CMD_p2
prev2_txt
		movlw	'P'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		movlw	'v'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		return
				;
exit_txt	; "EXIT "
		movlw	0xC0
		call	SEND_CMD_p2
exit2_txt
		movlw	'E'
		call	OUTCH_p2
		movlw	'X'
		call	OUTCH_p2
		movlw	'I'
		call	OUTCH_p2
		movlw	'T'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		return
			;
left_right_txt	; " <--  --> "
		movlw	0xC5
		call	SEND_CMD_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	' '
		call	OUTCH_p2
		movlw	'<'
		call	OUTCH_p2
		movlw	'-'
		call	OUTCH_p2
		movlw	'-'
		call	OUTCH_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'-'
		call	OUTCH_p2
		movlw	'-'
		call	OUTCH_p2
		movlw	'>'
		call	OUTCH_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	' '
		return
				;
dn_up_txt	; " !dn  up^ "
		movlw	0xC5
		call	SEND_CMD_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	' '
		call	OUTCH_p2
		movlw	0x02	; down arrow
		call	OUTCH_p2
		movlw	'd'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	' '
		call	OUTCH_p2
		movlw	'u'
		call	OUTCH_p2
		movlw	'p'
		call	OUTCH_p2
		movlw	0x01	; up arrow
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
	ENDIF
		return
				;
enter_txt	; "ENTER"
	IF	DISP24
		movlw	0xD3
	ELSE
		movlw	0xCF
	ENDIF
		call	SEND_CMD_p2
		movlw	'E'
		call	OUTCH_p2
		movlw	'N'
		call	OUTCH_p2
		movlw	'T'
		call	OUTCH_p2
		movlw	'E'
		call	OUTCH_p2
		movlw	'R'
		call	OUTCH_p2
		return
				;
save_txt	; " SAVE"
	IF	DISP24
		movlw	0xD3
	ELSE
		movlw	0xCF
	ENDIF
		call	SEND_CMD_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'S'
		call	OUTCH_p2
		movlw	'A'
		call	OUTCH_p2
		movlw	'V'
		call	OUTCH_p2
		movlw	'E'
		call	OUTCH_p2
		return
				;
load_txt	; " LOAD"
	IF	DISP24
		movlw	0xD3
	ELSE
		movlw	0xCF
	ENDIF
		call	SEND_CMD_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'L'
		call	OUTCH_p2
		movlw	'O'
		call	OUTCH_p2
		movlw	'A'
		call	OUTCH_p2
		movlw	'D'
		call	OUTCH_p2
		return
		;
yes_txt	; " YES "
	IF	DISP24
		movlw	0xD3
	ELSE
		movlw	0xCF
	ENDIF
		call	SEND_CMD_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'Y'
		call	OUTCH_p2
		movlw	'E'
		call	OUTCH_p2
		movlw	'S'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		return
		;
		;
yes_no_button_txt	; "Prev  ---   NO YES "
		movlw	0xC0
		call	SEND_CMD_p2
		movlw	'P'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		movlw	'v'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	' '
		call	OUTCH_p2
		movlw	'-'
		call	OUTCH_p2
		movlw	'-'
		call	OUTCH_p2
		movlw	'-'
		call	OUTCH_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'N'
		call	OUTCH_p2
		movlw	'O'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	' '
		call	OUTCH_p2
		movlw	'Y'
		call	OUTCH_p2
		movlw	'E'
		call	OUTCH_p2
		movlw	'S'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		return
		;
yes_button_txt	; " ---  ---  --- YES "
		movlw	0xC0
		call	SEND_CMD_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'-'
		call	OUTCH_p2
		movlw	'-'
		call	OUTCH_p2
		movlw	'-'
		call	OUTCH_p2
		movlw	' '
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
	ENDIF
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'-'
		call	OUTCH_p2
		movlw	'-'
		call	OUTCH_p2
		movlw	'-'
		call	OUTCH_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'-'
		call	OUTCH_p2
		movlw	'-'
		call	OUTCH_p2
		movlw	'-'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	' '
		call	OUTCH_p2
		movlw	'Y'
		call	OUTCH_p2
		movlw	'E'
		call	OUTCH_p2
		movlw	'S'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		return
		;
		; ***************************
		; * Additional text strings *
		; ***************************

		;
version_txt	; " R.H.L.  Ver x.x.x "
		movlw	0xC0
		call	SEND_CMD_p2
		movlw	' '
		call	OUTCH_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	'R'
		call	OUTCH_p2
		movlw	'.'
		call	OUTCH_p2
		movlw	'H'
		call	OUTCH_p2
		movlw	'.'
		call	OUTCH_p2
		movlw	'L'
		call	OUTCH_p2
		movlw	'.'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'V'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	VERSION_H
		call	OUTCH_p2
		movlw	'.'
		call	OUTCH_p2
		movlw	VERSION_M
		call	OUTCH_p2
		movlw	'.'
		call	OUTCH_p2
		movlw	VERSION_L
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		return
		;
		;
		;	character generator programming
make_up_dn_arrow
		movlw	0x48
		call	SEND_CMD_p2
		movlw	B'00000100'
		call	OUTCH_p2
		movlw	B'00000100'
		call	OUTCH_p2
		movlw	B'00001110'
		call	OUTCH_p2
		movlw	B'00010101'
		call	OUTCH_p2
		movlw	B'00000100'
		call	OUTCH_p2
		movlw	B'00000100'
		call	OUTCH_p2
		movlw	B'00000100'
		call	OUTCH_p2
		movlw	B'00000100'
		call	OUTCH_p2
;		return
		;
; make_down_arrow
		movlw	0x50
		call	SEND_CMD_p2
		movlw	B'00000100'
		call	OUTCH_p2
		movlw	B'00000100'
		call	OUTCH_p2
		movlw	B'00000100'
		call	OUTCH_p2
		movlw	B'00000100'
		call	OUTCH_p2
		movlw	B'00010101'
		call	OUTCH_p2
		movlw	B'00001110'
		call	OUTCH_p2
		movlw	B'00000100'
		call	OUTCH_p2
		movlw	B'00000100'
		call	OUTCH_p2
		return
		;
saving_prog_txt	; "   Saving Program   "
		movlw	0xC0
		call	SEND_CMD_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'S'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'v'
		call	OUTCH_p2
		movlw	'i'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	'g'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'P'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	'g'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'm'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		return
		;
running_program_txt		; "Running Current Prog"
		movlw	0x80
		call	SEND_CMD_p2
		movlw	'R'
		call	OUTCH_p2
		movlw	'u'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	'i'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	'g'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'C'
		call	OUTCH_p2
		movlw	'u'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'P'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	'g'
		call	OUTCH_p2
	IF	DISP24
		movlw	'r'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'm'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		return
		;
		;
stepping_program_txt	; " Stepping Thru Prog "
		movlw	0x80
		call	SEND_CMD_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'S'
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		movlw	'p'
		call	OUTCH_p2
		movlw	'p'
		call	OUTCH_p2
		movlw	'i'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	'g'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'T'
		call	OUTCH_p2
		movlw	'h'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'u'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'P'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	'g'
		call	OUTCH_p2
	IF	DISP24
		movlw	'r'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'm'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	' '
		call	OUTCH_p2
		return
		;
		;
no_program_txt	; "No Program      EXIT"
		movlw	0xC0
		call	SEND_CMD_p2
		movlw	'N'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'P'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	'g'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'm'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'E'
		call	OUTCH_p2
		movlw	'X'
		call	OUTCH_p2
		movlw	'I'
		call	OUTCH_p2
		movlw	'T'
		call	OUTCH_p2
		return
		;
		;
run_2d_step_txt	; "HALT Elev=   Pan=        "
		movlw	0xC0
		call	SEND_CMD_p2
		movlw	'H'			; C0
		call	OUTCH_p2
		movlw	'A'			; C1
		call	OUTCH_p2
		movlw	'L'			; C2
		call	OUTCH_p2
		movlw	'T'			; C3
		call	OUTCH_p2
		movlw	' '			; C4
		call	OUTCH_p2
	IF	DISP24
		movlw	'E'			; C5
		call	OUTCH_p2
		movlw	'l'			; C6
		call	OUTCH_p2
		movlw	'e'			; C7
		call	OUTCH_p2
		movlw	'v'			; C8
		call	OUTCH_p2
		movlw	'='			; C9
		call	OUTCH_p2
		movlw	' '			; CA
		call	OUTCH_p2
		movlw	' '			; CB
		call	OUTCH_p2
		movlw	' '			; CC
		call	OUTCH_p2
		movlw	'P'			; CD
		call	OUTCH_p2
		movlw	'a'			; CE
		call	OUTCH_p2
		movlw	'n'			; CF
		call	OUTCH_p2
		movlw	'='			; D0
		call	OUTCH_p2
		movlw	' '			; D1
		call	OUTCH_p2
		movlw	' '			; D2
		call	OUTCH_p2
	ELSE
		movlw	'E'			; C5
		call	OUTCH_p2
		movlw	'='			; C6
		call	OUTCH_p2
		movlw	' '			; C7
		call	OUTCH_p2
		movlw	' '			; C8
		call	OUTCH_p2
		movlw	' '			; C9
		call	OUTCH_p2
		movlw	'P'			; CA
		call	OUTCH_p2
		movlw	'='			; CB
		call	OUTCH_p2
		movlw	' '			; CC
		call	OUTCH_p2
		movlw	' '			; CD
		call	OUTCH_p2
		movlw	' '			; CE
		call	OUTCH_p2
	ENDIF
		movlw	' '			; D3 / CF
		call	OUTCH_p2
		movlw	' '			; D4 / D0
		call	OUTCH_p2
		movlw	' '			; D5 / D1
		call	OUTCH_p2
		movlw	' '			; D6 / D2
		call	OUTCH_p2
		movlw	' '			; D7 / D3
		call	OUTCH_p2
		return
		;
run_1d_step_txt	; "HALT     Pan=        "
		movlw	0xC0
		call	SEND_CMD_p2
		movlw	'H'			; C0
		call	OUTCH_p2
		movlw	'A'			; C1
		call	OUTCH_p2
		movlw	'L'			; C2
		call	OUTCH_p2
		movlw	'T'			; C3
		call	OUTCH_p2
		movlw	' '			; C4
		call	OUTCH_p2
	IF	DISP24
		movlw	' '			; C5
		call	OUTCH_p2
		movlw	' '			; C6
		call	OUTCH_p2
		movlw	' '			; C7
		call	OUTCH_p2
		movlw	' '			; C8
		call	OUTCH_p2
		movlw	' '			; C9
		call	OUTCH_p2
		movlw	' '			; CA
		call	OUTCH_p2
		movlw	' '			; CB
		call	OUTCH_p2
		movlw	' '			; CC
		call	OUTCH_p2
		movlw	'P'			; CD
		call	OUTCH_p2
		movlw	'a'			; CE
		call	OUTCH_p2
		movlw	'n'			; CF
		call	OUTCH_p2
		movlw	'='			; D0
		call	OUTCH_p2
		movlw	' '			; D1
		call	OUTCH_p2
		movlw	' '			; D2
		call	OUTCH_p2
	ELSE
		movlw	' '			; C5
		call	OUTCH_p2
		movlw	' '			; C6
		call	OUTCH_p2
		movlw	' '			; C7
		call	OUTCH_p2
		movlw	' '			; C8
		call	OUTCH_p2
		movlw	' '			; C9
		call	OUTCH_p2
		movlw	'P'			; CA
		call	OUTCH_p2
		movlw	'='			; CB
		call	OUTCH_p2
		movlw	' '			; CC
		call	OUTCH_p2
		movlw	' '			; CD
		call	OUTCH_p2
		movlw	' '			; CE
		call	OUTCH_p2
	ENDIF
		movlw	' '			; D3 / CF
		call	OUTCH_p2
		movlw	' '			; D4 / D0
		call	OUTCH_p2
		movlw	' '			; D5 / D1
		call	OUTCH_p2
		movlw	' '			; D6 / D2
		call	OUTCH_p2
		movlw	' '			; D7 / D3
		call	OUTCH_p2
		return
		;
slow_prog_run_initial_txt	; "  Slow Pan Program  "
		;					; "Moving to Start Pos."
		movlw	0x80
		call	SEND_CMD_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'S'
		call	OUTCH_p2
		movlw	'l'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	'w'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'P'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'P'		
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	'g'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'm'
		call	OUTCH_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		;
		movlw	0xC0
		call	SEND_CMD_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	'M'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	'v'
		call	OUTCH_p2
		movlw	'i'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	'g'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'S'
		call	OUTCH_p2
		movlw	't'		
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'P'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	's'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		;
		return

slow_prog_start_prompt_txt	; "HALT  Slow pan START"
		movlw	0xC0
		call	SEND_CMD_p2
							;
		movlw	'H'			; C0
		call	OUTCH_p2
		movlw	'A'			; C1
		call	OUTCH_p2
		movlw	'L'			; C2
		call	OUTCH_p2
		movlw	'T'			; C3
		call	OUTCH_p2
		movlw	' '			; C4
		call	OUTCH_p2
		movlw	' '			; C5
		call	OUTCH_p2
	IF	DISP24
		movlw	' '			; C6
		call	OUTCH_p2
		movlw	' '			; C7
		call	OUTCH_p2
		movlw	' '			; C8
		call	OUTCH_p2
		movlw	' '			; C9
		call	OUTCH_p2
	ENDIF
		movlw	'S'			; C6/CA
		call	OUTCH_p2
		movlw	'l'			; C7/CB
		call	OUTCH_p2
		movlw	'o'			; C8/CC
		call	OUTCH_p2
		movlw	'w'			; C9/CD
		call	OUTCH_p2
		movlw	' '			; CA/CE
		call	OUTCH_p2
		movlw	'P'			; CB/CF
		call	OUTCH_p2
		movlw	'a'			; CC/D0
		call	OUTCH_p2
		movlw	'n'			; CD/D1
		call	OUTCH_p2
		movlw	' '			; CE/D2
		call	OUTCH_p2
		movlw	'S'			; CF/D3
		call	OUTCH_p2
		movlw	'T'			; D0/D4
		call	OUTCH_p2
		movlw	'A'			; D1/D5
		call	OUTCH_p2
		movlw	'R'			; D2/D6
		call	OUTCH_p2
		movlw	'T'			; D3/D7
		call	OUTCH_p2
		return
		;
slow_prog_running_prompt_txt	; "Running"
		movlw	0xC0
		call	SEND_CMD_p2
		movlw	'H'			; C0
		call	OUTCH_p2
		movlw	'A'			; C1
		call	OUTCH_p2
		movlw	'L'			; C2
		call	OUTCH_p2
		movlw	'T'			; C3
		call	OUTCH_p2
		movlw	' '			; C4
		call	OUTCH_p2
	IF	DISP24
		movlw	' '			; C5
		call	OUTCH_p2
		movlw	' '			; C6	
		call	OUTCH_p2
	ENDIF	

		movlw	'P'			; C5/C7
		call	OUTCH_p2
		movlw	'a'			; C6/C8
		call	OUTCH_p2
		movlw	'n'			; C7/C9
		call	OUTCH_p2
		movlw	'='			; C8/CA
		call	OUTCH_p2
		movlw	' '			; C9/CB
		call	OUTCH_p2
		movlw	' '			; CA/CC
		call	OUTCH_p2
		movlw	' '			; CB/CD
		call	OUTCH_p2
		movlw	' '			; CC/CE
		call	OUTCH_p2
		movlw	' '			; CD/CF
		call	OUTCH_p2
	IF	DISP24
		movlw	' '			; CD/D0
		call	OUTCH_p2
		movlw	' '			; CD/D1	
		call	OUTCH_p2
	ENDIF
		movlw	' '			; CE/D2
		call	OUTCH_p2
		movlw	' '			; CF/D3
		call	OUTCH_p2
		movlw	' '			; D0/D4
		call	OUTCH_p2
		movlw	' '			; D1/D5
		call	OUTCH_p2
		movlw	' '			; D2/D6
		call	OUTCH_p2
		movlw	' '			; D3/D7
		call	OUTCH_p2
		;
		return
		;
; slow_prog_exit_prompt_txt	; "   EXIT"
prog_exit_prompt_txt	; "   EXIT"
	IF	DISP24
		movlw	0xD3
	ELSE
		movlw	0xCF
	ENDIF
		call	SEND_CMD_p2
		;
		movlw	' '
		call	OUTCH_p2
		movlw	'E'
		call	OUTCH_p2
		movlw	'X'
		call	OUTCH_p2
		movlw	'I'
		call	OUTCH_p2
		movlw	'T'
		call	OUTCH_p2
		return
		;
sys_prog_txt	; " Program Sys Params "                 
		movlw	0x80
		call	SEND_CMD_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'P'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	'g'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'm'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'S'
		call	OUTCH_p2
		movlw	'y'
		call	OUTCH_p2
		movlw	's'
		call	OUTCH_p2
	IF	DISP24
		movlw	't'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		movlw	'm'
		call	OUTCH_p2
	ENDIF
		movlw	' '
		call	OUTCH_p2
		movlw	'P'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'm'
		call	OUTCH_p2
		movlw	's'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		return
		;
do_halt_prompt_text		; "Halting Current Program "
						; "      Continue     ABORT"
		movlw	0x80
		call	SEND_CMD_p2
		movlw	'H'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'l'
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		movlw	'i'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	'g'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'C'
		call	OUTCH_p2
		movlw	'u'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'P'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	'g'
		call	OUTCH_p2
	IF	DISP24
		movlw	'r'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'm'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		;				; "      Continue     ABORT"
		movlw	0xC0
		call	SEND_CMD_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'C'
		call	OUTCH_p2
		movlw	'O'
		call	OUTCH_p2
		movlw	'N'
		call	OUTCH_p2
		movlw	'T'
		call	OUTCH_p2
		movlw	'I'
		call	OUTCH_p2
		movlw	'N'
		call	OUTCH_p2
		movlw	'U'
		call	OUTCH_p2
		movlw	'E'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	'A'
		call	OUTCH_p2
		movlw	'B'
		call	OUTCH_p2
		movlw	'O'
		call	OUTCH_p2
		movlw	'R'
		call	OUTCH_p2
		movlw	'T'
		call	OUTCH_p2
		return
		;
do_continue_text	; "   Continuing Program   "
		movlw	0x80
		call	SEND_CMD_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	' '
		call	OUTCH_p2
		movlw	'C'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	't'
		call	OUTCH_p2
		movlw	'i'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	'u'
		call	OUTCH_p2
		movlw	'i'
		call	OUTCH_p2
		movlw	'n'
		call	OUTCH_p2
		movlw	'g'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'P'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'o'
		call	OUTCH_p2
		movlw	'g'
		call	OUTCH_p2
		movlw	'r'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'm'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		return
				;
keypad_locked_txt_p2	; "  Keypad is LOCKED  "
		movlw	0xC0
		call	SEND_CMD_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'K'
		call	OUTCH_p2
		movlw	'e'
		call	OUTCH_p2
		movlw	'y'
		call	OUTCH_p2
		movlw	'p'
		call	OUTCH_p2
		movlw	'a'
		call	OUTCH_p2
		movlw	'd'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'i'
		call	OUTCH_p2
		movlw	's'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	'L'
		call	OUTCH_p2
		movlw	'O'
		call	OUTCH_p2
		movlw	'C'
		call	OUTCH_p2
		movlw	'K'
		call	OUTCH_p2
		movlw	'E'
		call	OUTCH_p2
		movlw	'D'
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	IF	DISP24
		movlw	' '
		call	OUTCH_p2
		movlw	' '
		call	OUTCH_p2
	ENDIF
		return
				;
				; ***************************************
				; ** Page 2 character output routines  **
				; ***************************************
OUTCH_p2
		movwf	CHAR			; save w
		movlw	HIGH OUTCH		; set up PC high bits
		movwf	PCLATH
		movf	CHAR,w			; get it back
		call	OUTCH
		movlw	HIGH OUTCH_p2	; restore PC high bits
		movwf	PCLATH
		return
				;
clear_display
		movlw	CLR_DISP		; display clear command
				;
SEND_CMD_p2
		movwf	CHAR			; save w
		movlw	HIGH SEND_CMD	; set up PC high bits
		movwf	PCLATH
		movf	CHAR,w			; get it back
		call	SEND_CMD
		movlw	HIGH SEND_CMD_p2	; restore PC high bits
		movwf	PCLATH
		return




		END                       ; directive 'end of program'
